@@ -6,7 +6,7 @@
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -17,8 +17,10 @@ int main(void)
    BITMAP *active_page;
    int c;
 
-   allegro_init();
+   if (allegro_init() != 0)
+      return 1;
    install_timer();
+   install_keyboard();
 
    /* Some platforms do page flipping by making one large screen that you
     * can then scroll, while others give you several smaller, unique
@@ -33,27 +35,32 @@ int main(void)
     * size, and if that doesn't work, for 320x200.
     */
    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 400) != 0) {
-      if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
-	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	 allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
-	 return 1;
+      if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) != 0) {
+	 if (set_gfx_mode(GFX_SAFE, 320, 200, 0, 0) != 0) {
+	    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	    allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
+	    return 1;
+         }
       }
    }
 
    set_palette(desktop_palette);
-   text_mode(-1);
 
    /* allocate the memory buffer */
    buffer = create_bitmap(SCREEN_W, SCREEN_H);
 
    /* first with a double buffer... */
+   clear_keybuf();
    c = retrace_count+32;
    while (retrace_count-c <= SCREEN_W+32) {
       clear_to_color(buffer, makecol(255, 255, 255));
       circlefill(buffer, retrace_count-c, SCREEN_H/2, 32, makecol(0, 0, 0));
-      textprintf(buffer, font, 0, 0, makecol(0, 0, 0),
-		 "Double buffered (%s)", gfx_driver->name);
+      textprintf_ex(buffer, font, 0, 0, makecol(0, 0, 0), -1,
+		    "Double buffered (%s)", gfx_driver->name);
       blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
+
+      if (keypressed())
+	 break;
    }
 
    destroy_bitmap(buffer);
@@ -71,17 +78,21 @@ int main(void)
    active_page = page2;
 
    /* do the animation using page flips... */
+   clear_keybuf();
    for (c=-32; c<=SCREEN_W+32; c++) {
       clear_to_color(active_page, makecol(255, 255, 255));
       circlefill(active_page, c, SCREEN_H/2, 32, makecol(0, 0, 0));
-      textprintf(active_page, font, 0, 0, makecol(0, 0, 0),
-		 "Page flipping (%s)", gfx_driver->name);
+      textprintf_ex(active_page, font, 0, 0, makecol(0, 0, 0), -1,
+		    "Page flipping (%s)", gfx_driver->name);
       show_video_bitmap(active_page);
 
       if (active_page == page1)
 	 active_page = page2;
       else
 	 active_page = page1;
+
+      if (keypressed())
+	 break;
    }
 
    destroy_bitmap(page1);
@@ -90,4 +101,4 @@ int main(void)
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
