@@ -14,73 +14,44 @@
 #  resulting archive will be datestamped. This is in particular useful
 #  for making SVN snapshots.
 
-export LC_ALL=C
 
-for tool in doxygen git cmake gcc unzip zip perl ; do
-	which $tool >/dev/null 2>&1 || \
-		{ echo "error: program $tool not found" ; exit 1; }
-done
-
-if [[ -z $1 ]]; then
-    VERSION=$(grep '^set(ALLEGRO_VERSION ' CMakeLists.txt | tr -cd '[0-9.]')
-else
-    VERSION=$1
+if [ $# -lt 1 -o $# -gt 2 ]; then
+   echo "Usage: zipup archive_name [previous_archive]" 1>&2
+   exit 1
 fi
-name=allegro-$VERSION
-tmpdir=/tmp/"$name"
-currdir="$PWD"
 
-git_branch=$(git branch 2>/dev/null| sed -n '/^\*/s/^\* //p')
-rm -rf "$tmpdir"
-git clone . "$tmpdir"
-cd "$tmpdir"
-git checkout "$git_branch"
-rm -rf .git
+if [ -n "$2" ] && [ ! -r ../"$2" ]; then
+   echo "Previous archive $2 not in parent directory, aborting"
+   exit 1
+fi
+
+
+# This used to be necessary so that fixdll.sh would sort consistently.
+# It probably isn't necessary any more.
+export LC_ALL=en_US
+
+
+# strip off the path and extension from our arguments
+name=$(echo "$1" | sed -e 's/.*[\\\/]//; s/\.zip//')
+prev=$(echo "$2" | sed -e 's/.*[\\\/]//; s/\.zip//')
+
+# make a datestamped archive if specified
+if test $name = datestamp; then
+  date=`date '+%Y%m%d'`
+  name=allegro_$date
+fi
+
+# make sure all the makefiles are in Unix text format
+# for file in makefile.*; do
+#    mv $file _tmpfile
+#    tr -d \\\r < _tmpfile > $file
+#    touch -r _tmpfile $file
+#    rm _tmpfile
+# done
 
 # make sure all shell scripts are executable
 find . -name '*.sh' -exec chmod +x {} ';'
 
-# convert documentation from the ._tx source files
-echo "Building artifacts and documentation..."
-(
-    set -e
-    mkdir -p .dist/build
-    cd .dist/build
-    rm -rf docs
-    cmake $tmpdir
-    make -j2 docs dat
-    umask 022
-    for x in build txt ; do
-        mkdir -p $tmpdir/docs/$x
-    done
-    for x in AUTHORS CHANGES THANKS readme.txt ; do
-        cp -v docs/$x $TOP
-    done
-    cp -v docs/build/* $tmpdir/docs/build
-    cp -v docs/txt/*.txt $tmpdir/docs/txt
-) || exit 1
-
-
-# generate docs for AllegroGL addon
-echo "Generating AllegroGL docs..."
-(cd addons/allegrogl/docs; doxygen >/dev/null)
-
-# create language.dat and keyboard.dat files
-export DAT="$tmpdir/.dist/build/tools/dat"
-misc/mkdata.sh || exit 1
-rm -rf "$tmpdir/.dist"
-
-# generate the tar.gz first as the files are in Unix format
-# This is missing the manifest file, but who cares?
-(
-cd ..
-tar zcf "$name".tar.gz "$name"
-tar cJf "$name".tar.xz "$name"
-mv "$name".tar.gz "$currdir"
-mv "$name".tar.xz "$currdir"
-)
-
-# now the DOS stuff...
 
 proc_filelist()
 {
@@ -117,8 +88,36 @@ utod()
 }
 
 
+# convert documentation from the ._tx source files
+echo "Converting documentation..."
+TOP=$PWD
+(
+    set -e
+    mkdir -p .dist/Build
+    cd .dist/Build
+    rm -rf docs
+    cmake $TOP
+    make -j2 docs
+    umask 022
+    mkdir -p $TOP/docs/{build,txt}
+    cp -v docs/{AUTHORS,CHANGES,THANKS,readme.txt} $TOP
+    cp -v docs/build/* $TOP/docs/build
+    cp -v docs/txt/*.txt $TOP/docs/txt
+) || exit 1
+
+
+# generate docs for AllegroGL addon
+echo "Generating AllegroGL docs..."
+(cd addons/allegrogl/docs; doxygen >/dev/null)
+
+
+# create language.dat and keyboard.dat files
+misc/mkdata.sh || exit 1
+
+
 # build the main zip archive
 echo "Creating $name.zip..."
+rm -f .dist/$name.zip
 mkdir -p .dist/allegro
 # It is recommended to build releases from a clean checkout.
 # However, for testing it may be easier to use an existing workspace
@@ -133,9 +132,19 @@ find . -type f \
 	-print0 |
     xargs -0 cp -a --parents -t .dist/allegro
 
+
 # from now on, the scripts runs inside .dist
 cd .dist
 
+
+# generate the tar.gz first as the files are in Unix format
+# This is missing the manifest file, but who cares?
+VERSION=$(grep '^set(ALLEGRO_VERSION ' allegro/CMakeLists.txt | tr -cd '[0-9.]')
+mv allegro allegro-$VERSION
+tar zcf allegro-$VERSION.tar.gz allegro-$VERSION
+mv allegro-$VERSION allegro
+
+
 # convert files to DOS format for .zip and .7z archives
 proc_filelist
 utod "$AL_FILELIST"
@@ -157,7 +166,62 @@ echo "allegro/allegro.mft" >> allegro/allegro.mft
 utod allegro/allegro.mft
 zip -9 $name.zip allegro/allegro.mft
 
-mv $name.zip "$currdir"
-cd "$currdir"
 
-echo "Done! Check . for the output."
+# if we are building diffs as well, do those
+if [ $# -eq 2 ]; then
+
+   echo "Inflating current version ($name.zip)..."
+   mkdir current
+   unzip -q $name.zip -d current
+
+   echo "Inflating previous version ($2)..."
+   mkdir previous
+   unzip -q ../../"$2" -d previous || exit 1
+
+   echo "Generating diffs..."
+   set +e
+   diff -U 3 -N --recursive previous/ current/ > $name.diff
+   set -e
+
+   echo "Deleting temp files..."
+   rm -r previous
+   rm -r current
+
+
+   # generate the .txt file for the diff archive
+   echo "This is a set of diffs to upgrade from $prev to $name." > $name.txt
+   echo >> $name.txt
+   echo "Date: $(date '+%A %B %d, %Y, %H:%M')" >> $name.txt
+   echo >> $name.txt
+   echo "To apply this patch, copy $name.diff into the same directory where you" >> $name.txt
+   echo "installed Allegro (this should be one level up from the allegro/ dir, eg." >> $name.txt
+   echo "if your Allegro installation lives in c:/djgpp/allegro/, you should be in" >> $name.txt
+   echo "c:/djgpp/). Then type the command:" >> $name.txt
+   echo >> $name.txt
+   echo "   patch -p1 < $name.diff" >> $name.txt
+   echo >> $name.txt
+   echo "Change into the allegro directory, run make, and enjoy!" >> $name.txt
+
+   utod $name.txt
+
+
+   # zip up the diff archive
+   echo "Creating ${name}_diff.zip..."
+   if [ -f ${name}_diff.zip ]; then rm ${name}_diff.zip; fi
+   zip -9 ${name}_diff.zip $name.diff $name.txt
+
+
+   # find if we need to add any binary files as well
+   bin=$(sed -n -e "s/Binary files previous\/\(.*\) and current.* differ/\1/p" $name.diff)
+
+   if [ "$bin" != "" ]; then
+      echo "Adding binary diffs..."
+      zip -9 ${name}_diff.zip $bin
+   fi
+
+   rm $name.diff $name.txt
+
+fi
+
+
+echo "Done! Check .dist for the output."
