@@ -25,6 +25,17 @@
 
 
 
+#define PREFIX_I                "al-main INFO: "
+#define PREFIX_W                "al-main WARNING: "
+#define PREFIX_E                "al-main ERROR: "
+
+#ifdef ALLEGRO_AMIGA
+   #define LOGFILE "RAM:allegro.log"
+#else
+   #define LOGFILE "allegro.log"
+#endif
+
+
 /* in case you want to report version numbers */
 char allegro_id[] = "Allegro " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR;
 
@@ -45,6 +56,14 @@ int *allegro_errno = NULL;
 int _allegro_count = 0;
 
 
+/* flag to know whether we are being called by the exit mechanism */
+int _allegro_in_exit = FALSE;
+
+
+/* flag to decide whether to disable the screensaver */
+int _screensaver_policy = FULLSCREEN_DISABLED;
+
+
 /* the graphics driver currently in use */
 GFX_DRIVER *gfx_driver = NULL;
 
@@ -52,8 +71,6 @@ GFX_DRIVER *gfx_driver = NULL;
 /* abilities of the current graphics driver */
 int gfx_capabilities = 0;
 
-/* pointer to list of valid video modes */
-GFX_MODE_LIST *gfx_mode_list = NULL;
 
 /* a bitmap structure for accessing the physical screen */
 BITMAP *screen = NULL;
@@ -181,7 +198,7 @@ int os_multitasking = FALSE;
 
 
 /* processor information */
-char cpu_vendor[32] = EMPTY_STRING;
+char cpu_vendor[_AL_CPU_VENDOR_SIZE] = EMPTY_STRING;
 int cpu_family = 0;
 int cpu_model = 0;
 int cpu_capabilities = 0;
@@ -195,7 +212,7 @@ static FILE *assert_file = NULL;
 static FILE *trace_file = NULL;
 
 static int (*assert_handler)(AL_CONST char *msg) = NULL;
-static int (*trace_handler)(AL_CONST char *msg) = NULL;
+int (*_al_trace_handler)(AL_CONST char *msg) = NULL;
 
 
 /* Module linking system stuff: if an object file is linked in, then its
@@ -211,6 +228,7 @@ struct _AL_LINKER_MOUSE *_al_linker_mouse = NULL;
 /* dynamic registration system for cleanup code */
 struct al_exit_func {
    void (*funcptr)(void);
+   AL_CONST char *desc;
    struct al_exit_func *next;
 };
 
@@ -220,8 +238,10 @@ static struct al_exit_func *exit_func_list = NULL;
 
 /* _add_exit_func:
  *  Adds a function to the list that need to be called by allegro_exit().
+ *  `desc' should point to a statically allocated string to help with
+ *  debugging.
  */
-void _add_exit_func(void (*func)(void))
+void _add_exit_func(void (*func)(void), AL_CONST char *desc)
 {
    struct al_exit_func *n;
 
@@ -229,12 +249,13 @@ void _add_exit_func(void (*func)(void))
       if (n->funcptr == func)
 	 return;
 
-   n = malloc(sizeof(struct al_exit_func));
+   n = _AL_MALLOC(sizeof(struct al_exit_func));
    if (!n)
       return;
 
    n->next = exit_func_list;
    n->funcptr = func;
+   n->desc = desc;
    exit_func_list = n;
 }
 
@@ -253,7 +274,7 @@ void _remove_exit_func(void (*func)(void))
 	    prev->next = iter->next;
 	 else
 	    exit_func_list = iter->next;
-	 free(iter);
+	 _AL_FREE(iter);
 	 return;
       }
       prev = iter;
@@ -263,10 +284,22 @@ void _remove_exit_func(void (*func)(void))
 
 
 
-/* install_allegro:
+/* allegro_exit_stub:
+ *  Stub function registered by the library via atexit.
+ */
+static void allegro_exit_stub(void)
+{
+   _allegro_in_exit = TRUE;
+
+   allegro_exit();
+}
+
+
+
+/* _install_allegro:
  *  Initialises the Allegro library, activating the system driver.
  */
-int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func)(void)))
+static int _install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func)(void)))
 {
    RGB black_rgb = {0, 0, 0, 0};
    char tmp1[64], tmp2[64];
@@ -282,6 +315,8 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
       _midi_constructor();
       _mouse_constructor();
       _register_bitmap_file_type_init();
+      _register_sample_file_type_init();
+      _register_font_file_type_init();
    #endif
 
    if (errno_ptr)
@@ -315,6 +350,9 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
    /* nasty stuff to set up the config system before the system driver */
    system_driver = _system_driver_list[0].driver;
 
+   /* needed in case set_config_file was called before allegro_init */
+   _reload_config();
+
    reload_config_texts(NULL);
 
    if (system_id == SYSTEM_AUTODETECT)
@@ -340,28 +378,82 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
       }
    }
 
-   if (!system_driver) {
-      AL_CONST char *msg = get_config_text("Fatal error: unable to activate the Allegro system");
+   if (!system_driver)
+      return -1;
 
-      if (ugetc(allegro_error))
-	 allegro_message(uconvert_ascii("%s\n%s\n", tmp1), msg, allegro_error);
-      else
-	 allegro_message(uconvert_ascii("%s\n", tmp1), msg);
-
-      exit(EXIT_FAILURE);
-   }
+   /* disable close button */
+   set_close_button_callback(NULL);
 
    /* detect CPU type */
    check_cpu();
 
+#if defined(ALLEGRO_UNIX) || defined(ALLEGRO_WINDOWS)
+   /* detect filename encoding used by libc */
+   /* XXX This should be done for all platforms but I'm not set up to check
+    * that all platforms have been covered before making the 4.2.3 release.
+    * --pw
+    */
+   _al_detect_filename_encoding();
+#endif
+
+   /* set up screensaver policy */
+   _screensaver_policy = get_config_int(uconvert_ascii("system", tmp1),
+                                        uconvert_ascii("disable_screensaver", tmp2),
+                                        FULLSCREEN_DISABLED);
+
    /* install shutdown handler */
    if (_allegro_count == 0) {
       if (atexit_ptr)
-	 atexit_ptr(allegro_exit);
+	 atexit_ptr(allegro_exit_stub);
    }
 
    _allegro_count++;
 
+   TRACE(PREFIX_I "Allegro initialised (instance %d)\n", _allegro_count);
+   return 0;
+}
+
+
+
+/* _install_allegro_version_check:
+ *  Initialises the Allegro library, but return with an error if an
+ *  incompatible version is found.
+ */
+int _install_allegro_version_check(int system_id, int *errno_ptr,
+   int (*atexit_ptr)(void (*func)(void)), int version)
+{
+   int r = _install_allegro(system_id, errno_ptr, atexit_ptr);
+
+   int build_wip = version & 255;
+   int build_ver = version & ~255;
+
+   int version_ok;
+
+   if (r != 0) {
+      /* failed */
+      return r;
+   }
+
+#if ALLEGRO_SUB_VERSION & 1
+   /* This is a WIP runtime, so enforce strict compatibility. */
+   version_ok = version == MAKE_VERSION(ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, ALLEGRO_WIP_VERSION);
+#else
+   /* This is a stable runtime, so the runtime should be at least as new
+    * as the build headers (otherwise we may get a crash, since some
+    * functions may have been used which aren't available in this runtime).
+    */
+   version_ok = (MAKE_VERSION(ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, 0) == build_ver) &&
+      (ALLEGRO_WIP_VERSION >= build_wip);
+#endif
+
+   if (!version_ok) {
+      uszprintf(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text(
+         "The detected dynamic Allegro library (%d.%d.%d) is "
+         "not compatible with this program (%d.%d.%d)."),
+         ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, ALLEGRO_WIP_VERSION,
+         build_ver >> 16, (build_ver >> 8) & 255, build_wip);
+      return -1;
+   }
    return 0;
 }
 
@@ -370,10 +462,13 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
 /* allegro_exit:
  *  Closes down the Allegro system.
  */
-void allegro_exit()
+void allegro_exit(void)
 {
-   while (exit_func_list)
-      (*(exit_func_list->funcptr))();
+   while (exit_func_list) {
+      void (*func)(void) = exit_func_list->funcptr;
+      _remove_exit_func(func);
+      (*func)();
+   }
 
    if (system_driver) {
       system_driver->exit();
@@ -381,7 +476,7 @@ void allegro_exit()
    }
 
    if (_scratch_mem) {
-      free(_scratch_mem);
+      _AL_FREE(_scratch_mem);
       _scratch_mem = NULL;
       _scratch_mem_size = 0;
    }
@@ -394,10 +489,10 @@ void allegro_exit()
  */
 void allegro_message(AL_CONST char *msg, ...)
 {
-   char *buf = malloc(ALLEGRO_MESSAGE_SIZE);
-   char *tmp = malloc(ALLEGRO_MESSAGE_SIZE);
-
+   char *buf = _AL_MALLOC_ATOMIC(ALLEGRO_MESSAGE_SIZE);
+   char *tmp = _AL_MALLOC_ATOMIC(ALLEGRO_MESSAGE_SIZE);
    va_list ap;
+   ASSERT(msg);
    va_start(ap, msg);
    uvszprintf(buf, ALLEGRO_MESSAGE_SIZE, msg, ap);
    va_end(ap);
@@ -407,8 +502,8 @@ void allegro_message(AL_CONST char *msg, ...)
    else
       fputs(uconvert(buf, U_CURRENT, tmp, U_ASCII_CP, ALLEGRO_MESSAGE_SIZE), stdout);
 
-   free(buf);
-   free(tmp);
+   _AL_FREE(buf);
+   _AL_FREE(tmp);
 }
 
 
@@ -419,6 +514,7 @@ void allegro_message(AL_CONST char *msg, ...)
 void get_executable_name(char *output, int size)
 {
    ASSERT(system_driver);
+   ASSERT(output);
 
    if (system_driver->get_executable_name) {
       system_driver->get_executable_name(output, size);
@@ -432,6 +528,23 @@ void get_executable_name(char *output, int size)
 
 
 
+/* set_close_button_callback:
+ *  Installs a callback function to be called when the close button
+ *  of the window or any equivalent device is activated. Returns 0 on
+ *  success and -1 on failure (e.g. feature not supported).
+ */
+int set_close_button_callback(void (*proc)(void))
+{
+   ASSERT(system_driver);
+
+   if (system_driver->set_close_button_callback)
+      return system_driver->set_close_button_callback(proc);
+
+   return -1;
+}
+
+
+
 /* debug_exit:
  *  Closes the debugging output files.
  */
@@ -485,7 +598,7 @@ void al_assert(AL_CONST char *file, int line)
 	 assert_file = NULL;
 
       if (debug_trace_virgin)
-	 _add_exit_func(debug_exit);
+	 _add_exit_func(debug_exit, "debug_exit");
 
       debug_assert_virgin = FALSE;
    }
@@ -527,8 +640,8 @@ void al_trace(AL_CONST char *msg, ...)
    vsprintf(buf, msg, ap);
    va_end(ap);
 
-   if (trace_handler) {
-      if (trace_handler(buf))
+   if (_al_trace_handler) {
+      if (_al_trace_handler(buf))
 	 return;
    }
 
@@ -538,10 +651,10 @@ void al_trace(AL_CONST char *msg, ...)
       if (s)
 	 trace_file = fopen(s, "w");
       else
-	 trace_file = fopen("allegro.log", "w");
+	 trace_file = fopen(LOGFILE, "w");
 
       if (debug_assert_virgin)
-	 _add_exit_func(debug_exit);
+	 _add_exit_func(debug_exit, "debug_exit");
 
       debug_trace_virgin = FALSE;
    }
@@ -571,7 +684,7 @@ void register_assert_handler(int (*handler)(AL_CONST char *msg))
  */
 void register_trace_handler(int (*handler)(AL_CONST char *msg))
 {
-   trace_handler = handler;
+   _al_trace_handler = handler;
 }
 
 
@@ -580,7 +693,7 @@ void register_trace_handler(int (*handler)(AL_CONST char *msg))
  *  Wrapper for when a program needs to manipulate memory that has been
  *  allocated by the Allegro DLL.
  */
-void *_al_malloc(int size)
+void *_al_malloc(size_t size)
 {
    return malloc(size);
 }
@@ -602,13 +715,30 @@ void _al_free(void *mem)
  *  Wrapper for when a program needs to manipulate memory that has been
  *  allocated by the Allegro DLL.
  */
-void *_al_realloc(void *mem, int size)
+void *_al_realloc(void *mem, size_t size)
 {
    return realloc(mem, size);
 }
 
 
 
+/* _al_strdup:
+ *  Wrapper for when a program needs to duplicate a string in a way that
+ *  uses any user overloaded memory allocation system in use.
+ *  The result of this function _must_ be freed with _AL_FREE().
+ */
+char *_al_strdup(AL_CONST char *string)
+{
+   char *newstring = _AL_MALLOC(strlen(string) + 1);
+
+   if (newstring)
+      strcpy(newstring, string);
+
+   return newstring;
+}
+
+
+
 /* a simple system driver for platform-independent code */
 static int sys_none_init(void) { return 0; }
 static void sys_none_exit(void) { }
@@ -625,8 +755,9 @@ SYSTEM_DRIVER system_none =
    "Agnostic",
    sys_none_init,
    sys_none_exit,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    sys_no_driver, sys_no_driver, sys_no_driver, sys_no_driver,
    sys_no_driver, sys_no_driver, sys_no_driver
 };
