@@ -15,6 +15,8 @@
  *      desktop_color_depth() and yield_timeslice() support added by
  *      Angelo Mottola.
  *
+ *      Synchronization functions added by Eric Botcazou.
+ *
  *      See readme.txt for copyright information.
  */
 
@@ -25,7 +27,7 @@
 #include <sys/utsname.h>
 #endif
 
-#ifndef ALLEGRO_BEOS
+#if !defined ALLEGRO_BEOS && !defined ALLEGRO_HAIKU
 #error something is wrong with the makefile
 #endif              
 
@@ -34,7 +36,9 @@
 
 #define EXE_NAME_UNKNOWN "./UNKNOWN"
 
-#define YIELD_TIME			30000
+#define PREFIX_I                "al-bsys INFO: "
+#define PREFIX_W                "al-bsys WARNING: "
+#define PREFIX_E                "al-bsys ERROR: "
 
 
 
@@ -49,6 +53,7 @@ static thread_id main_thread_id = -1;
 static sem_id system_started = -1;
 
 static char app_path[MAXPATHLEN] = ".";
+char wnd_title[WND_TITLE_SIZE];
 
 
 static bool      using_custom_allegro_app;
@@ -91,10 +96,10 @@ extern "C" void _be_sys_get_executable_name(char *output, int size)
    cookie = 0;
 
    if (get_next_image_info(0, &cookie, &info) == B_NO_ERROR) {
-      strncpy(output, info.name, size);
+      _al_sane_strncpy(output, info.name, size);
    }
    else {
-      strncpy(output, EXE_NAME_UNKNOWN, size);
+      _al_sane_strncpy(output, EXE_NAME_UNKNOWN, size);
    }
 
    output[size-1] = '\0';
@@ -115,7 +120,7 @@ static int32 system_thread(void *data)
 
       _be_sys_get_executable_name(exe, sizeof(exe));
 
-      strncat(sig, get_filename(exe), sizeof(sig));
+      strncat(sig, get_filename(exe), sizeof(sig)-1);
       sig[sizeof(sig)-1] = '\0';
 
       _be_allegro_app = new BeAllegroApp(sig);
@@ -130,7 +135,7 @@ static int32 system_thread(void *data)
          p = &exe[strlen(exe) - 1];
          while (*p != '/') p--;
          *(p + 1) = '\0';
-         strcpy(app_path, exe);
+         _al_sane_strncpy(app_path, exe, MAXPATHLEN);
       }
    }
    else {
@@ -139,7 +144,9 @@ static int32 system_thread(void *data)
 
    _be_allegro_app->Run();
 
-   AL_TRACE("system thread exited\n");
+   /* XXX commented out due to conflicting TRACE in Haiku
+   TRACE(PREFIX_I "system thread exited\n");
+   */
 
    return 0;
 }
@@ -153,6 +160,7 @@ extern "C" int be_sys_init(void)
    int32       cookie;
    thread_info info;
    struct utsname os_name;
+   char path[MAXPATHLEN];
 
    cookie = 0;
 
@@ -195,11 +203,13 @@ extern "C" int be_sys_init(void)
    uname(&os_name);
    os_type = OSTYPE_BEOS;
    os_multitasking = TRUE;
-   os_version = atoi(strtok(os_name.release, "."));
-   os_revision = atoi(strtok(NULL, "."));
 
    chdir(app_path);
 
+   _be_sys_get_executable_name(path, sizeof(path));
+   path[sizeof(path)-1] = '\0';
+   do_uconvert(get_filename(path), U_CURRENT, wnd_title, U_UTF8, WND_TITLE_SIZE);
+
    return 0;
 
    cleanup: {
@@ -329,35 +339,30 @@ int be_sys_find_resource(char *dest, AL_CONST char *resource, int size)
 
 extern "C" void be_sys_set_window_title(AL_CONST char *name)
 {
-   char uname[256];
-
-   do_uconvert(name, U_CURRENT, uname, U_UTF8, sizeof(uname));
+   do_uconvert(name, U_CURRENT, wnd_title, U_UTF8, WND_TITLE_SIZE);
    if (_be_window)
-      _be_window->SetTitle(uname);
+      _be_window->SetTitle(wnd_title);
 }
 
 
 
-extern "C" int be_sys_set_window_close_button(int enable)
+extern "C" int be_sys_set_close_button_callback(void (*proc)(void))
 {
    if (!_be_window)
       return -1;
-   if (enable)
+
+   _be_window_close_hook = proc;
+
+   if (proc)
       _be_window->SetFlags(_be_window->Flags() & ~B_NOT_CLOSABLE);
    else
       _be_window->SetFlags(_be_window->Flags() | B_NOT_CLOSABLE);
+
    return 0;
 }
 
 
 
-extern "C" void be_sys_set_window_close_hook(void (*proc)(void))
-{
-   _be_window_close_hook = proc;
-}
-
-
-
 extern "C" void be_sys_message(AL_CONST char *msg)
 {
    char  filename[MAXPATHLEN];
@@ -413,9 +418,103 @@ extern "C" int be_sys_get_desktop_resolution(int *width, int *height)
 
 
 
+extern "C" void be_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
+{
+   *driver = GFX_BWINDOW;
+   mode->width = 320;
+   mode->height = 200;
+   mode->bpp = 8;
+}
+
+
+
 extern "C" void be_sys_yield_timeslice(void)
 {
-   snooze(YIELD_TIME);
+   snooze(0);
+}
+
+
+
+/* custom mutex that supports nested locking */
+struct my_mutex {
+   int lock_count;                /* level of nested locking     */
+   thread_id owner;               /* thread which owns the mutex */
+   sem_id actual_mutex;           /* underlying mutex object     */
+};
+
+
+
+/* be_sys_create_mutex:
+ *  Creates a mutex and returns a pointer to it.
+ */
+extern "C" void *be_sys_create_mutex(void)
+{
+   struct my_mutex *mx;
+
+   mx = (struct my_mutex *)malloc(sizeof(struct my_mutex));
+   if (!mx) {
+      *allegro_errno = ENOMEM;
+      return NULL;
+   }
+
+   mx->lock_count = 0;
+   mx->owner = (thread_id) 0;
+
+   mx->actual_mutex = create_sem(1, "internal mutex");
+   if (mx->actual_mutex < 0) {
+      free(mx);
+      return NULL;
+   }
+
+   return (void *)mx;
+}
+
+
+
+/* be_sys_destroy_mutex:
+ *  Destroys a mutex.
+ */
+extern "C" void be_sys_destroy_mutex(void *handle)
+{
+   struct my_mutex *mx = (struct my_mutex *)handle;
+
+   delete_sem(mx->actual_mutex);
+
+   free(mx);
+}
+
+
+
+/* be_sys_lock_mutex:
+ *  Locks a mutex.
+ */
+extern "C" void be_sys_lock_mutex(void *handle)
+{
+   struct my_mutex *mx = (struct my_mutex *)handle;
+
+   if (mx->owner != find_thread(NULL)) {
+      acquire_sem(mx->actual_mutex);
+      mx->owner = find_thread(NULL);      
+   }
+
+   mx->lock_count++;
+}
+
+
+
+/* be_sys_unlock_mutex:
+ *  Unlocks a mutex.
+ */
+extern "C" void be_sys_unlock_mutex(void *handle)
+{
+   struct my_mutex *mx = (struct my_mutex *)handle;
+
+   mx->lock_count--;
+
+   if (mx->lock_count == 0) {
+      mx->owner = (thread_id) 0;
+      release_sem(mx->actual_mutex);
+   }
 }
 
 
@@ -471,7 +570,7 @@ int32 killer_thread(void *data)
 
 
 
-void be_terminate(thread_id caller, bool exit_caller)
+void _be_terminate(thread_id caller, bool exit_caller)
 {
    thread_id killer;
 
