@@ -18,6 +18,8 @@
  */
 
 
+#include <string.h>
+
 #include "allegro.h"
 #include "allegro/internal/aintern.h"
 
@@ -47,7 +49,7 @@ typedef struct BITMAPFILEHEADER
 typedef struct BITMAPINFOHEADER
 {
    unsigned long  biWidth;
-   unsigned long  biHeight;
+   signed long    biHeight;
    unsigned short biBitCount;
    unsigned long  biCompression;
 } BITMAPINFOHEADER;
@@ -56,7 +58,7 @@ typedef struct BITMAPINFOHEADER
 typedef struct WINBMPINFOHEADER  /* size: 40 */
 {
    unsigned long  biWidth;
-   unsigned long  biHeight;
+   signed long    biHeight;
    unsigned short biPlanes;
    unsigned short biBitCount;
    unsigned long  biCompression;
@@ -150,17 +152,25 @@ static int read_os2_bminfoheader(PACKFILE *f, BITMAPINFOHEADER *infoheader)
 /* read_bmicolors:
  *  Loads the color palette for 1,4,8 bit formats.
  */
-static void read_bmicolors(int ncols, RGB *pal, PACKFILE *f,int win_flag)
+static void read_bmicolors(int bytes, RGB *pal, PACKFILE *f, int win_flag)
 {
-   int i;
+   int i, j;
 
-   for (i=0; i<ncols; i++) {
-      pal[i].b = pack_getc(f) / 4;
-      pal[i].g = pack_getc(f) / 4;
-      pal[i].r = pack_getc(f) / 4;
-      if (win_flag)
+   for (i=j=0; (i+3 <= bytes && j < PAL_SIZE); j++) {
+      pal[j].b = pack_getc(f) / 4;
+      pal[j].g = pack_getc(f) / 4;
+      pal[j].r = pack_getc(f) / 4;
+
+      i += 3;
+
+      if (win_flag && i < bytes) {
 	 pack_getc(f);
+	 i++;
+      }
    }
+
+   for (; i<bytes; i++)
+      pack_getc(f);
 }
 
 
@@ -247,59 +257,185 @@ static void read_8bit_line(int length, PACKFILE *f, BITMAP *bmp, int line)
 
 
 
+/* read_16bit_line:
+ *  Support function for reading the 16 bit bitmap file format, doing
+ *  our best to convert it down to a 256 color palette.
+ */
+static void read_16bit_line(int length, PACKFILE *f, BITMAP *bmp, int line)
+{
+   int i, w;
+   RGB c;
+
+   for (i=0; i<length; i++) {
+      w = pack_igetw(f);
+
+      /* the format is like a 15-bpp bitmap, not 16bpp */
+      c.r = (w >> 10) & 0x1f;
+      c.g = (w >> 5) & 0x1f;
+      c.b = w & 0x1f;
+
+      bmp_write16(bmp->line[line]+i*2,
+	 makecol16(_rgb_scale_5[c.r],
+	    _rgb_scale_5[c.g],
+	    _rgb_scale_5[c.b]));
+   }
+
+   /* padding */
+   i = (i * 2) % 4;
+   if (i != 0) {
+      while (i++ < 4)
+	 pack_getc(f);
+   }
+}
+
+
+
 /* read_24bit_line:
  *  Support function for reading the 24 bit bitmap file format, doing
  *  our best to convert it down to a 256 color palette.
  */
 static void read_24bit_line(int length, PACKFILE *f, BITMAP *bmp, int line)
 {
-   int i, nbytes;
+   int i;
    RGB c;
 
-   nbytes=0;
-
    for (i=0; i<length; i++) {
       c.b = pack_getc(f);
       c.g = pack_getc(f);
       c.r = pack_getc(f);
       WRITE3BYTES(bmp->line[line]+i*3, makecol24(c.r, c.g, c.b));
-      nbytes += 3;
    }
 
-   nbytes = nbytes % 4;
-   if (nbytes != 0)
-      for (i=nbytes; i<4; i++)
-	 pack_getc(f);
+   /* padding */
+   i = (i * 3) % 4;
+   if (i != 0) {
+      while (i++ < 4)
+         pack_getc(f);
+   }
 }
 
 
 
+/* read_32bit_line:
+ *  Support function for reading the 32 bit bitmap file format, doing
+ *  our best to convert it down to a 256 color palette.
+ */
+static void read_32bit_line(int length, PACKFILE *f, BITMAP *bmp, int line)
+{
+   int i;
+   RGB c;
+   char a;
+
+   for (i=0; i<length; i++) {
+      c.b = pack_getc(f);
+      c.g = pack_getc(f);
+      c.r = pack_getc(f);
+      a = pack_getc(f);
+      WRITE3BYTES(bmp->line[line]+i*4, makeacol32(c.r, c.g, c.b, a));
+   }
+}
+
+
+
+/* read_bitfields_image:
+ *  For reading the bitfield compressed BMP image format.
+ */
+static void read_bitfields_image(PACKFILE *f, BITMAP *bmp, AL_CONST BITMAPINFOHEADER *infoheader)
+{
+   int k, i, line, height, dir;
+   int bpp;
+   int bytes_per_pixel;
+   int red, grn, blu;
+   unsigned long buffer;
+
+   height = infoheader->biHeight;
+   line   = height < 0 ? 0 : height-1;
+   dir    = height < 0 ? 1 : -1;
+   height = ABS(height);
+   
+   bpp = bitmap_color_depth(bmp);
+   bytes_per_pixel = BYTES_PER_PIXEL(bpp);
+
+   for (i=0; i<height; i++, line+=dir) {
+      for (k=0; k<(int)infoheader->biWidth; k++) {
+
+	 pack_fread(&buffer, bytes_per_pixel, f);
+
+	 if (bpp == 15) {
+	    red = (buffer >> 10) & 0x1f;
+	    grn = (buffer >> 5) & 0x1f;
+	    blu = (buffer) & 0x1f;
+	    buffer = (red << _rgb_r_shift_15) |
+		     (grn << _rgb_g_shift_15) |
+		     (blu << _rgb_b_shift_15);
+	 }
+	 else if (bpp == 16) {
+	    red = (buffer >> 11) & 0x1f;
+	    grn = (buffer >> 5) & 0x3f;
+	    blu = (buffer) & 0x1f;
+	    buffer = (red << _rgb_r_shift_16) |
+		     (grn << _rgb_g_shift_16) |
+		     (blu << _rgb_b_shift_16);
+	 }
+	 else {
+	    red = (buffer >> 16) & 0xff;
+	    grn = (buffer >> 8) & 0xff;
+	    blu = (buffer) & 0xff;
+	    buffer = (red << _rgb_r_shift_32) |
+		     (grn << _rgb_g_shift_32) |
+		     (blu << _rgb_b_shift_32);
+	 }
+
+	 memcpy(&bmp->line[line][k * bytes_per_pixel], &buffer, bytes_per_pixel);
+      }
+
+      /* padding */
+      k = (k * bytes_per_pixel) % 4;
+      if (k > 0) {
+	 while (k++ < 4)
+	    pack_getc(f);
+      }
+   }
+}
+
+
 /* read_image:
  *  For reading the noncompressed BMP image format.
  */
 static void read_image(PACKFILE *f, BITMAP *bmp, AL_CONST BITMAPINFOHEADER *infoheader)
 {
-   int i, line;
+   int i, line, height, dir;
 
-   for (i=0; i<(int)infoheader->biHeight; i++) {
-      line = i;
+   height = infoheader->biHeight;
+   line   = height < 0 ? 0: height-1;
+   dir    = height < 0 ? 1: -1;
+   height = ABS(height);
 
+   for (i=0; i<height; i++, line+=dir) {
       switch (infoheader->biBitCount) {
 
 	 case 1:
-	    read_1bit_line(infoheader->biWidth, f, bmp, infoheader->biHeight-i-1);
+	    read_1bit_line(infoheader->biWidth, f, bmp, line);
 	    break;
 
 	 case 4:
-	    read_4bit_line(infoheader->biWidth, f, bmp, infoheader->biHeight-i-1);
+	    read_4bit_line(infoheader->biWidth, f, bmp, line);
 	    break;
 
 	 case 8:
-	    read_8bit_line(infoheader->biWidth, f, bmp, infoheader->biHeight-i-1);
+	    read_8bit_line(infoheader->biWidth, f, bmp, line);
+	    break;
+
+	 case 16:
+	    read_16bit_line(infoheader->biWidth, f, bmp, line);
 	    break;
 
 	 case 24:
-	    read_24bit_line(infoheader->biWidth, f, bmp, infoheader->biHeight-i-1);
+	    read_24bit_line(infoheader->biWidth, f, bmp, line);
+	    break;
+
+	 case 32:
+	    read_32bit_line(infoheader->biWidth, f, bmp, line);
 	    break;
       }
    }
@@ -463,16 +599,41 @@ static void read_RLE4_compressed_image(PACKFILE *f, BITMAP *bmp, AL_CONST BITMAP
  *  Thanks to Seymour Shlien for contributing this function.
  */
 BITMAP *load_bmp(AL_CONST char *filename, RGB *pal)
+{
+   PACKFILE *f;
+   BITMAP *bmp;
+   ASSERT(filename);
+
+   f = pack_fopen(filename, F_READ);
+   if (!f)
+      return NULL;
+
+   bmp = load_bmp_pf(f, pal);
+
+   pack_fclose(f);
+
+   return bmp;
+}
+
+
+
+/* load_bmp_pf:
+ *  Like load_bmp, but starts loading from the current place in the PACKFILE
+ *  specified. If successful the offset into the file will be left just after
+ *  the image data. If unsuccessful the offset into the file is unspecified,
+ *  i.e. you must either reset the offset to some known place or close the
+ *  packfile. The packfile is not closed by this function.
+ */
+BITMAP *load_bmp_pf(PACKFILE *f, RGB *pal)
 {
    BITMAPFILEHEADER fileheader;
    BITMAPINFOHEADER infoheader;
-   PACKFILE *f;
    BITMAP *bmp;
    PALETTE tmppal;
    int want_palette = TRUE;
-   int ncol;
    unsigned long biSize;
    int bpp, dest_depth;
+   ASSERT(f);
 
    /* we really need a palette */
    if (!pal) {
@@ -480,12 +641,7 @@ BITMAP *load_bmp(AL_CONST char *filename, RGB *pal)
       pal = tmppal;
    }
 
-   f = pack_fopen(filename, F_READ);
-   if (!f)
-      return NULL;
-
    if (read_bmfileheader(f, &fileheader) != 0) {
-      pack_fclose(f);
       return NULL;
    }
 
@@ -493,37 +649,55 @@ BITMAP *load_bmp(AL_CONST char *filename, RGB *pal)
 
    if (biSize == WININFOHEADERSIZE) {
       if (read_win_bminfoheader(f, &infoheader) != 0) {
-	 pack_fclose(f);
 	 return NULL;
       }
-      /* compute number of colors recorded */
-      ncol = (fileheader.bfOffBits - 54) / 4;
-      read_bmicolors(ncol, pal, f, 1);
+      if (infoheader.biCompression != BI_BITFIELDS)
+	 read_bmicolors(fileheader.bfOffBits - 54, pal, f, 1);
    }
    else if (biSize == OS2INFOHEADERSIZE) {
       if (read_os2_bminfoheader(f, &infoheader) != 0) {
-	 pack_fclose(f);
 	 return NULL;
       }
-      /* compute number of colors recorded */
-      ncol = (fileheader.bfOffBits - 26) / 3;
-      read_bmicolors(ncol, pal, f, 0);
+      if (infoheader.biCompression != BI_BITFIELDS)
+	 read_bmicolors(fileheader.bfOffBits - 26, pal, f, 0);
    }
    else {
-      pack_fclose(f);
       return NULL;
    }
 
    if (infoheader.biBitCount == 24)
       bpp = 24;
+   else if (infoheader.biBitCount == 16)
+      bpp = 16;
+   else if (infoheader.biBitCount == 32)
+      bpp = 32;
    else
       bpp = 8;
 
+   if (infoheader.biCompression == BI_BITFIELDS) {
+      unsigned long redMask = pack_igetl(f);
+      unsigned long grnMask = pack_igetl(f);
+      unsigned long bluMask = pack_igetl(f);
+
+      (void)grnMask;
+
+      if ((bluMask == 0x001f) && (redMask == 0x7C00))
+	 bpp = 15;
+      else if ((bluMask == 0x001f) && (redMask == 0xF800))
+	 bpp = 16;
+      else if ((bluMask == 0x0000FF) && (redMask == 0xFF0000))
+	 bpp = 32;
+      else {
+	 /* Unrecognised bit masks/depth, refuse to load. */
+	 return NULL;
+      }
+   }
+
+
    dest_depth = _color_load_depth(bpp, FALSE);
 
-   bmp = create_bitmap_ex(bpp, infoheader.biWidth, infoheader.biHeight);
+   bmp = create_bitmap_ex(bpp, infoheader.biWidth, ABS(infoheader.biHeight));
    if (!bmp) {
-      pack_fclose(f);
       return NULL;
    }
 
@@ -543,6 +717,10 @@ BITMAP *load_bmp(AL_CONST char *filename, RGB *pal)
 	 read_RLE4_compressed_image(f, bmp, &infoheader);
 	 break;
 
+      case BI_BITFIELDS:
+	 read_bitfields_image(f, bmp, &infoheader);
+	 break;
+
       default:
 	 destroy_bitmap(bmp);
 	 bmp = NULL;
@@ -553,14 +731,15 @@ BITMAP *load_bmp(AL_CONST char *filename, RGB *pal)
       if ((bpp != 8) && (!want_palette))
 	 pal = NULL;
 
-      bmp = _fixup_loaded_bitmap(bmp, pal, dest_depth);
+      if (bmp) {
+	 bmp = _fixup_loaded_bitmap(bmp, pal, dest_depth);
+      }
    }
    
    /* construct a fake palette if 8-bit mode is not involved */
    if ((bpp != 8) && (dest_depth != 8) && want_palette)
       generate_332_palette(pal);
 
-   pack_fclose(f);
    return bmp;
 }
 
@@ -573,12 +752,43 @@ BITMAP *load_bmp(AL_CONST char *filename, RGB *pal)
 int save_bmp(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal) 
 {
    PACKFILE *f;
+   int ret;
+   ASSERT(filename);
+
+   f = pack_fopen(filename, F_WRITE);
+   if (!f) 
+      return -1;
+
+   ret = save_bmp_pf(f, bmp, pal);
+
+   pack_fclose(f);
+
+   return ret;
+}
+
+
+
+/* save_bmp_pf:
+ *  Like save_bmp but writes into the PACKFILE given instead of a new file.
+ *  The packfile is not closed after writing is completed. On success the
+ *  offset into the file is left after the TGA file just written. On failure
+ *  the offset is left at the end of whatever incomplete data was written.
+ */
+int save_bmp_pf(PACKFILE *f, BITMAP *bmp, AL_CONST RGB *pal) 
+{
    PALETTE tmppal;
    int bfSize;
-   int depth = bitmap_color_depth(bmp);
-   int bpp = (depth == 8) ? 8 : 24;
-   int filler = 3 - ((bmp->w*(bpp/8)-1) & 3);
+   int biSizeImage;
+   int depth;
+   int bpp;
+   int filler;
    int c, i, j;
+   ASSERT(f);
+   ASSERT(bmp);
+
+   depth = bitmap_color_depth(bmp);
+   bpp = (depth == 8) ? 8 : 24;
+   filler = 3 - ((bmp->w*(bpp/8)-1) & 3);
 
    if (!pal) {
       get_palette(tmppal);
@@ -586,18 +796,17 @@ int save_bmp(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal)
    }
 
    if (bpp == 8) {
-      bfSize = 54                      /* header */
-	       +256*4                  /* palette */
-	       +bmp->w*bmp->h;         /* image data */
+      biSizeImage = (bmp->w + filler) * bmp->h;
+      bfSize = (54		       /* header */
+		+ 256*4		       /* palette */
+		+ biSizeImage);	       /* image data */
    }
    else {
-      bfSize = 54                      /* header */
-	       +bmp->w*bmp->h*3;       /* image data */
+      biSizeImage = (bmp->w*3 + filler) * bmp->h;
+      bfSize = 54 + biSizeImage;       /* header + image data */
    }
 
-   f = pack_fopen(filename, F_WRITE);
-   if (!f) 
-      return *allegro_errno;
+   *allegro_errno = 0;
 
    /* file_header */
    pack_iputw(0x4D42, f);              /* bfType ("BM") */
@@ -611,17 +820,15 @@ int save_bmp(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal)
       pack_iputl(54, f); 
 
    /* info_header */
-   bfSize = bmp->w * bmp->h * bpp/8;
-
    pack_iputl(40, f);                  /* biSize */
    pack_iputl(bmp->w, f);              /* biWidth */
    pack_iputl(bmp->h, f);              /* biHeight */
    pack_iputw(1, f);                   /* biPlanes */
    pack_iputw(bpp, f);                 /* biBitCount */
    pack_iputl(0, f);                   /* biCompression */
-   pack_iputl(bfSize, f);              /* biSizeImage */
-   pack_iputl(0, f);                   /* biXPelsPerMeter */
-   pack_iputl(0, f);                   /* biYPelsPerMeter */
+   pack_iputl(biSizeImage, f);         /* biSizeImage */
+   pack_iputl(0xB12, f);               /* biXPelsPerMeter (0xB12 = 72 dpi) */
+   pack_iputl(0xB12, f);               /* biYPelsPerMeter */
 
    if (bpp == 8) {
       pack_iputl(256, f);              /* biClrUsed */
@@ -658,7 +865,9 @@ int save_bmp(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal)
 	 pack_putc(0, f);
    }
 
-   pack_fclose(f);
-   return *allegro_errno;
+   if (*allegro_errno)
+      return -1;
+   else
+      return 0;
 }
 
