@@ -21,8 +21,10 @@
 #define PP_DEPTH               32
 
 #define PIXEL_PTR              uint32_t*
+#define PTR_PER_PIXEL          1
 #define OFFSET_PIXEL_PTR(p,x)  ((PIXEL_PTR) (p) + (x))
 #define INC_PIXEL_PTR(p)       ((p)++)
+#define INC_PIXEL_PTR_N(p,d)   ((p) += d)
 #define DEC_PIXEL_PTR(p)       ((p)--)
 
 #define PUT_PIXEL(p,c)         bmp_write32((uintptr_t) (p), (c))
@@ -48,6 +50,7 @@
 #define DLS_BLENDER            BLENDER_FUNC
 #define MAKE_DLS_BLENDER(a)    _blender_func32
 #define DLS_BLEND(b,a,n)       ((*(b))(_blender_col_32, (n), (a)))
+#define DLSX_BLEND(b,n)        ((*(b))(_blender_col_32, (n), _blender_alpha))
 
 /* Blender for poly_scanline_*_lit.  */
 #define PS_BLENDER             BLENDER_FUNC
@@ -74,6 +77,7 @@
 #define FUNC_LINEAR_VLINE                   _linear_vline32
 
 #define FUNC_LINEAR_DRAW_SPRITE             _linear_draw_sprite32
+#define FUNC_LINEAR_DRAW_SPRITE_EX          _linear_draw_sprite_ex32
 #define FUNC_LINEAR_DRAW_256_SPRITE         _linear_draw_256_sprite32
 #define FUNC_LINEAR_DRAW_SPRITE_V_FLIP      _linear_draw_sprite_v_flip32
 #define FUNC_LINEAR_DRAW_SPRITE_H_FLIP      _linear_draw_sprite_h_flip32
