@@ -20,7 +20,7 @@
 
 #ifdef ALLEGRO_COLOR8
 
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 #include "cdefs8.h"
 #include "cgfx.h"
 
