@@ -20,7 +20,9 @@
 
 #ifdef ALLEGRO_COLOR32
 
-#include "allegro/aintern.h"
+#undef _bma_scan_gcol
+
+#include "allegro/internal/aintern.h"
 #include "cdefs32.h"
 #include "cscan.h"
 
