@@ -1,4 +1,4 @@
-/*         ______   ___    ___ 
+/*         ______   ___    ___
  *        /\  _  \ /\_ \  /\_ \ 
  *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
@@ -14,12 +14,12 @@
  *
  *      Hook functions added by Martijn Versteegh.
  *
+ *      Annie Testes lifted several hardcoded length limitations.
+ *
  *      See readme.txt for copyright information.
  */
 
 
-#include <string.h>
-
 #include "allegro.h"
 #include "allegro/internal/aintern.h"
 
@@ -62,6 +62,10 @@ static CONFIG_HOOK *config_hook = NULL;
 
 static int config_installed = FALSE;
 
+static char **config_argv = NULL;
+static char *argv_buf = NULL;
+static int argv_buf_size = 0;
+
 
 
 /* flush_config:
@@ -133,7 +137,7 @@ static void destroy_config(CONFIG *cfg)
       flush_config(cfg);
 
       if (cfg->filename)
-	 free(cfg->filename);
+	 _AL_FREE(cfg->filename);
 
       /* destroy the variable list */
       pos = cfg->head;
@@ -143,15 +147,15 @@ static void destroy_config(CONFIG *cfg)
 	 pos = pos->next;
 
 	 if (prev->name)
-	    free(prev->name);
+	    _AL_FREE(prev->name);
 
 	 if (prev->data)
-	    free(prev->data);
+	    _AL_FREE(prev->data);
 
-	 free(prev);
+	 _AL_FREE(prev);
       }
 
-      free(cfg);
+      _AL_FREE(cfg);
    }
 }
 
@@ -193,16 +197,24 @@ static void config_cleanup(void)
 
       while (hook) {
 	 if (hook->section)
-	    free(hook->section);
+	    _AL_FREE(hook->section);
 
 	 nexthook = hook->next; 
-	 free(hook);
+	 _AL_FREE(hook);
 	 hook = nexthook;
       }
 
       config_hook = NULL;
    }
 
+   _AL_FREE(config_argv);
+   config_argv = NULL;
+   
+   _AL_FREE(argv_buf);
+   argv_buf = NULL;
+   
+   argv_buf_size = 0;
+
    _remove_exit_func(config_cleanup);
    config_installed = FALSE;
 }
@@ -219,7 +231,7 @@ static void init_config(int loaddata)
    char filename[1024], tmp[128], *cfg_name;
 
    if (!config_installed) {
-      _add_exit_func(config_cleanup);
+      _add_exit_func(config_cleanup, "config_cleanup");
       config_installed = TRUE;
    }
 
@@ -236,7 +248,7 @@ static void init_config(int loaddata)
    }
 
    if (!system_config) {
-      system_config = malloc(sizeof(CONFIG));
+      system_config = _AL_MALLOC(sizeof(CONFIG));
       if (system_config) {
 	 system_config->head = NULL;
 	 system_config->filename = NULL;
@@ -247,12 +259,18 @@ static void init_config(int loaddata)
 
 
 
-/* get_line: 
- *  Helper for splitting files up into individual lines.
+/* get_line:
+ *  Helper for splitting files up into individual lines. Returns the length
+ *  in bytes of the sequence of characters delimited by the first EOL marker
+ *  in the array DATA of length LENGTH, and allocates NAME and VAL to record
+ *  the name and the value of the config entry respectively; otherwise set
+ *  NAME to NULL and returns a copy of the line through VAL if the line was
+ *  blank or a comment. Returns -1 and set allegro_errno on failure.
  */
-static int get_line(AL_CONST char *data, int length, char *name, int name_size, char *val, int val_size)
+static int get_line(AL_CONST char *data, int length, char **name, char **val)
 {
-   char buf[512], buf2[512];
+   char *buf;
+   int buf_size=256;
    int inpos, outpos, i, j;
    int c, c2, w0;
 
@@ -260,7 +278,14 @@ static int get_line(AL_CONST char *data, int length, char *name, int name_size,
    outpos = 0;
    w0 = ucwidth(0);
 
-   while ((inpos<length) && (outpos<(int)sizeof(buf)-w0)) {
+   buf = _AL_MALLOC_ATOMIC(buf_size);
+   if (!buf) {
+     *allegro_errno = ENOMEM;
+     return -1;
+   }
+
+   /* search for an EOL marker */
+   while (inpos<length) {
       c = ugetc(data+inpos);
       if ((c == '\r') || (c == '\n')) {
 	 inpos += uwidth(data+inpos);
@@ -272,6 +297,16 @@ static int get_line(AL_CONST char *data, int length, char *name, int name_size,
 	 break;
       }
 
+      /* increase the buffer size if needed */
+      if (outpos>=(int)buf_size-w0) {
+	 buf_size *= 2;
+	 buf = _al_sane_realloc(buf, buf_size);
+	 if (!buf) {
+	    *allegro_errno = ENOMEM;
+	    return -1;
+	 }
+      }
+
       outpos += usetc(buf+outpos, c);
       inpos += uwidth(data+inpos);
    }
@@ -290,35 +325,53 @@ static int get_line(AL_CONST char *data, int length, char *name, int name_size,
    /* read name string */
    j = 0;
 
+   /* compute name length */
    while ((c) && (!uisspace(c)) && (c != '=') && (c != '#')) {
-      j += usetc(buf2+j, c);
+      j += ucwidth(c);
       i += uwidth(buf+i);
       c = ugetc(buf+i);
    }
 
    if (j) {
       /* got a variable */
-      usetc(buf2+j, 0);
-      ustrzcpy(name, name_size, buf2);
+      *name = _AL_MALLOC_ATOMIC(j+w0);
+      if (!(*name)) {
+	 *allegro_errno = ENOMEM;
+	 _AL_FREE(buf);
+	 return -1;
+      }
+
+      ustrzcpy(*name, j+w0, buf+i-j);
 
       while ((c) && ((uisspace(c)) || (c == '='))) {
 	 i += uwidth(buf+i);
 	 c = ugetc(buf+i);
       }
 
-      ustrzcpy(val, val_size, buf+i);
+      *val = _al_ustrdup(buf+i);
+      if (!(*val)) {
+	 _AL_FREE(name);
+	 _AL_FREE(buf);
+	 return -1;
+      }
 
       /* strip trailing spaces */
-      i = ustrlen(val) - 1;
-      while ((i >= 0) && (uisspace(ugetat(val, i))))
-	 usetat(val, i--, 0);
+      i = ustrlen(*val) - 1;
+      while ((i >= 0) && (uisspace(ugetat(*val, i))))
+	 usetat(*val, i--, 0);
    }
    else {
       /* blank line or comment */
-      usetc(name, 0);
-      ustrzcpy(val, val_size, buf);
+      *name = NULL;
+      *val = _al_ustrdup(buf);
+      if (!(*val)) {
+	 _AL_FREE(buf);
+	 return -1;
+      }
    }
 
+   _AL_FREE(buf);
+
    return inpos;
 }
 
@@ -329,9 +382,9 @@ static int get_line(AL_CONST char *data, int length, char *name, int name_size,
  */
 static void set_config(CONFIG **config, AL_CONST char *data, int length, AL_CONST char *filename)
 {
-   char name[256], val[256];
    CONFIG_ENTRY **prev, *p;
-   int pos;
+   char *name, *val;
+   int ret, pos;
 
    init_config(FALSE);
 
@@ -340,15 +393,23 @@ static void set_config(CONFIG **config, AL_CONST char *data, int length, AL_CONS
       *config = NULL;
    }
 
-   *config = malloc(sizeof(CONFIG));
-   if (!(*config))
+   *config = _AL_MALLOC(sizeof(CONFIG));
+   if (!(*config)) {
+      *allegro_errno = ENOMEM;
       return;
+   }
 
    (*config)->head = NULL;
    (*config)->dirty = FALSE;
 
-   if (filename)
-      (*config)->filename = ustrdup(filename);
+   if (filename) {
+      (*config)->filename = _al_ustrdup(filename);
+      if (!(*config)->filename) {
+	 _AL_FREE(*config);
+	 *config = NULL;
+	 return;
+      }
+   }
    else
       (*config)->filename = NULL;
 
@@ -356,18 +417,25 @@ static void set_config(CONFIG **config, AL_CONST char *data, int length, AL_CONS
    pos = 0;
 
    while (pos < length) {
-      pos += get_line(data+pos, length-pos, name, sizeof(name), val, sizeof(val));
-
-      p = malloc(sizeof(CONFIG_ENTRY));
-      if (!p)
+      ret = get_line(data+pos, length-pos, &name, &val);
+      if (ret<0) {
+	 _AL_FREE(*config);
+	 *config = NULL;
 	 return;
+      }
 
-      if (ugetc(name))
-         p->name = ustrdup(name);
-      else
-	 p->name = NULL;
+      pos += ret;
 
-      p->data = ustrdup(val);
+      p = _AL_MALLOC(sizeof(CONFIG_ENTRY));
+      if (!p) {
+	 *allegro_errno = ENOMEM;
+	 _AL_FREE(*config);
+	 *config = NULL;
+	 return;
+      }
+
+      p->name = name;
+      p->data = val;
 
       p->next = NULL;
       *prev = p;
@@ -383,20 +451,26 @@ static void set_config(CONFIG **config, AL_CONST char *data, int length, AL_CONS
 static void load_config_file(CONFIG **config, AL_CONST char *filename, AL_CONST char *savefile)
 {
    char *tmp, *tmp2;
-   int length;
+   uint64_t length;
 
    if (*config) {
       destroy_config(*config);
       *config = NULL;
    }
 
-   length = file_size(filename);
+   /* Special case when allegro_init has not been called yet. */
+   if (!system_driver) {
+      set_config(config, NULL, 0, savefile);
+      return;
+   }
+
+   length = file_size_ex(filename);
 
    if (length > 0) {
       PACKFILE *f = pack_fopen(filename, F_READ);
 
       if (f) {
-	 tmp = malloc(length+1);
+	 tmp = _AL_MALLOC_ATOMIC(length+1);
 
 	 if (tmp) {
 	    pack_fread(tmp, length, f);
@@ -404,10 +478,10 @@ static void load_config_file(CONFIG **config, AL_CONST char *filename, AL_CONST
 
 	    if (need_uconvert(tmp, U_UTF8, U_CURRENT)) {
 	       length = uconvert_size(tmp, U_UTF8, U_CURRENT);
-	       tmp2 = malloc(length);
+	       tmp2 = _AL_MALLOC_ATOMIC(length);
 
 	       if (tmp2)
-		  do_uconvert(tmp, U_UTF8, tmp2, U_CURRENT, -1);
+		  do_uconvert(tmp, U_UTF8, tmp2, U_CURRENT, length);
 
 	       length -= ucwidth(0);
 	    }
@@ -418,10 +492,10 @@ static void load_config_file(CONFIG **config, AL_CONST char *filename, AL_CONST
 	       set_config(config, tmp2, length, savefile);
 
 	       if (tmp2 != tmp)
-		  free(tmp2);
+		  _AL_FREE(tmp2);
 	    }
 
-	    free(tmp);
+	    _AL_FREE(tmp);
 	 }
 	 else
 	    set_config(config, NULL, 0, savefile);
@@ -442,6 +516,7 @@ static void load_config_file(CONFIG **config, AL_CONST char *filename, AL_CONST
  */
 void set_config_file(AL_CONST char *filename)
 {
+   ASSERT(filename);
    load_config_file(&config[0], filename, filename);
 }
 
@@ -453,6 +528,7 @@ void set_config_file(AL_CONST char *filename)
  */
 void set_config_data(AL_CONST char *data, int length)
 {
+   ASSERT(data);
    set_config(&config[0], data, length, NULL);
 }
 
@@ -463,7 +539,14 @@ void set_config_data(AL_CONST char *data, int length)
  */
 void override_config_file(AL_CONST char *filename)
 {
-   load_config_file(&config_override, filename, NULL);
+   /* load other configuration file to override settings */
+   if (filename)
+      load_config_file(&config_override, filename, filename);
+   /* destroy the current one */
+   else if (config_override) {
+      destroy_config(config_override);
+      config_override = NULL;
+   }
 }
 
 
@@ -474,6 +557,7 @@ void override_config_file(AL_CONST char *filename)
  */
 void override_config_data(AL_CONST char *data, int length)
 {
+   ASSERT(data);
    set_config(&config_override, data, length, NULL);
 }
 
@@ -574,8 +658,8 @@ void hook_config_section(AL_CONST char *section, int (*intgetter)(AL_CONST char
 	 else {
 	    /* remove a hook */
 	    *prev = hook->next;
-	    free(hook->section);
-	    free(hook);
+	    _AL_FREE(hook->section);
+	    _AL_FREE(hook);
 	 }
 
 	 return;
@@ -586,13 +670,13 @@ void hook_config_section(AL_CONST char *section, int (*intgetter)(AL_CONST char
    }
 
    /* add a new hook */
-   hook = malloc(sizeof(CONFIG_HOOK));
+   hook = _AL_MALLOC(sizeof(CONFIG_HOOK));
    if (!hook)
       return;
 
-   hook->section = ustrdup(section_name);
+   hook->section = _al_ustrdup(section_name);
    if (!(hook->section)) {
-      free(hook);
+      _AL_FREE(hook);
       return;
    }
 
@@ -832,11 +916,9 @@ int get_config_id(AL_CONST char *section, AL_CONST char *name, int def)
  */
 char **get_config_argv(AL_CONST char *section, AL_CONST char *name, int *argc)
 {
-   #define MAX_ARGV  16
-
-   static char buf[256];
-   static char *argv[MAX_ARGV];
    int pos, ac, q, c;
+   int s_size;
+   int i;
 
    AL_CONST char *s = get_config_string(section, name, NULL);
 
@@ -845,44 +927,97 @@ char **get_config_argv(AL_CONST char *section, AL_CONST char *name, int *argc)
       return NULL;
    }
 
-   ustrzcpy(buf, sizeof(buf), s);
+   /* clean up the old argv that was allocated the last time this function was
+    * called.
+    */
+   _AL_FREE(config_argv);
+   config_argv = NULL;
+
+   /* increase the buffer size if needed */
+   s_size = ustrsizez(s);
+   if (s_size>argv_buf_size) {
+      argv_buf_size = s_size;
+      argv_buf = _al_sane_realloc(argv_buf, argv_buf_size);
+      if (!argv_buf) {
+         *allegro_errno = ENOMEM;
+         *argc = 0;
+         return NULL;
+      }
+   }
+
+   ustrzcpy(argv_buf, argv_buf_size, s);
    pos = 0;
    ac = 0;
 
-   c = ugetc(buf);
-
-   while ((ac<MAX_ARGV) && (c) && (c != '#')) {
+   /* tokenize the buffer and count the number of words; every space character
+    * as well as single and double quotes are replaced with zeros; comments * that start with # are also cut off with a 0
+    */
+   c = ugetc(argv_buf);
+   while ((c) && (c != '#')) {
+      /* replace all spaces up to the next word with 0 */
       while ((c) && (uisspace(c))) {
-	 pos += ucwidth(c);
-	 c = ugetc(buf+pos);
+	 usetat(argv_buf+pos, 0, 0);
+	 pos += ucwidth(0);
+	 c = ugetc(argv_buf+pos);
       }
 
+      /* quit if we reached the end of the buffer or a comment */
       if ((c) && (c != '#')) {
-	 if ((c == '\'') || (c == '"')) {
-	    q = c;
-	    pos += ucwidth(c);
-	    c = ugetc(buf+pos);
-	 }
-	 else
-	    q = 0;
+         /* found another word! */
+         ac++;
 
-	 argv[ac++] = buf+pos;
+         /* cut away quotes by replacing them with 0 */
+         if ((c == '\'') || (c == '"')) {
+            q = c;
+            usetat(argv_buf+pos, 0, 0);
+            pos += ucwidth(0);
+            c = ugetc(argv_buf+pos);
+         }
+         else {
+            q = 0;
+         }
 
-	 while ((c) && ((q) ? (c != q) : (!uisspace(c)))) {
-	    pos += ucwidth(c);
-	    c = ugetc(buf+pos);
-	 }
+         /* search for the end of the word */
+         while ((c) && ((q) ? (c != q) : (!uisspace(c)))) {
+            pos += ucwidth(c);
+            c = ugetc(argv_buf+pos);
+         }
+      }
+   }
 
-	 if (c) {
-	    usetat(buf+pos, 0, 0);
-	    pos += ucwidth(0);
-	    c = ugetc(buf+pos);
-	 }
+   /* now that we know how many words there are in the buffer, allocate enough
+    * space for a list of pointers to them, or return 0 if there are no words
+    */
+   if (ac > 0) {
+      config_argv = _AL_MALLOC(ac*sizeof *config_argv);
+   }
+   else {
+      *argc = 0;
+      return NULL;
+   }
+
+   /* go through the tokenized buffer and assign pointers in argv to point to
+    * the beginning of each individual word
+    */
+   for (i=0,pos=0,c=ugetc(argv_buf); i<ac; i++) {
+      /* find next word */
+      while (!c) {
+         pos += ucwidth(c);
+         c = ugetc(argv_buf+pos);
+      }
+
+      /* assign pointer */
+      config_argv[i] = argv_buf+pos;
+
+      /* find end of the word */
+      while (c) {
+         pos += ucwidth(c);
+         c = ugetc(argv_buf+pos);
       }
    }
 
    *argc = ac;
-   return argv;
+   return config_argv;
 }
 
 
@@ -892,18 +1027,18 @@ char **get_config_argv(AL_CONST char *section, AL_CONST char *name, int *argc)
  */
 static CONFIG_ENTRY *insert_variable(CONFIG *the_config, CONFIG_ENTRY *p, AL_CONST char *name, AL_CONST char *data)
 {
-   CONFIG_ENTRY *n = malloc(sizeof(CONFIG_ENTRY));
+   CONFIG_ENTRY *n = _AL_MALLOC(sizeof(CONFIG_ENTRY));
 
    if (!n)
       return NULL;
 
    if (name)
-      n->name = ustrdup(name);
+      n->name = _al_ustrdup(name);
    else
       n->name = NULL;
 
    if (data)
-      n->data = ustrdup(data);
+      n->data = _al_ustrdup(data);
    else
       n->data = NULL;
 
@@ -950,6 +1085,8 @@ void set_config_string(AL_CONST char *section, AL_CONST char *name, AL_CONST cha
    /* decide which config file to use */
    if ((ugetc(name) == '#') || ((ugetc(section_name) == '[') && (ugetat(section_name, 1) == '#')))
       the_config = system_config;
+   else if (config_override)
+      the_config = config_override;
    else
       the_config = config[0];
 
@@ -960,24 +1097,24 @@ void set_config_string(AL_CONST char *section, AL_CONST char *name, AL_CONST cha
 	 if ((val) && (ugetc(val))) {
 	    /* modify existing variable */
 	    if (p->data)
-	       free(p->data);
+	       _AL_FREE(p->data);
 
-	    p->data = ustrdup(val);
+	    p->data = _al_ustrdup(val);
 	 }
 	 else {
 	    /* delete variable */
 	    if (p->name)
-	       free(p->name);
+	       _AL_FREE(p->name);
 
 	    if (p->data)
-	       free(p->data);
+	       _AL_FREE(p->data);
 
 	    if (prev)
 	       prev->next = p->next;
 	    else
 	       the_config->head = p->next;
 
-	    free(p);
+	    _AL_FREE(p);
 	 }
       }
       else {
@@ -1092,6 +1229,21 @@ void set_config_id(AL_CONST char *section, AL_CONST char *name, int val)
 
 
 
+/* _reload_config:
+ *  Internal helper to reload the configuration from allegro_init, in case
+ *  set_config_file was called before allegro_init.
+ */
+void _reload_config(void)
+{
+   if (config[0]) {
+      char *name = _al_ustrdup(config[0]->filename);
+      set_config_file(name);
+      _AL_FREE(name);
+   }
+}
+
+
+
 /* reload_config_texts:
  *  Reads in a block of translated system text, looking for either a
  *  user-specified file, a ??text.cfg file, or a language.dat#??TEXT_CFG 
@@ -1116,7 +1268,7 @@ void reload_config_texts(AL_CONST char *new_language)
    name = get_config_string(uconvert_ascii("system", tmp1), uconvert_ascii("language", tmp2), NULL);
 
    if ((name) && (ugetc(name))) {
-      namecpy = ustrdup(name);
+      namecpy = _al_ustrdup(name);
       ustrlwr (namecpy);
       if ((ustrlen(namecpy)<4) || (ustricmp(namecpy+uoffset(namecpy, -4), uconvert_ascii("text", tmp1)) != 0))
 	 ext = uconvert_ascii("text.cfg", tmp1);
@@ -1126,15 +1278,15 @@ void reload_config_texts(AL_CONST char *new_language)
       datafile = uconvert_ascii("language.dat", tmp2);
 
       if (find_allegro_resource(filename, namecpy, ext, datafile, NULL, NULL, NULL, sizeof(filename)) == 0) {
-	 free(namecpy);
+	 _AL_FREE(namecpy);
 	 load_config_file(&config_language, filename, NULL);
 	 return;
       }
 
-      free(namecpy);
+      _AL_FREE(namecpy);
    }
 
-   config_language = malloc(sizeof(CONFIG));
+   config_language = _AL_MALLOC(sizeof(CONFIG));
    if (config_language ) {
       config_language ->head = NULL;
       config_language ->filename = NULL;
@@ -1151,62 +1303,214 @@ void reload_config_texts(AL_CONST char *new_language)
  */
 AL_CONST char *get_config_text(AL_CONST char *msg)
 {
-   char tmp1[256], tmp2[256], name[256];
-   AL_CONST char *section = uconvert_ascii("[language]", tmp1);
-   AL_CONST char *umsg = uconvert_ascii(msg, tmp2);
+   char tmp1[256];
+   AL_CONST char *section;
+   AL_CONST char *umsg;
    AL_CONST char *s;
+   AL_CONST char *ret = NULL;
+   char *name;
    CONFIG_HOOK *hook;
    CONFIG_ENTRY *p;
-   int c, pos;
+   int c, pos, size;
+   ASSERT(msg);
+
+   /* Hack: the inline definition of install_allegro() from 4.2.0 calls
+    * get_config_text() even before Allegro has been initialised, leading
+    * to a crash in get_executable_name().  To retain binary compatibility
+    * we check for this case.
+    */
+   if (_allegro_count == 0) {
+      return msg;
+   }
 
    init_config(TRUE);
 
-   /* kludge to make it easier to translate the message */
-   if (strcmp(msg, ALLEGRO_WINDOW_CLOSE_MESSAGE) == 0) {
-      do_uconvert("allegro_window_close_message", U_ASCII, name, U_CURRENT, sizeof(name));
-   }
-   else {
-      s = umsg;
-      pos = 0;
+   section = uconvert_ascii("[language]", tmp1);
 
-      while ((c = ugetxc(&s)) != 0) {
-	 if ((uisspace(c)) || (c == '=') || (c == '#'))
-	    pos += usetc(name+pos, '_');
-	 else
-	    pos += usetc(name+pos, c);
+   /* allocate memory and convert message to current encoding format */
+   if (need_uconvert(msg, U_ASCII, U_CURRENT)) {
+      size = uconvert_size(msg, U_ASCII, U_CURRENT);
+      umsg = _AL_MALLOC_ATOMIC(size);
+      if (!umsg) {
+	 *allegro_errno = ENOMEM;
+	 return empty_string;
       }
 
-      usetc(name+pos, 0);
+      name = _AL_MALLOC_ATOMIC(size);
+      if (!name) {
+	 _AL_FREE((char *)umsg);  /* remove constness */
+	 *allegro_errno = ENOMEM;
+	 return empty_string;
+      }
+
+      do_uconvert(msg, U_ASCII, (char*)umsg, U_CURRENT, size);
    }
+   else {
+      umsg = msg;
+      name = _AL_MALLOC_ATOMIC(ustrsizez(msg));
+      if (!name) {
+	 *allegro_errno = ENOMEM;
+	 return empty_string;
+      }
+   }
+
+   s = umsg;
+   pos = 0;
+
+   while ((c = ugetxc(&s)) != 0) {
+      if ((uisspace(c)) || (c == '=') || (c == '#'))
+	 pos += usetc(name+pos, '_');
+      else
+	 pos += usetc(name+pos, c);
+   }
+
+   usetc(name+pos, 0);
 
    /* check for hooked sections */
    hook = config_hook;
 
    while (hook) {
       if (ustricmp(section, hook->section) == 0) {
-	 if (hook->stringgetter)
-	    return hook->stringgetter(name, umsg);
+	 if (hook->stringgetter) {
+	    ret = hook->stringgetter(name, umsg);
+	    break;
+	 }
       }
+
       hook = hook->next;
    }
 
-   /* find the string */
-   p = find_config_string(config_override, section, name, NULL);
+   if (!ret) {
+      /* find the string */
+      p = find_config_string(config_override, section, name, NULL);
 
-   if (!p) {
-      p = find_config_string(config[0], section, name, NULL);
+      if (!p) {
+	 p = find_config_string(config[0], section, name, NULL);
 
-      if (!p)
-	 p = find_config_string(config_language, section, name, NULL);
+	 if (!p)
+	    p = find_config_string(config_language, section, name, NULL);
+      }
+
+      if (p) {
+	 ret = (p->data ? p->data : empty_string);
+      }
+      else {
+	 /* no translation, so store off this value in the file */
+	 p = config_language->head;
+	 insert_variable(config_language, NULL, name, umsg);
+	 config_language->head->next = p;
+	 ret = config_language->head->data;
+      }
    }
 
-   if (p)
-      return (p->data ? p->data : empty_string);
+   /* free memory */
+   if (umsg!=msg)
+      _AL_FREE((char*) umsg);  /* remove constness */
 
-   /* no translation, so store off this value in the file */
-   p = config_language->head;
-   insert_variable(config_language, NULL, name, umsg);
-   config_language->head->next = p;
-   return config_language->head->data;
+   _AL_FREE(name);
+
+   return ret;
 }
 
+
+
+/* add_unique_name
+ *  Helper to add a name to a list of names.
+ */
+static int add_unique_name(AL_CONST char ***names, int n, char const *name)
+{
+   int i;
+   /* FIXME: use better search algorithm */
+   for (i = 0; i < n; i++)
+      if (!ustrcmp((*names)[i], name))
+         return n;
+
+   *names = _al_sane_realloc((void *)*names, (n + 1) * sizeof **names);
+   (*names)[n] = name;
+   return n + 1;
+}
+
+
+
+/* attach_config_entries
+ *  Helper function to attach key or section names to a list of strings.
+ */
+static int attach_config_entries(CONFIG *conf, AL_CONST char *section,
+   int n, AL_CONST char ***names, int list_sections)
+{
+   CONFIG_ENTRY *p;
+   char section_name[256];
+   int in_section;
+
+   prettify_section_name(section, section_name, sizeof(section_name));
+
+   if (conf) {
+      p = conf->head;
+
+      /* If section is NULL, only initial, section-less entries are used. */
+      if (ugetc(section_name))
+         in_section = FALSE;
+      else
+         in_section = TRUE;
+
+      while (p) {
+         if (p->name) {
+            /* a section start is just a list entry enclosed in [] */
+            if (ugetc(p->name) == '[' && ugetat(p->name, -1) == ']') {
+               if (list_sections) {
+                  n = add_unique_name(names, n, p->name);
+               }
+               in_section = (ustricmp(section_name, p->name) == 0);
+            }
+            else if (in_section && !list_sections) {
+               n = add_unique_name(names, n, p->name);
+            }
+         }
+         p = p->next;
+      }
+   }
+   return n;
+}
+
+
+
+/* list_config_entires:
+ *  Returns the names of all config entries in a section. The names parameter is
+ *  a pointer to a strings array that will contain the config keys. If it points to
+ *  a NULL pointer, it will be allocated, or else re-allocated accordingly. The
+ *  return value tells how many valid string pointers it contains after the
+ *  function returns.
+ */
+int list_config_entries(AL_CONST char *section, AL_CONST char ***names)
+{
+   int n = 0;
+   n = attach_config_entries(config_override, section, n, names, 0);
+   n = attach_config_entries(config[0], section, n, names, 0);
+   return n;
+}
+
+
+
+/* list_config_sections:
+ *  Returns the names of all current config sections, enclodes in []. The names
+ *  parameter and return value is like in list_config_entires above.
+ */
+int list_config_sections(AL_CONST char ***names)
+{
+   int n = 0;
+   n = attach_config_entries(config_override, NULL, n, names, 1);
+   n = attach_config_entries(config[0], NULL, n, names, 1);
+   return n;
+}
+
+
+
+/* free_config_entries:
+ *  Frees the entries list returned by list_config_entires or
+ *  list_config_sections again.
+ */
+void free_config_entries(AL_CONST char ***names)
+{
+    _AL_FREE(*names);
+    *names = NULL;
+}
