@@ -34,4 +34,9 @@ BEGIN_JOYSTICK_DRIVER_LIST
    JOYSTICK_DRIVER_SNESPAD
    JOYSTICK_DRIVER_PSXPAD
    JOYSTICK_DRIVER_N64PAD
+   JOYSTICK_DRIVER_DB9
+   JOYSTICK_DRIVER_TURBOGRAFX
+   JOYSTICK_DRIVER_IFSEGA_ISA
+   JOYSTICK_DRIVER_IFSEGA_PCI
+   JOYSTICK_DRIVER_IFSEGA_PCI_FAST
 END_JOYSTICK_DRIVER_LIST
