@@ -96,7 +96,7 @@ static int fli_mem_pos = 0;            /* position in the memory FLI */
 static FLI_HEADER fli_header;          /* header structure */
 static FLI_FRAME frame_header;         /* frame header structure */
 
-static char _fli_broken_data[3 * 256]; /* data substituted for broken chunks */
+static unsigned char _fli_broken_data[3 * 256]; /* data substituted for broken chunks */
 
 
 
@@ -184,20 +184,33 @@ static void fli_skip(int bytes)
 
 
 /* helpers for reading FLI chunk data */
+#if 0
+/* the "cast expression as lvalue" extension is deprecated in GCC 3.4 */
 #define READ_BYTE_NC(p)    (*((unsigned char *)(p))++)
 #define READ_CHAR_NC(p)    (*((signed char *)(p))++)
+#else
+#define READ_BYTE_NC(p)    (*(unsigned char *)(p)++)
+#define READ_CHAR_NC(p)    (*(signed char *)(p)++)
+#endif
 
-#if (defined ALLEGRO_GCC) && (defined ALLEGRO_I386)
+#if (defined ALLEGRO_GCC) && (defined ALLEGRO_LITTLE_ENDIAN) && (!defined ALLEGRO_ARM) && (!defined ALLEGRO_PSP)
 
-/* for gcc on i386 */
-#define READ_WORD_NC(p)    (*((unsigned short *)(p))++)
-#define READ_SHORT_NC(p)   (*((signed short *)(p))++)
-#define READ_ULONG_NC(p)   (*((unsigned long *)(p))++)
-#define READ_LONG_NC(p)    (*((signed long *)(p))++)
+#if 0
+/* the "cast expression as lvalue" extension is deprecated in GCC 3.4 */
+#define READ_WORD_NC(p)    (*((uint16_t *)(p))++)
+#define READ_SHORT_NC(p)   (*((int16_t *)(p))++)
+#define READ_ULONG_NC(p)   (*((uint32_t *)(p))++)
+#define READ_LONG_NC(p)    (*((int32_t *)(p))++)
+#else
+/* we use the "statement-expression" extension instead */
+#define READ_WORD_NC(p)    ({ uint16_t *__p = (uint16_t *)(p); p+=2; *__p; })
+#define READ_SHORT_NC(p)   ({ int16_t *__p = (int16_t *)(p); p+=2; *__p; })
+#define READ_ULONG_NC(p)   ({ uint32_t *__p = (uint32_t *)(p); p+=4; *__p; })
+#define READ_LONG_NC(p)    ({ int32_t *__p = (int32_t *)(p); p+=4; *__p; })
+#endif
 
 #else
 
-/* for other compilers/platforms */
 static unsigned short _fli_read_word_nc(unsigned char **p)
 {
    unsigned short t;
@@ -257,7 +270,7 @@ static signed long _fli_read_long_nc(unsigned char **p)
 #define READ_RLE_WORD_NC(p,pos,size)                            \
 {                                                               \
    int c;                                                       \
-   unsigned short *ptr = (unsigned short*) (pos);               \
+   uint16_t *ptr = (uint16_t*) (pos);                           \
    unsigned short v = READ_WORD_NC(p);                          \
 								\
    for (c = 0; c < (size); c++)                                 \
@@ -330,7 +343,7 @@ static void do_fli_256_color(unsigned char *p, int sz)
       if (end > PAL_SIZE)
 	 return;
       else if ((sz -= length * 3) < 0) {
-	 FLI_KLUDGE((char *)p, sz, length * 3);
+	 FLI_KLUDGE(p, sz, length * 3);
       }
 
       fli_pal_dirty_from = MIN(fli_pal_dirty_from, offset);
@@ -396,7 +409,7 @@ static void do_fli_delta(unsigned char *p, int sz)
 	    if ((curr + size * 2) > bitmap_end)
 	       return;
 	    else if ((sz -= size * 2) < 0) {
-	       FLI_KLUDGE((char *)p, sz, size * 2);
+	       FLI_KLUDGE(p, sz, size * 2);
 	    }
 	    READ_BLOCK_NC(p, curr, size*2);
 	    curr += size*2;
@@ -406,7 +419,7 @@ static void do_fli_delta(unsigned char *p, int sz)
 	    if ((curr + size * 2) > bitmap_end)
 	       return;
 	    else if ((sz -= 2) < 0) {
-	       FLI_KLUDGE((char *)p, sz, 2);
+	       FLI_KLUDGE(p, sz, 2);
 	    }
 	    READ_RLE_WORD_NC(p, curr, size);
 	    curr += size*2;
@@ -446,7 +459,7 @@ static void do_fli_color(unsigned char *p, int sz)
       if (end > PAL_SIZE)
 	 return;
       else if ((sz -= length * 3) < 0) {
-	 FLI_KLUDGE((char *)p, sz, length * 3);
+	 FLI_KLUDGE(p, sz, length * 3);
       }
 
       fli_pal_dirty_from = MIN(fli_pal_dirty_from, offset);
@@ -503,7 +516,7 @@ static void do_fli_lc(unsigned char *p, int sz)
 	    if ((curr + size) > bitmap_end)
 	       return;
 	    else if ((sz -= size) < 0) {
-	       FLI_KLUDGE((char *)p, sz, size);
+	       FLI_KLUDGE(p, sz, size);
 	    }
 	    READ_BLOCK_NC(p, curr, size);
 	    curr += size;
@@ -513,7 +526,7 @@ static void do_fli_lc(unsigned char *p, int sz)
 	    if ((curr + size) > bitmap_end)
 	       return;
 	    else if ((sz -= 1) < 0) {
-	       FLI_KLUDGE((char *)p, sz, 1);
+	       FLI_KLUDGE(p, sz, 1);
 	    }
 	    READ_RLE_BYTE_NC(p, curr, size);
 	    curr += size;
@@ -572,7 +585,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       else if ((sz -= size) < 0) {
-		  FLI_KLUDGE((char *)p, sz, size);
+		  FLI_KLUDGE(p, sz, size);
 	       }
 	       READ_BLOCK_NC(p, curr, size);
 	       curr += size;
@@ -581,7 +594,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       else if ((sz -= 1) < 0) {
-		  FLI_KLUDGE((char *)p, sz, 1);
+		  FLI_KLUDGE(p, sz, 1);
 	       }
 	       READ_RLE_BYTE_NC(p, curr, size);
 	       curr += size;
@@ -600,7 +613,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       if ((sz -= size) < 0) {
-		  FLI_KLUDGE((char *)p, sz, size);
+		  FLI_KLUDGE(p, sz, size);
 	       }
 	       READ_BLOCK_NC(p, curr, size);
 	       curr += size;
@@ -609,7 +622,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       if ((sz -= 1) < 0) {
-		  FLI_KLUDGE((char *)p, sz, 1);
+		  FLI_KLUDGE(p, sz, 1);
 	       }
 	       READ_RLE_BYTE_NC(p, curr, size);
 	       curr += size;
@@ -809,6 +822,12 @@ static void read_frame(void)
       }
 
       p += sz;
+
+      /* alignment */
+      if (sz & 1) {
+	 p++;
+	 frame_size--;
+      }
    }
 
    /* move on to the next frame */
@@ -851,9 +870,10 @@ static int do_play_fli(BITMAP *bmp, int loop, int (*callback)(void))
 
       ret = next_fli_frame(loop);
 
-      do {
+      while (fli_timer <= 0) {
 	 /* wait a bit */
-      } while (fli_timer <= 0);
+	 rest(0);
+      }
    }
 
    close_fli();
@@ -877,6 +897,9 @@ static int do_play_fli(BITMAP *bmp, int loop, int (*callback)(void))
  */
 int play_fli(AL_CONST char *filename, BITMAP *bmp, int loop, int (*callback)(void))
 {
+   ASSERT(filename);
+   ASSERT(bmp);
+   
    if (open_fli(filename) != FLI_OK)
       return FLI_ERROR;
 
@@ -891,6 +914,9 @@ int play_fli(AL_CONST char *filename, BITMAP *bmp, int loop, int (*callback)(voi
  */
 int play_memory_fli(void *fli_data, BITMAP *bmp, int loop, int (*callback)(void))
 {
+   ASSERT(fli_data);
+   ASSERT(bmp);
+   
    if (open_memory_fli(fli_data) != FLI_OK)
       return FLI_ERROR;
 
@@ -961,15 +987,17 @@ static int do_open_fli(void)
  */
 int open_fli(AL_CONST char *filename)
 {
+   ASSERT(filename);
+   
    if (fli_status != FLI_NOT_OPEN)
       return FLI_ERROR;
 
    if (fli_filename) {
-      free(fli_filename);
+      _AL_FREE(fli_filename);
       fli_filename = NULL;
    }
 
-   fli_filename = ustrdup(filename);
+   fli_filename = _al_ustrdup(filename);
    if (!fli_filename)
       return FLI_ERROR;
 
@@ -988,6 +1016,8 @@ int open_fli(AL_CONST char *filename)
  */
 int open_memory_fli(void *fli_data)
 {
+   ASSERT(fli_data);
+   
    if (fli_status != FLI_NOT_OPEN)
       return FLI_ERROR;
 
@@ -1002,7 +1032,7 @@ int open_memory_fli(void *fli_data)
 /* close_fli:
  *  Shuts down the FLI player at the end of the file.
  */
-void close_fli()
+void close_fli(void)
 {
    remove_int(fli_timer_callback);
 
@@ -1012,7 +1042,7 @@ void close_fli()
    }
 
    if (fli_filename) {
-      free(fli_filename);
+      _AL_FREE(fli_filename);
       fli_filename = NULL;
    }
 
@@ -1068,7 +1098,7 @@ int next_fli_frame(int loop)
  *  Clears the information about which parts of the FLI bitmap and palette
  *  are dirty, after the screen hardware has been updated.
  */
-void reset_fli_variables()
+void reset_fli_variables(void)
 {
    fli_bmp_dirty_from = INT_MAX;
    fli_bmp_dirty_to = INT_MIN;
