@@ -21,6 +21,14 @@
 #include "allegro.h"
 #include "allegro/internal/aintern.h"
 
+extern void blit_end(void);   /* for LOCK_FUNCTION; defined in blit.c */
+
+
+
+#define PREFIX_I                "al-gfx INFO: "
+#define PREFIX_W                "al-gfx WARNING: "
+#define PREFIX_E                "al-gfx ERROR: "
+
 
 
 static int gfx_virgin = TRUE;          /* is the graphics system active? */
@@ -40,7 +48,9 @@ COLOR_MAP *color_map = NULL;           /* translucency/lighting table */
 int _color_depth = 8;                  /* how many bits per pixel? */
 
 int _refresh_rate_request = 0;         /* requested refresh rate */
-int _current_refresh_rate = 0;         /* refresh rate set by the driver */
+static int current_refresh_rate = 0;   /* refresh rate set by the driver */
+
+int _wait_for_vsync = TRUE;            /* vsync when page-flipping? */
 
 int _color_conv = COLORCONV_TOTAL;     /* which formats to auto convert? */
 
@@ -88,10 +98,10 @@ int _rgb_a_shift_32 = DEFAULT_RGB_A_SHIFT_32;
 /* lookup table for scaling 5 bit colors up to 8 bits */
 int _rgb_scale_5[32] =
 {
-   0,   8,   16,  24,  32,  41,  49,  57,
-   65,  74,  82,  90,  98,  106, 115, 123,
-   131, 139, 148, 156, 164, 172, 180, 189,
-   197, 205, 213, 222, 230, 238, 246, 255
+   0,   8,   16,  24,  33,  41,  49,  57,
+   66,  74,  82,  90,  99,  107, 115, 123,
+   132, 140, 148, 156, 165, 173, 181, 189,
+   198, 206, 214, 222, 231, 239, 247, 255
 };
 
 
@@ -100,12 +110,12 @@ int _rgb_scale_6[64] =
 {
    0,   4,   8,   12,  16,  20,  24,  28,
    32,  36,  40,  44,  48,  52,  56,  60,
-   64,  68,  72,  76,  80,  85,  89,  93,
+   65,  69,  73,  77,  81,  85,  89,  93,
    97,  101, 105, 109, 113, 117, 121, 125,
-   129, 133, 137, 141, 145, 149, 153, 157,
-   161, 165, 170, 174, 178, 182, 186, 190,
-   194, 198, 202, 206, 210, 214, 218, 222,
-   226, 230, 234, 238, 242, 246, 250, 255
+   130, 134, 138, 142, 146, 150, 154, 158,
+   162, 166, 170, 174, 178, 182, 186, 190,
+   195, 199, 203, 207, 211, 215, 219, 223,
+   227, 231, 235, 239, 243, 247, 251, 255
 };
 
 
@@ -116,13 +126,25 @@ typedef struct VRAM_BITMAP             /* list of video memory bitmaps */
 {
    int x, y, w, h;
    BITMAP *bmp;
-   struct VRAM_BITMAP *next;
+   struct VRAM_BITMAP *next_x, *next_y;
 } VRAM_BITMAP;
 
 
 static VRAM_BITMAP *vram_bitmap_list = NULL;
 
 
+#define BMP_MAX_SIZE  46340   /* sqrt(INT_MAX) */
+
+/* the product of these must fit in an int */
+static int failed_bitmap_w = BMP_MAX_SIZE;
+static int failed_bitmap_h = BMP_MAX_SIZE;
+
+
+
+static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_config);
+static int _set_gfx_mode_safe(int card, int w, int h, int v_w, int v_h);
+
+
 
 /* lock_bitmap:
  *  Locks all the memory used by a bitmap structure.
@@ -144,7 +166,7 @@ void lock_bitmap(BITMAP *bmp)
  */
 void request_refresh_rate(int rate)
 {
-    _refresh_rate_request = rate;
+   _refresh_rate_request = rate;
 }
 
 
@@ -154,7 +176,25 @@ void request_refresh_rate(int rate)
  */
 int get_refresh_rate(void)
 {
-    return _current_refresh_rate;
+   return current_refresh_rate;
+}
+
+
+
+/* _set_current_refresh_rate:
+ *  Sets the current refresh rate.
+ *  (This function must be called by the gfx drivers)
+ */
+void _set_current_refresh_rate(int rate)
+{
+   /* sanity check to discard bogus values */
+   if ((rate<40) || (rate>200))
+      rate = 0;
+
+   current_refresh_rate = rate;
+
+   /* adjust retrace speed */
+   _vsync_speed = rate ? BPS_TO_TIMER(rate) : BPS_TO_TIMER(70);
 }
 
 
@@ -202,7 +242,9 @@ GFX_MODE_LIST *get_gfx_mode_list(int card)
    _DRIVER_INFO *list_entry;
    GFX_DRIVER *drv = NULL;
    GFX_MODE_LIST *gfx_mode_list = NULL;
-   
+
+   ASSERT(system_driver);
+
    /* ask the system driver for a list of graphics hardware drivers */
    if (system_driver->gfx_drivers)
       list_entry = system_driver->gfx_drivers();
@@ -245,9 +287,9 @@ void destroy_gfx_mode_list(GFX_MODE_LIST *gfx_mode_list)
 {
    if (gfx_mode_list) {
       if (gfx_mode_list->mode)
-         free(gfx_mode_list->mode);
+         _AL_FREE(gfx_mode_list->mode);
 
-      free(gfx_mode_list);
+      _AL_FREE(gfx_mode_list);
    }
 }
 
@@ -273,6 +315,16 @@ void set_color_depth(int depth)
 
 
 
+/* get_color_depth:
+ *  Returns the current color depth.
+ */
+int get_color_depth(void)
+{
+   return _color_depth;
+}
+
+
+
 /* set_color_conversion:
  *  Sets a bit mask specifying which types of color format conversions are
  *  valid when loading data from disk.
@@ -286,6 +338,17 @@ void set_color_conversion(int mode)
 
 
 
+/* get_color_conversion:
+ *  Returns the bitmask specifying which types of color format
+ *  conversion are valid when loading data from disk.
+ */
+int get_color_conversion(void)
+{
+   return _color_conv;
+}
+
+
+
 /* _color_load_depth:
  *  Works out which color depth an image should be loaded as, given the
  *  current conversion mode.
@@ -404,16 +467,71 @@ static void shutdown_gfx(void)
 }
 
 
+#define GFX_DRIVER_FULLSCREEN_FLAG    0x01
+#define GFX_DRIVER_WINDOWED_FLAG      0x02
+
+
+/* gfx_driver_is_valid:
+ *  Checks that the graphics driver 'drv' fulfills the condition
+ *  expressed by the bitmask 'flags'.
+ */
+static int gfx_driver_is_valid(GFX_DRIVER *drv, int flags)
+{
+   if ((flags & GFX_DRIVER_FULLSCREEN_FLAG) && drv->windowed)
+      return FALSE;
+
+   if ((flags & GFX_DRIVER_WINDOWED_FLAG) && !drv->windowed)
+      return FALSE;
+
+   return TRUE;
+}
+
+
+
+/* get_gfx_driver_from_id:
+ *  Retrieves a pointer to the graphics driver identified by 'card' from
+ *  the list 'driver_list' or NULL if it doesn't exist.
+ */
+static GFX_DRIVER *get_gfx_driver_from_id(int card, _DRIVER_INFO *driver_list)
+{
+   int c;
+
+   for (c=0; driver_list[c].driver; c++) {
+      if (driver_list[c].id == card)
+	 return driver_list[c].driver;
+   }
+
+   return NULL;
+}
+
+
+
+/* init_gfx_driver:
+ *  Helper function for initializing a graphics driver.
+ */
+static BITMAP *init_gfx_driver(GFX_DRIVER *drv, int w, int h, int v_w, int v_h)
+{
+   drv->name = drv->desc = get_config_text(drv->ascii_name);
+
+   /* set gfx_driver so that it is visible when initializing the driver */
+   gfx_driver = drv;
+
+   return drv->init(w, h, v_w, v_h, _color_depth);
+}
+
+
 
 /* get_config_gfx_driver:
- *  Helper function for set_gfx_mode: it reads the gfx_card* config variables
- *  and sets the gfx_driver variable if the config variable was found
+ *  Helper function for set_gfx_mode: it reads the gfx_card* config variables and
+ *  tries to set the graphics mode if a matching driver is found. Returns TRUE if
+ *  at least one matching driver was found, FALSE otherwise.
  */
-static int get_config_gfx_driver(char *gfx_card, int check_mode, int require_window, _DRIVER_INFO *driver_list, int card, int w, int h, int v_w, int v_h)
+static int get_config_gfx_driver(char *gfx_card, int w, int h, int v_w, int v_h, int flags, _DRIVER_INFO *driver_list)
 {
-   int tried = FALSE;
    char buf[512], tmp[64];
-   int c, n;
+   GFX_DRIVER *drv;
+   int found = FALSE;
+   int card, n;
 
    /* try the drivers that are listed in the config file */
    for (n=-2; n<255; n++) {
@@ -439,39 +557,30 @@ static int get_config_gfx_driver(char *gfx_card, int check_mode, int require_win
 	    uszprintf(buf, sizeof(buf), uconvert_ascii("%s%d", tmp), gfx_card, n);
 	    break;
       }
-      card = get_config_id(uconvert_ascii("graphics", tmp), buf, GFX_AUTODETECT);
 
-      if (card != GFX_AUTODETECT) {
-	 for (c=0; driver_list[c].driver; c++) {
-	    if (driver_list[c].id == card) {
-	       gfx_driver = driver_list[c].driver;
-	       if (check_mode) {
-		  if (((require_window) && (!gfx_driver->windowed)) ||
-		      ((!require_window) && (gfx_driver->windowed))) {
-		     gfx_driver = NULL;
-		     continue;
-		  }
-	       }
-	       break;
-	    }
-	 }
-	 if (gfx_driver) {
-	    tried = TRUE;
-	    gfx_driver->name = gfx_driver->desc = get_config_text(gfx_driver->ascii_name);
-	    screen = gfx_driver->init(w, h, v_w, v_h, _color_depth);
+      card = get_config_id(uconvert_ascii("graphics", tmp), buf, 0);
+
+      if (card) {
+	 drv = get_gfx_driver_from_id(card, driver_list);
+
+	 if (drv && gfx_driver_is_valid(drv, flags)) {
+	    found = TRUE;
+	    screen = init_gfx_driver(drv, w, h, v_w, v_h);
+
 	    if (screen)
 	       break;
-	    else
-	       gfx_driver = NULL;
 	 }
       }
       else {
+	 /* Stop searching the gfx_card#n (n>0) family at the first missing member,
+	  * except gfx_card1 which could have been identified with gfx_card.
+	  */
 	 if (n > 1)
 	    break;
       }
    }
 
-   return tried;
+   return found;
 }
 
 
@@ -489,71 +598,59 @@ static int get_config_gfx_driver(char *gfx_card, int check_mode, int require_win
  */
 int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 {
-   static int allow_config = TRUE;
-   extern void blit_end(void);
-   _DRIVER_INFO *driver_list;
-   int tried = FALSE;
-   char buf[ALLEGRO_ERROR_SIZE], tmp[64];
-   int c;
-   int check_mode = FALSE, require_window = FALSE;
-
-
+   TRACE(PREFIX_I "Called set_gfx_mode(%d, %d, %d, %d, %d).\n",
+	 card, w, h, v_w, v_h);
+         
+   /* TODO: shouldn't this be incremented only IF successful? */
    _gfx_mode_set_count++;
 
-   /* check specific fullscreen/windowed mode detection */
-   if ((card == GFX_AUTODETECT_FULLSCREEN) ||
-       (card == GFX_AUTODETECT_WINDOWED)) {
-      check_mode = TRUE;
-      require_window = (card == GFX_AUTODETECT_WINDOWED ? TRUE : FALSE);
-      card = GFX_AUTODETECT;
-   }
-
    /* special bodge for the GFX_SAFE driver */
-   if (card == GFX_SAFE) {
-      _safe_gfx_mode_change = 1;
-      ustrzcpy(buf, sizeof(buf), allegro_error);
-      if (set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0) != 0) {
-	 set_color_depth(8);
-	 allow_config = FALSE;
-	 _safe_gfx_mode_change = 0;
-      #ifdef GFX_SAFE_ID
-	 if (set_gfx_mode(GFX_SAFE_ID, GFX_SAFE_W, GFX_SAFE_H, 0, 0) != 0) {
-      #else
-	 if (set_gfx_mode(GFX_AUTODETECT, 0, 0, 0, 0) != 0) {
-      #endif
-	    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	    allegro_message(uconvert_ascii("%s\n", tmp), get_config_text("Fatal error: unable to set GFX_SAFE"));
-	    return -1;
-	 }
-	 allow_config = TRUE;
-      }
+   if (card == GFX_SAFE)
+      return _set_gfx_mode_safe(card, w, h, v_w, v_h);
+   else
+      return _set_gfx_mode(card, w, h, v_w, v_h, TRUE);
+}
 
-      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, buf);
-      _safe_gfx_mode_change = 0;
-      return 0;
-   }
+
+
+/* _set_gfx_mode:
+ *  Called by set_gfx_mode(). Separated to make a clear difference between
+ *  the virtual GFX_SAFE driver and the rest. The allow_config parameter,
+ *  if true, allows the configuration to override the graphics card/driver
+ *  when using GFX_AUTODETECT.
+ */
+static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_config)
+{
+   _DRIVER_INFO *driver_list;
+   GFX_DRIVER *drv;
+   char tmp1[64], tmp2[64];
+   AL_CONST char *dv;
+   int flags = 0;
+   int c;
+   ASSERT(system_driver);
+   ASSERT(card != GFX_SAFE);
 
    /* remember the current console state */
    if (gfx_virgin) {
+      TRACE(PREFIX_I "First call, remembering console state.\n");
       LOCK_FUNCTION(_stub_bank_switch);
       LOCK_FUNCTION(blit);
 
       if (system_driver->save_console_state)
 	 system_driver->save_console_state();
 
-      _add_exit_func(shutdown_gfx);
+      _add_exit_func(shutdown_gfx, "shutdown_gfx");
 
       gfx_virgin = FALSE;
    }
 
-   if (system_driver->display_switch_lock)
-      system_driver->display_switch_lock(TRUE, TRUE);
-
    timer_simulate_retrace(FALSE);
    _screen_split_position = 0;
 
    /* close down any existing graphics driver */
    if (gfx_driver) {
+      TRACE(PREFIX_I "Closing graphics driver (%p) ", gfx_driver);
+      TRACE("%s.\n", gfx_driver->ascii_name);
       if (_al_linker_mouse)
          _al_linker_mouse->show_mouse(NULL);
 
@@ -601,85 +698,100 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 
    gfx_capabilities = 0;
 
-   _current_refresh_rate = 0;
+   _set_current_refresh_rate(0);
 
    /* return to text mode? */
    if (card == GFX_TEXT) {
+      TRACE(PREFIX_I "Closing, restoring original console state.\n");
       if (system_driver->restore_console_state)
 	 system_driver->restore_console_state();
 
       if (_gfx_bank) {
-	 free(_gfx_bank);
+	 _AL_FREE(_gfx_bank);
 	 _gfx_bank = NULL;
       }
 
-      if (system_driver->display_switch_lock)
-	 system_driver->display_switch_lock(FALSE, FALSE);
-
+      TRACE(PREFIX_I "Graphic mode closed.\n");
       return 0;
    }
 
+   /* now to the interesting part: let's try to find a graphics driver */
+   usetc(allegro_error, 0);
+
    /* ask the system driver for a list of graphics hardware drivers */
    if (system_driver->gfx_drivers)
       driver_list = system_driver->gfx_drivers();
    else
       driver_list = _gfx_driver_list;
 
-   usetc(allegro_error, 0);
+   /* filter specific fullscreen/windowed driver requests */
+   if (card == GFX_AUTODETECT_FULLSCREEN) {
+      flags |= GFX_DRIVER_FULLSCREEN_FLAG;
+      card = GFX_AUTODETECT;
+   }
+   else if (card == GFX_AUTODETECT_WINDOWED) {
+      flags |= GFX_DRIVER_WINDOWED_FLAG;
+      card = GFX_AUTODETECT;
+   }
 
-   /* try windowed mode drivers first if GFX_AUTODETECT_WINDOWED was selected */
-   if ((card == GFX_AUTODETECT) && (allow_config) && (require_window))
-      tried = get_config_gfx_driver(uconvert_ascii("gfx_cardw", tmp), check_mode, require_window, driver_list, card, w, h, v_w, v_h);
+   if (card == GFX_AUTODETECT) {
+      /* autodetect the driver */
+      int found = FALSE;
 
-   /* check the gfx_card config variable if gfx_cardw wasn't used */
-   if ((card == GFX_AUTODETECT) && (allow_config) && (!gfx_driver))
-      tried = get_config_gfx_driver(uconvert_ascii("gfx_card", tmp), check_mode, require_window, driver_list, card, w, h, v_w, v_h);
+      tmp1[0] = '\0';
 
-   if (!tried) {
-      /* search table for the requested driver */
-      for (c=0; driver_list[c].driver; c++) {
-	 if (driver_list[c].id == card) {
-	    gfx_driver = driver_list[c].driver;
-	    break;
-	 }
+      /* first try the config variables */
+      if (allow_config) {
+	 /* try the gfx_card variable if GFX_AUTODETECT or GFX_AUTODETECT_FULLSCREEN was selected */
+	 if (!(flags & GFX_DRIVER_WINDOWED_FLAG))
+	    found = get_config_gfx_driver(uconvert_ascii("gfx_card", tmp1), w, h, v_w, v_h, flags, driver_list);
+
+	 /* try the gfx_cardw variable if GFX_AUTODETECT or GFX_AUTODETECT_WINDOWED was selected */
+	 if (!(flags & GFX_DRIVER_FULLSCREEN_FLAG) && !found)
+	    found = get_config_gfx_driver(uconvert_ascii("gfx_cardw", tmp1), w, h, v_w, v_h, flags, driver_list);
       }
 
-      if (gfx_driver) {                            /* specific driver? */
-	 gfx_driver->name = gfx_driver->desc = get_config_text(gfx_driver->ascii_name);
-	 screen = gfx_driver->init(w, h, v_w, v_h, _color_depth);
-      }
-      else {                                      /* otherwise autodetect */
+      /* go through the list of autodetected drivers if none was previously found */
+      if (!found) {
+	 TRACE(PREFIX_I "Autodetecting graphic driver.\n");
 	 for (c=0; driver_list[c].driver; c++) {
 	    if (driver_list[c].autodetect) {
-	       gfx_driver = driver_list[c].driver;
-	       if (check_mode) {
-		  if (((require_window) && (!gfx_driver->windowed)) ||
-		      ((!require_window) && (gfx_driver->windowed)))
-		  continue;
+	       drv = driver_list[c].driver;
+
+	       if (gfx_driver_is_valid(drv, flags)) {
+		  screen = init_gfx_driver(drv, w, h, v_w, v_h);
+
+		  if (screen)
+		     break;
 	       }
-	       gfx_driver->name = gfx_driver->desc = get_config_text(gfx_driver->ascii_name);
-	       screen = gfx_driver->init(w, h, v_w, v_h, _color_depth);
-	       if (screen)
-		  break;
 	    }
 	 }
       }
+      else {
+	 TRACE(PREFIX_I "GFX_AUTODETECT overridden through configuration:"
+	       " %s.\n", tmp1);
+      }
+   }
+   else {
+      /* search the list for the requested driver */
+      drv = get_gfx_driver_from_id(card, driver_list);
+
+      if (drv)
+	 screen = init_gfx_driver(drv, w, h, v_w, v_h);
    }
 
-
+   /* gracefully handle failure */
    if (!screen) {
-      gfx_driver = NULL;
-      screen = NULL;
+      gfx_driver = NULL;  /* set by init_gfx_driver() */
 
       if (!ugetc(allegro_error))
 	 ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Unable to find a suitable graphics driver"));
 
-      if (system_driver->display_switch_lock)
-	 system_driver->display_switch_lock(FALSE, FALSE);
-
+      TRACE(PREFIX_E "Failed setting graphic driver %d.\n", card);
       return -1;
    }
 
+   /* set the basic capabilities of the driver */
    if ((VIRTUAL_W > SCREEN_W) || (VIRTUAL_H > SCREEN_H)) {
       if (gfx_driver->scroll)
 	 gfx_capabilities |= GFX_CAN_SCROLL;
@@ -688,6 +800,23 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 	 gfx_capabilities |= GFX_CAN_TRIPLE_BUFFER;
    }
 
+   /* check whether we are instructed to disable vsync */
+   dv = get_config_string(uconvert_ascii("graphics", tmp1),
+                          uconvert_ascii("disable_vsync", tmp2),
+                          NULL);
+
+   if ((dv) && ((c = ugetc(dv)) != 0) && ((c == 'y') || (c == 'Y') || (c == '1')))
+      _wait_for_vsync = FALSE;
+   else
+      _wait_for_vsync = TRUE;
+
+   TRACE(PREFIX_I "The driver %s wait for vsync.\n",
+	 (_wait_for_vsync) ? "will" : "won't");
+
+   /* Give the gfx driver an opportunity to set the drawing mode */
+   if ((gfx_driver->drawing_mode) && (!_dispsw_status))
+      gfx_driver->drawing_mode();
+
    clear_bitmap(screen);
 
    /* set up the default colors */
@@ -714,14 +843,76 @@ int set_gfx_mode(int card, int w, int h, int v_w, int v_h)
 
    _register_switch_bitmap(screen, NULL);
 
-   if (system_driver->display_switch_lock)
-      system_driver->display_switch_lock(FALSE, FALSE);
-
+   TRACE(PREFIX_I "set_gfx_card success for %dx%dx%d.\n",
+	 screen->w, screen->h, bitmap_color_depth(screen));
    return 0;
 } 
 
 
 
+/* _set_gfx_mode_safe:
+ *  Special wrapper used when the card parameter of set_gfx_mode()
+ *  is GFX_SAFE. In this case the function tries to query the
+ *  system driver for a "safe" resolution+driver it knows it will
+ *  work, and set it. If the system driver cannot get a "safe"
+ *  resolution+driver, it will try the given parameters.
+ */
+static int _set_gfx_mode_safe(int card, int w, int h, int v_w, int v_h)
+{
+   char buf[ALLEGRO_ERROR_SIZE], tmp1[64];
+   struct GFX_MODE mode;
+   int ret, driver;
+
+   ASSERT(card == GFX_SAFE);
+   ASSERT(system_driver);
+   TRACE(PREFIX_I "Trying to set a safe graphics mode.\n");
+   
+   if (system_driver->get_gfx_safe_mode) {
+      ustrzcpy(buf, sizeof(buf), allegro_error);
+
+      /* retrieve the safe graphics mode */
+      system_driver->get_gfx_safe_mode(&driver, &mode);
+      TRACE(PREFIX_I "The system driver suggests %dx%dx%d\n",
+	    mode.width, mode.height, mode.bpp);
+
+      /* try using the specified resolution but current depth */
+      if (_set_gfx_mode(driver, w, h, 0, 0, TRUE) == 0)
+         return 0;
+
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, buf);
+
+      /* finally use the safe settings */
+      set_color_depth(mode.bpp);
+      if (_set_gfx_mode(driver, mode.width, mode.height, 0, 0, TRUE) == 0)
+         return 0;
+
+      ASSERT(FALSE);  /* the safe graphics mode must always work */
+   }
+   else {
+      TRACE(PREFIX_W "The system driver was unable to get a safe mode, "
+	    "I'll try with the specified parameters...\n");
+      /* no safe graphics mode, try hard-coded autodetected modes with
+       * custom settings */
+      _safe_gfx_mode_change = 1;
+
+      ret = _set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0, TRUE);
+
+      _safe_gfx_mode_change = 0;
+
+      if (ret == 0)
+         return 0;
+   }
+
+   /* failing to set GFX_SAFE is a fatal error */
+   TRACE(PREFIX_E "Bad bad, not even GFX_SAFE works?\n");
+   _set_gfx_mode(GFX_TEXT, 0, 0, 0, 0, TRUE);
+   allegro_message(uconvert_ascii("%s\n", tmp1),
+		   get_config_text("Fatal error: unable to set GFX_SAFE"));
+   return -1;
+}
+
+
+
 /* _sort_out_virtual_width:
  *  Decides how wide the virtual screen really needs to be. That is more 
  *  complicated than it sounds, because the Allegro graphics primitives 
@@ -760,7 +951,7 @@ void _sort_out_virtual_width(int *width, GFX_DRIVER *driver)
  *  switching table using bank size/granularity information from the 
  *  specified graphics driver.
  */
-BITMAP *_make_bitmap(int w, int h, unsigned long addr, GFX_DRIVER *driver, int color_depth, int bpl)
+BITMAP *_make_bitmap(int w, int h, uintptr_t addr, GFX_DRIVER *driver, int color_depth, int bpl)
 {
    GFX_VTABLE *vtable = _get_vtable(color_depth);
    int i, bank, size;
@@ -771,13 +962,13 @@ BITMAP *_make_bitmap(int w, int h, unsigned long addr, GFX_DRIVER *driver, int c
 
    size = sizeof(BITMAP) + sizeof(char *) * h;
 
-   b = (BITMAP *)malloc(size);
+   b = (BITMAP *)_AL_MALLOC(size);
    if (!b)
       return NULL;
 
-   _gfx_bank = realloc(_gfx_bank, h * sizeof(int));
+   _gfx_bank = _AL_REALLOC(_gfx_bank, h * sizeof(int));
    if (!_gfx_bank) {
-      free(b);
+      _AL_FREE(b);
       return NULL;
    }
 
@@ -840,8 +1031,14 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
 {
    GFX_VTABLE *vtable;
    BITMAP *bitmap;
+   int nr_pointers;
+   int padding;
    int i;
 
+   ASSERT(width >= 0);
+   ASSERT(height > 0);
+   ASSERT(system_driver);
+
    if (system_driver->create_bitmap)
       return system_driver->create_bitmap(color_depth, width, height);
 
@@ -849,13 +1046,23 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
    if (!vtable)
       return NULL;
 
-   bitmap = malloc(sizeof(BITMAP) + (sizeof(char *) * height));
+   /* We need at least two pointers when drawing, otherwise we get crashes with
+    * Electric Fence.  We think some of the assembly code assumes a second line
+    * pointer is always available.
+    */
+   nr_pointers = MAX(2, height);
+   bitmap = _AL_MALLOC(sizeof(BITMAP) + (sizeof(char *) * nr_pointers));
    if (!bitmap)
       return NULL;
 
-   bitmap->dat = malloc(width * height * BYTES_PER_PIXEL(color_depth));
+   /* This avoids a crash for assembler code accessing the last pixel, as it
+    * read 4 bytes instead of 3.
+    */
+   padding = (color_depth == 24) ? 1 : 0;
+
+   bitmap->dat = _AL_MALLOC_ATOMIC(width * height * BYTES_PER_PIXEL(color_depth) + padding);
    if (!bitmap->dat) {
-      free(bitmap);
+      _AL_FREE(bitmap);
       return NULL;
    }
 
@@ -871,9 +1078,11 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
    bitmap->y_ofs = 0;
    bitmap->seg = _default_ds();
 
-   bitmap->line[0] = bitmap->dat;
-   for (i=1; i<height; i++)
-      bitmap->line[i] = bitmap->line[i-1] + width * BYTES_PER_PIXEL(color_depth);
+   if (height > 0) {
+      bitmap->line[0] = bitmap->dat;
+      for (i=1; i<height; i++)
+         bitmap->line[i] = bitmap->line[i-1] + width * BYTES_PER_PIXEL(color_depth);
+   }
 
    if (system_driver->created_bitmap)
       system_driver->created_bitmap(bitmap);
@@ -888,6 +1097,8 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
  */
 BITMAP *create_bitmap(int width, int height)
 {
+   ASSERT(width >= 0);
+   ASSERT(height > 0);
    return create_bitmap_ex(_color_depth, width, height);
 }
 
@@ -904,18 +1115,13 @@ BITMAP *create_bitmap(int width, int height)
 BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
 {
    BITMAP *bitmap;
+   int nr_pointers;
    int i;
 
+   ASSERT(parent);
+   ASSERT((x >= 0) && (y >= 0) && (x < parent->w) && (y < parent->h));
    ASSERT((width > 0) && (height > 0));
-
-   if (!parent) 
-      return NULL;
-
-   if (x < 0) 
-      x = 0; 
-
-   if (y < 0) 
-      y = 0;
+   ASSERT(system_driver);
 
    if (x+width > parent->w) 
       width = parent->w-x;
@@ -930,7 +1136,9 @@ BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
       return system_driver->create_sub_bitmap(parent, x, y, width, height);
 
    /* get memory for structure and line pointers */
-   bitmap = malloc(sizeof(BITMAP) + (sizeof(char *) * height));
+   /* (see create_bitmap for the reason we need at least two) */
+   nr_pointers = MAX(2, height);
+   bitmap = _AL_MALLOC(sizeof(BITMAP) + (sizeof(char *) * nr_pointers));
    if (!bitmap)
       return NULL;
 
@@ -993,60 +1201,83 @@ BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
 
 
 
-/* try_vram_location:
- *  Attempts to create a video memory bitmap out of the specified region
- *  of the larger screen surface, returning a pointer to it if that area
- *  is free.
+/* add_vram_block:
+ *  Creates a video memory bitmap out of the specified region
+ *  of the larger screen surface, returning a pointer to it.
  */
-static BITMAP *try_vram_location(int x, int y, int w, int h)
+static BITMAP *add_vram_block(int x, int y, int w, int h)
 {
-   VRAM_BITMAP *block = vram_bitmap_list;
+   VRAM_BITMAP *b, *new_b;
+   VRAM_BITMAP **last_p;
 
-   if ((x<0) || (y<0) || (x+w > VIRTUAL_W) || (y+h > VIRTUAL_H))
+   new_b = _AL_MALLOC(sizeof(VRAM_BITMAP));
+   if (!new_b)
       return NULL;
 
-   while (block) {
-      if (((x < block->x+block->w) && (x+w > block->x)) &&
-	  ((y < block->y+block->h) && (y+h > block->y)))
-	 return NULL;
+   new_b->x = x;
+   new_b->y = y;
+   new_b->w = w;
+   new_b->h = h;
 
-      block = block->next;
-   }
-
-   block = malloc(sizeof(VRAM_BITMAP));
-   if (!block)
-      return NULL;
-
-   block->x = x;
-   block->y = y;
-   block->w = w;
-   block->h = h;
-
-   block->bmp = create_sub_bitmap(screen, x, y, w, h);
-   if (!(block->bmp)) {
-      free(block);
+   new_b->bmp = create_sub_bitmap(screen, x, y, w, h);
+   if (!new_b->bmp) {
+      _AL_FREE(new_b);
       return NULL;
    }
 
-   block->next = vram_bitmap_list;
-   vram_bitmap_list = block;
+   /* find sorted y-position */
+   last_p = &vram_bitmap_list;
+   for (b = vram_bitmap_list; b && (b->y < new_b->y); b = b->next_y)
+      last_p = &b->next_y;
 
-   return block->bmp;
+   /* insert */
+   *last_p = new_b;
+   new_b->next_y = b;
+
+   return new_b->bmp;
 }
 
 
 
 /* create_video_bitmap:
  *  Attempts to make a bitmap object for accessing offscreen video memory.
+ *
+ *  The algorithm is similar to algorithms for drawing polygons. Think of
+ *  a horizontal stripe whose height is equal to that of the new bitmap.
+ *  It is initially aligned to the top of video memory and then it moves
+ *  downwards, stopping each time its top coincides with the bottom of a
+ *  video bitmap. For each stop, create a list of video bitmaps intersecting
+ *  the stripe, sorted by the left coordinate, from left to right, in the
+ *  next_x linked list. We look through this list and stop as soon as the
+ *  gap between the rightmost right edge seen so far and the current left
+ *  edge is big enough for the new video bitmap. In that case, we are done.
+ *  If we don't find such a gap, we move the stripe further down.
+ *
+ *  To make it efficient to find new bitmaps intersecting the stripe, the
+ *  list of all bitmaps is always sorted by top, from top to bottom, in
+ *  the next_y linked list. The list of bitmaps intersecting the stripe is
+ *  merely updated, not recalculated from scratch, when we move the stripe.
+ *  So every bitmap gets bubbled to its correct sorted x-position at most
+ *  once. Bubbling a bitmap takes at most as many steps as the number of
+ *  video bitmaps. So the algorithm behaves at most quadratically with
+ *  regard to the number of video bitmaps. I think that a linear behaviour
+ *  is more typical in practical applications (this happens, for instance,
+ *  if you allocate many bitmaps of the same height).
+ *
+ *  There's one more optimization: if a call fails, we cache the size of the
+ *  requested bitmap, so that next time we can detect very quickly that this
+ *  size is too large (this needs to be maintained when destroying bitmaps).
  */
 BITMAP *create_video_bitmap(int width, int height)
 {
-   VRAM_BITMAP origin_vram_bitmap;
-   VRAM_BITMAP *blockx;
-   VRAM_BITMAP *blocky;
-   VRAM_BITMAP *block;
+   VRAM_BITMAP *active_list, *b, *vram_bitmap;
+   VRAM_BITMAP **last_p;
    BITMAP *bmp;
+   int x = 0, y = 0;
 
+   ASSERT(width >= 0);
+   ASSERT(height > 0);
+   
    if (_dispsw_status)
       return NULL;
 
@@ -1056,32 +1287,111 @@ BITMAP *create_video_bitmap(int width, int height)
       if (!bmp)
 	 return NULL;
 
-      block = malloc(sizeof(VRAM_BITMAP));
-      block->x = -1;
-      block->y = -1;
-      block->w = 0;
-      block->h = 0;
-      block->bmp = bmp;
-      block->next = vram_bitmap_list;
-      vram_bitmap_list = block;
+      b = _AL_MALLOC(sizeof(VRAM_BITMAP));
+      b->x = -1;
+      b->y = -1;
+      b->w = 0;
+      b->h = 0;
+      b->bmp = bmp;
+      b->next_y = vram_bitmap_list;
+      vram_bitmap_list = b;
 
       return bmp;
    }
 
-   /* otherwise fall back on subdividing the normal screen surface:
-    *  it is sufficient to test the nodes of the grid made up of the X-axis, the Y-axis
-    *  and the straight lines carrying the right edge or the bottom edge of the bitmaps
-    */
-   memset(&origin_vram_bitmap, 0, sizeof(VRAM_BITMAP));  /* brings X-axis and Y-axis */
-   origin_vram_bitmap.next = vram_bitmap_list;
+   /* check bad args */
+   if ((width > VIRTUAL_W) || (height > VIRTUAL_H) ||
+       (width < 0) || (height < 0))
+      return NULL;
 
-   /* do the search in O(n^3) time */
-   for (blocky = &origin_vram_bitmap; blocky; blocky = blocky->next)
-      for (blockx = &origin_vram_bitmap; blockx; blockx = blockx->next)
-	 if ((bmp = try_vram_location((blockx->x+blockx->w+15)&~15, blocky->y+blocky->h, width, height)) != NULL)
-	    return bmp;
+   /* check cached bitmap size */
+   if ((width >= failed_bitmap_w) && (height >= failed_bitmap_h))
+      return NULL;
 
-   return NULL;
+   vram_bitmap = vram_bitmap_list;
+   active_list = NULL;
+   y = 0;
+
+   while (TRUE) {
+      /* Move the blocks from the VRAM_BITMAP_LIST that intersect the
+       * stripe to the ACTIVE_LIST: look through the VRAM_BITMAP_LIST
+       * following the next_y link, grow the ACTIVE_LIST following
+       * the next_x link and sorting by x-position.
+       * (this loop runs in amortized quadratic time)
+       */
+      while (vram_bitmap && (vram_bitmap->y < y+height)) {
+	 /* find sorted x-position */
+	 last_p = &active_list;
+	 for (b = active_list; b && (vram_bitmap->x > b->x); b = b->next_x)
+	    last_p = &b->next_x;
+
+	 /* insert */
+	 *last_p = vram_bitmap;
+	 vram_bitmap->next_x = b;
+
+	 /* next video bitmap */
+	 vram_bitmap = vram_bitmap->next_y;
+      }
+
+      x = 0;
+
+      /* Look for holes to put our bitmap in.
+       * (this loop runs in quadratic time)
+       */
+      for (b = active_list; b; b = b->next_x) {
+	 if (x+width <= b->x)  /* hole is big enough? */
+	    return add_vram_block(x, y, width, height);
+
+         /* Move search x-position to the right edge of the
+          * skipped bitmap if we are not already past it.
+          * And check there is enough room before continuing.
+          */
+	 if (x < b->x + b->w) {
+	    x = (b->x + b->w + 15) & ~15;
+	    if (x+width > VIRTUAL_W)
+	       break;
+	 }
+      }
+
+      /* If the whole ACTIVE_LIST was scanned, there is a big
+       * enough hole to the right of the rightmost bitmap.
+       */
+      if (b == NULL)
+	 return add_vram_block(x, y, width, height);
+
+      /* Move search y-position to the topmost bottom edge
+       * of the bitmaps intersecting the stripe.
+       * (this loop runs in quadratic time)
+       */
+      y = active_list->y + active_list->h;
+      for (b = active_list->next_x; b; b = b->next_x) {
+	 if (y > b->y + b->h)
+	    y = b->y + b->h;
+      }
+
+      if (y+height > VIRTUAL_H) {  /* too close to bottom? */
+	 /* Before failing, cache this bitmap size so that later calls can
+	  * learn from it. Use area as a measure to sort the bitmap sizes.
+          */
+	 if (width * height < failed_bitmap_w * failed_bitmap_h) {
+	    failed_bitmap_w = width;
+	    failed_bitmap_h = height;
+	 }
+
+	 return NULL;
+      }
+
+      /* Remove the blocks that don't intersect the new stripe from ACTIVE_LIST.
+       * (this loop runs in quadratic time)
+       */
+      last_p = &active_list;
+      for (b = active_list; b; b = b->next_x) {
+	 if (y >= b->y + b->h)
+	    *last_p = b->next_x;
+	 else
+	    last_p = &b->next_x;
+      }
+   }
 }
 
 
@@ -1093,13 +1403,14 @@ BITMAP *create_system_bitmap(int width, int height)
 {
    BITMAP *bmp;
 
+   ASSERT(width >= 0);
+   ASSERT(height > 0);
    ASSERT(gfx_driver != NULL);
 
    if (gfx_driver->create_system_bitmap)
       return gfx_driver->create_system_bitmap(width, height);
 
    bmp = create_bitmap(width, height);
-   bmp->id |= BMP_ID_SYSTEM;
 
    return bmp;
 }
@@ -1124,23 +1435,35 @@ void destroy_bitmap(BITMAP *bitmap)
 	 while (pos) {
 	    if (pos->bmp == bitmap) {
 	       if (prev)
-		  prev->next = pos->next;
+		  prev->next_y = pos->next_y;
 	       else
-		  vram_bitmap_list = pos->next;
+		  vram_bitmap_list = pos->next_y;
 
 	       if (pos->x < 0) {
 		  /* the driver is in charge of this object */
 		  gfx_driver->destroy_video_bitmap(bitmap);
-		  free(pos);
+		  _AL_FREE(pos);
 		  return;
 	       } 
 
-	       free(pos);
+	       /* Update cached bitmap size using worst case scenario:
+		* the bitmap lies between two holes whose size is the cached
+		* size on each axis respectively.
+		*/
+	       failed_bitmap_w = failed_bitmap_w * 2 + ((bitmap->w + 15) & ~15);
+	       if (failed_bitmap_w > BMP_MAX_SIZE)
+		  failed_bitmap_w = BMP_MAX_SIZE;
+
+	       failed_bitmap_h = failed_bitmap_h * 2 + bitmap->h;
+	       if (failed_bitmap_h > BMP_MAX_SIZE)
+		  failed_bitmap_h = BMP_MAX_SIZE;
+
+	       _AL_FREE(pos);
 	       break;
 	    }
 
 	    prev = pos;
-	    pos = pos->next;
+	    pos = pos->next_y;
 	 }
 
 	 _unregister_switch_bitmap(bitmap);
@@ -1162,14 +1485,61 @@ void destroy_bitmap(BITMAP *bitmap)
       }
 
       if (bitmap->dat)
-	 free(bitmap->dat);
+	 _AL_FREE(bitmap->dat);
 
-      free(bitmap);
+      _AL_FREE(bitmap);
    }
 }
 
 
 
+/* set_clip_rect:
+ *  Sets the two opposite corners of the clipping rectangle to be used when
+ *  drawing to the bitmap. Nothing will be drawn to positions outside of this 
+ *  rectangle. When a new bitmap is created the clipping rectangle will be 
+ *  set to the full area of the bitmap.
+ */
+void set_clip_rect(BITMAP *bitmap, int x1, int y1, int x2, int y2)
+{
+   ASSERT(bitmap);
+
+   /* internal clipping is inclusive-exclusive */
+   x2++;
+   y2++;
+
+   bitmap->cl = CLAMP(0, x1, bitmap->w-1);
+   bitmap->ct = CLAMP(0, y1, bitmap->h-1);
+   bitmap->cr = CLAMP(0, x2, bitmap->w);
+   bitmap->cb = CLAMP(0, y2, bitmap->h);
+
+   if (bitmap->vtable->set_clip)
+      bitmap->vtable->set_clip(bitmap);
+}
+
+
+
+/* add_clip_rect:
+ *  Makes the new clipping rectangle the intersection between the given
+ *  rectangle and the current one.
+ */
+void add_clip_rect(BITMAP *bitmap, int x1, int y1, int x2, int y2)
+{
+   int cx1, cy1, cx2, cy2;
+
+   ASSERT(bitmap);
+
+   get_clip_rect(bitmap, &cx1, &cy1, &cx2, &cy2);
+
+   x1 = MAX(x1, cx1);
+   y1 = MAX(y1, cy1);
+   x2 = MIN(x2, cx2);
+   y2 = MIN(y2, cy2);
+
+   set_clip_rect(bitmap, x1, y1, x2, y2);
+}
+
+
+
 /* set_clip:
  *  Sets the two opposite corners of the clipping rectangle to be used when
  *  drawing to the bitmap. Nothing will be drawn to positions outside of this 
@@ -1186,14 +1556,8 @@ void set_clip(BITMAP *bitmap, int x1, int y1, int x2, int y2)
    ASSERT(bitmap);
 
    if ((!x1) && (!y1) && (!x2) && (!y2)) {
-      bitmap->clip = FALSE;
-      bitmap->cl = bitmap->ct = 0;
-      bitmap->cr = bitmap->w;
-      bitmap->cb = bitmap->h;
-
-      if (bitmap->vtable->set_clip)
-	 bitmap->vtable->set_clip(bitmap);
-
+      set_clip_rect(bitmap, 0, 0, bitmap->w-1, bitmap->h-1);
+      set_clip_state(bitmap, FALSE);
       return;
    }
 
@@ -1209,17 +1573,8 @@ void set_clip(BITMAP *bitmap, int x1, int y1, int x2, int y2)
       y2 = t;
    }
 
-   x2++;
-   y2++;
-
-   bitmap->clip = TRUE;
-   bitmap->cl = MID(0, x1, bitmap->w-1);
-   bitmap->ct = MID(0, y1, bitmap->h-1);
-   bitmap->cr = MID(0, x2, bitmap->w);
-   bitmap->cb = MID(0, y2, bitmap->h);
-
-   if (bitmap->vtable->set_clip)
-      bitmap->vtable->set_clip(bitmap);
+   set_clip_rect(bitmap, x1, y1, x2, y2);
+   set_clip_state(bitmap, TRUE);
 }
 
 
@@ -1321,7 +1676,7 @@ int request_scroll(int x, int y)
 /* poll_scroll:
  *  Checks whether a requested triple buffer flip has actually taken place.
  */
-int poll_scroll()
+int poll_scroll(void)
 {
    if ((!gfx_driver->poll_scroll) || (_dispsw_status))
       return FALSE;
@@ -1373,7 +1728,7 @@ int request_video_bitmap(BITMAP *bitmap)
  *  Asks a driver to turn on triple buffering mode, if it is capable
  *  of that.
  */
-int enable_triple_buffer()
+int enable_triple_buffer(void)
 {
    if (gfx_capabilities & GFX_CAN_TRIPLE_BUFFER)
       return 0;
@@ -1394,3 +1749,79 @@ int enable_triple_buffer()
 }
 
 
+
+/* get_gfx_mode_type:
+ *  Evaluates the type of the graphics driver card
+ *  and tells you whether it is a windowed, fullscreen,
+ *  definitely windowed or fullscreen, and/or a magic driver.
+ *  See gfx.h for the GFX_TYPE_* defines.
+ */
+int get_gfx_mode_type(int graphics_card)
+{
+   int gfx_type = GFX_TYPE_UNKNOWN;
+   _DRIVER_INFO *gfx_driver_info;
+   GFX_DRIVER *gfx_driver_entry;
+
+   ASSERT(system_driver);
+
+   /* Ask the system driver for a list of graphics hardware drivers */
+   if (system_driver->gfx_drivers) {
+      gfx_driver_info = system_driver->gfx_drivers();
+   }
+   else {
+      gfx_driver_info = _gfx_driver_list;
+   }
+
+   ASSERT(gfx_driver_info);
+
+   while (gfx_driver_info->driver) {
+      if (gfx_driver_info->id == graphics_card) {
+         gfx_driver_entry = (GFX_DRIVER*)gfx_driver_info->driver;
+         if (gfx_driver_entry->windowed) {
+            gfx_type |= (GFX_TYPE_WINDOWED   | GFX_TYPE_DEFINITE);
+         }
+         else {
+            gfx_type |= (GFX_TYPE_FULLSCREEN | GFX_TYPE_DEFINITE);
+         }
+         break;
+      }
+      gfx_driver_info++;
+   }
+
+   switch (graphics_card) {
+      case GFX_AUTODETECT:
+         gfx_type |= GFX_TYPE_MAGIC;
+         break;
+      case GFX_AUTODETECT_FULLSCREEN:
+         gfx_type |= (GFX_TYPE_MAGIC | GFX_TYPE_FULLSCREEN | GFX_TYPE_DEFINITE);
+         break;
+      case GFX_AUTODETECT_WINDOWED:
+         gfx_type |= (GFX_TYPE_MAGIC | GFX_TYPE_WINDOWED   | GFX_TYPE_DEFINITE);
+         break;
+      case GFX_SAFE:
+         gfx_type |= GFX_TYPE_MAGIC;
+         break;
+      case GFX_TEXT:
+         gfx_type |= GFX_TYPE_MAGIC;
+         break;
+   }
+   return gfx_type;
+}
+
+
+
+/* get_gfx_mode:
+ *  Tells you which graphics mode is currently set.
+ *  Useful for determining the actual driver that
+ *  GFX_AUTODETECT* or GFX_SAFE set successfully.
+ */
+int get_gfx_mode(void)
+{
+   if (!gfx_driver)
+      return GFX_NONE;
+
+   return gfx_driver->id;
+}
+
+
+
