@@ -28,18 +28,22 @@
 #include "allegro/internal/aintern.h"
 
 
+#ifdef ALLEGRO_WINDOWS
+/* exported address of d_clear_proc */
+int (*_d_clear_proc)(int, DIALOG *, int) = NULL;
+#endif
 
 /* typedef for the listbox callback functions */
 typedef char *(*getfuncptr)(int, int *);
 
 
 
-/* gui_textout:
+/* gui_textout_ex:
  *  Wrapper function for drawing text to the screen, which interprets the
  *  & character as an underbar for displaying keyboard shortcuts. Returns
  *  the width of the output string in pixels.
  */
-int gui_textout(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int centre)
+int gui_textout_ex(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int bg, int centre)
 {
    char tmp[1024];
    int hline_pos = -1;
@@ -47,6 +51,7 @@ int gui_textout(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int cent
    int in_pos = 0;
    int out_pos = 0;
    int pix_len, c;
+   ASSERT(s);
 
    while (((c = ugetc(s+in_pos)) != 0) && (out_pos<(int)(sizeof(tmp)-ucwidth(0)))) {
       if (c == '&') {
@@ -74,7 +79,7 @@ int gui_textout(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int cent
       x -= pix_len / 2;
 
    if (bmp) {
-      textout(bmp, font, tmp, x, y, color);
+      textout_ex(bmp, font, tmp, x, y, color, bg);
 
       if (hline_pos >= 0) {
 	 c = ugetat(tmp, hline_pos);
@@ -97,7 +102,8 @@ int gui_textout(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int cent
  */
 int gui_strlen(AL_CONST char *s)
 {
-   return gui_textout(NULL, s, 0, 0, 0, 0);
+   ASSERT(s);
+   return gui_textout_ex(NULL, s, 0, 0, 0, 0, 0);
 }
 
 
@@ -107,18 +113,19 @@ int gui_strlen(AL_CONST char *s)
  */
 static void dotted_rect(int x1, int y1, int x2, int y2, int fg, int bg)
 {
+   BITMAP *gui_bmp = gui_get_screen();
    int x = ((x1+y1) & 1) ? 1 : 0;
    int c;
 
    /* two loops to avoid bank switches */
    for (c=x1; c<=x2; c++)
-      putpixel(screen, c, y1, (((c+y1) & 1) == x) ? fg : bg);
+      putpixel(gui_bmp, c, y1, (((c+y1) & 1) == x) ? fg : bg);
    for (c=x1; c<=x2; c++)
-      putpixel(screen, c, y2, (((c+y2) & 1) == x) ? fg : bg);
+      putpixel(gui_bmp, c, y2, (((c+y2) & 1) == x) ? fg : bg);
 
    for (c=y1+1; c<y2; c++) {
-      putpixel(screen, x1, c, (((c+x1) & 1) == x) ? fg : bg);
-      putpixel(screen, x2, c, (((c+x2) & 1) == x) ? fg : bg);
+      putpixel(gui_bmp, x1, c, (((c+x1) & 1) == x) ? fg : bg);
+      putpixel(gui_bmp, x2, c, (((c+x2) & 1) == x) ? fg : bg);
    }
 }
 
@@ -131,7 +138,7 @@ static void dotted_rect(int x1, int y1, int x2, int y2, int fg, int bg)
 int d_yield_proc(int msg, DIALOG *d, int c)
 {
    if (msg == MSG_IDLE)
-      yield_timeslice();
+      rest(1);
 
    return D_O_K;
 }
@@ -144,9 +151,29 @@ int d_yield_proc(int msg, DIALOG *d, int c)
  */
 int d_clear_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
+
+#ifdef ALLEGRO_WINDOWS
+   /* kludge to get the exported address of d_clear_proc */
+   if (!_d_clear_proc)
+      _d_clear_proc = d->proc;
+#endif
+
    if (msg == MSG_DRAW) {
-      set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
-      clear_to_color(screen, d->bg);
+      BITMAP *gui_bmp = gui_get_screen();
+      int w, h;
+    
+      /* Get width and height of target bitmap. We can't use SCREEN_W and
+       * SCREEN_H because the target might not be the screen, but we cannot use
+       * bmp->w and bmp->h either because if it is the screen these are actually
+       * wrong. Ugh!
+       */
+      w = (gui_bmp == screen) ? SCREEN_W: gui_bmp->w;
+      h = (gui_bmp == screen) ? SCREEN_H: gui_bmp->h;
+
+      set_clip_rect(gui_bmp, 0, 0, w-1, h-1);
+      set_clip_state(gui_bmp, TRUE);
+      clear_to_color(gui_bmp, d->bg);
    }
 
    return D_O_K;
@@ -159,10 +186,13 @@ int d_clear_proc(int msg, DIALOG *d, int c)
  */
 int d_box_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
-      rectfill(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, d->bg);
-      rect(screen, d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg);
+      BITMAP *gui_bmp = gui_get_screen();
+
+      rectfill(gui_bmp, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, d->bg);
+      rect(gui_bmp, d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg);
    }
 
    return D_O_K;
@@ -175,13 +205,16 @@ int d_box_proc(int msg, DIALOG *d, int c)
  */
 int d_shadow_box_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       int black = makecol(0,0,0);
-      rectfill(screen, d->x+1, d->y+1, d->x+d->w-3, d->y+d->h-3, d->bg);
-      rect(screen, d->x, d->y, d->x+d->w-2, d->y+d->h-2, fg);
-      vline(screen, d->x+d->w-1, d->y+1, d->y+d->h-1, black);
-      hline(screen, d->x+1, d->y+d->h-1, d->x+d->w-1, black);
+      BITMAP *gui_bmp = gui_get_screen();
+
+      rectfill(gui_bmp, d->x+1, d->y+1, d->x+d->w-3, d->y+d->h-3, d->bg);
+      rect(gui_bmp, d->x, d->y, d->x+d->w-2, d->y+d->h-2, fg);
+      vline(gui_bmp, d->x+d->w-1, d->y+1, d->y+d->h-1, black);
+      hline(gui_bmp, d->x+1, d->y+d->h-1, d->x+d->w-1, black);
    }
 
    return D_O_K;
@@ -194,10 +227,12 @@ int d_shadow_box_proc(int msg, DIALOG *d, int c)
  */
 int d_bitmap_proc(int msg, DIALOG *d, int c)
 {
-   BITMAP *b = (BITMAP *)d->dp;
+   BITMAP *b;
+   ASSERT(d);
 
+   b = (BITMAP *)d->dp;
    if (msg==MSG_DRAW)
-      blit(b, screen, 0, 0, d->x, d->y, d->w, d->h);
+      blit(b, gui_get_screen(), 0, 0, d->x, d->y, d->w, d->h);
 
    return D_O_K;
 }
@@ -209,17 +244,15 @@ int d_bitmap_proc(int msg, DIALOG *d, int c)
  */
 int d_text_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
-      int rtm;
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
 
       if (d->dp2)
 	 font = d->dp2;
 
-      rtm = text_mode(d->bg);
-      gui_textout(screen, d->dp, d->x, d->y, fg, FALSE);
-      text_mode(rtm);
+      gui_textout_ex(gui_get_screen(), d->dp, d->x, d->y, fg, d->bg, FALSE);
 
       font = oldfont;
    }
@@ -235,17 +268,15 @@ int d_text_proc(int msg, DIALOG *d, int c)
  */
 int d_ctext_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
-      int rtm;
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
 
       if (d->dp2)
 	 font = d->dp2;
 
-      rtm = text_mode(d->bg);
-      gui_textout(screen, d->dp, d->x, d->y, fg, TRUE);
-      text_mode(rtm);
+      gui_textout_ex(gui_get_screen(), d->dp, d->x + d->w/2, d->y, fg, d->bg, TRUE);
 
       font = oldfont;
    }
@@ -261,17 +292,15 @@ int d_ctext_proc(int msg, DIALOG *d, int c)
  */
 int d_rtext_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
-      int rtm;
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
 
       if (d->dp2)
 	 font = d->dp2;
 
-      rtm = text_mode(d->bg);
-      gui_textout(screen, d->dp, d->x + d->w - gui_strlen(d->dp), d->y, fg, FALSE);
-      text_mode(rtm);
+      gui_textout_ex(gui_get_screen(), d->dp, d->x + d->w - gui_strlen(d->dp), d->y, fg, d->bg, FALSE);
 
       font = oldfont;
    }
@@ -289,11 +318,14 @@ int d_rtext_proc(int msg, DIALOG *d, int c)
  */
 int d_button_proc(int msg, DIALOG *d, int c)
 {
+   BITMAP *gui_bmp;
    int state1, state2;
    int black;
    int swap;
    int g;
-   int rtm;
+   ASSERT(d);
+   
+   gui_bmp = gui_get_screen();
 
    switch (msg) {
 
@@ -304,25 +336,23 @@ int d_button_proc(int msg, DIALOG *d, int c)
 	    state2 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	 }
 	 else {
-	    g = 0; 
+	    g = 0;
 	    state1 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	    state2 = d->bg;
 	 }
 
-	 rectfill(screen, d->x+1+g, d->y+1+g, d->x+d->w-3+g, d->y+d->h-3+g, state2);
-	 rect(screen, d->x+g, d->y+g, d->x+d->w-2+g, d->y+d->h-2+g, state1);
-	 rtm = text_mode(-1);
-	 gui_textout(screen, d->dp, d->x+d->w/2+g, d->y+d->h/2-text_height(font)/2+g, state1, TRUE);
-	 text_mode(rtm);
+	 rectfill(gui_bmp, d->x+1+g, d->y+1+g, d->x+d->w-3+g, d->y+d->h-3+g, state2);
+	 rect(gui_bmp, d->x+g, d->y+g, d->x+d->w-2+g, d->y+d->h-2+g, state1);
+	 gui_textout_ex(gui_bmp, d->dp, d->x+d->w/2+g, d->y+d->h/2-text_height(font)/2+g, state1, -1, TRUE);
 
 	 if (d->flags & D_SELECTED) {
-	    vline(screen, d->x, d->y, d->y+d->h-2, d->bg);
-	    hline(screen, d->x, d->y, d->x+d->w-2, d->bg);
+	    vline(gui_bmp, d->x, d->y, d->y+d->h-2, d->bg);
+	    hline(gui_bmp, d->x, d->y, d->x+d->w-2, d->bg);
 	 }
 	 else {
 	    black = makecol(0,0,0);
-	    vline(screen, d->x+d->w-1, d->y+1, d->y+d->h-2, black);
-	    hline(screen, d->x+1, d->y+d->h-1, d->x+d->w-1, black);
+	    vline(gui_bmp, d->x+d->w-1, d->y+1, d->y+d->h-2, black);
+	    hline(gui_bmp, d->x+1, d->y+d->h-1, d->x+d->w-1, black);
 	 }
 	 if ((d->flags & D_GOTFOCUS) && 
 	     (!(d->flags & D_SELECTED) || !(d->flags & D_EXIT)))
@@ -340,9 +370,7 @@ int d_button_proc(int msg, DIALOG *d, int c)
 
 	 /* or just toggle */
 	 d->flags ^= D_SELECTED;
-	 scare_mouse();
 	 object_message(d, MSG_DRAW, 0);
-	 unscare_mouse();
 	 break;
 
       case MSG_CLICK:
@@ -364,9 +392,7 @@ int d_button_proc(int msg, DIALOG *d, int c)
 	    if (((state1) && (!state2)) || ((state2) && (!state1))) {
 	       d->flags ^= D_SELECTED;
 	       state1 = d->flags & D_SELECTED;
-	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
-	       unscare_mouse();
 	    }
 
 	    /* let other objects continue to animate */
@@ -392,30 +418,31 @@ int d_button_proc(int msg, DIALOG *d, int c)
  */
 int d_check_proc(int msg, DIALOG *d, int c)
 {
-   int x;
+   BITMAP *gui_bmp = gui_get_screen();
+   int x, y, h;
    int fg, bg;
+   ASSERT(d);
 
    if (msg==MSG_DRAW) {
-      int rtm;
-
       fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       bg = (d->bg < 0) ? gui_bg_color : d->bg;
 
-      rtm = text_mode(d->bg);
+      h = text_height(font);
 
-      x = d->x + ((d->d1) ? 0 : gui_textout(screen, d->dp, d->x, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE) + text_height(font)/2);
-      rectfill(screen, x+1, d->y+1, x+d->h-2, d->y+d->h-2, bg);
-      rect(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
-      if (d->d1)
-	 gui_textout(screen, d->dp, x+d->h-1+text_height(font)/2, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE);
-      if (d->flags & D_SELECTED) {
-	 line(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
-	 line(screen, x, d->y+d->h-1, x+d->h-1, d->y, fg);
-      }
+      rectfill(gui_bmp, d->x, d->y, d->x+d->w-1, d->y+d->h-1, bg);
       if (d->flags & D_GOTFOCUS)
-	 dotted_rect(x+1, d->y+1, x+d->h-2, d->y+d->h-2, fg, bg);
+	 dotted_rect(d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg, bg);
 
-      text_mode(rtm);
+      y = d->y + ((d->h-(h-gui_font_baseline))/2);
+      x = d->x + ((d->d1) ? 0 : gui_textout_ex(gui_bmp, d->dp, d->x, y, fg, -1, FALSE) + h/2);
+
+      rect(gui_bmp, x, y, x+h-1, y+h-1, fg);
+      if (d->d1)
+	 gui_textout_ex(gui_bmp, d->dp, x+h+h/2, y, fg, -1, FALSE);
+      if (d->flags & D_SELECTED) {
+	 line(gui_bmp, x, y, x+h-1, y+h-1, fg);
+	 line(gui_bmp, x, y+h-1, x+h-1, y, fg);
+      }
 
       return D_O_K;
    } 
@@ -432,8 +459,10 @@ int d_check_proc(int msg, DIALOG *d, int c)
  */
 int d_radio_proc(int msg, DIALOG *d, int c)
 {
-   int x, center, r, ret, fg, bg;
-   int rtm;
+   BITMAP *gui_bmp = gui_get_screen();
+   int x, y, h, r, ret, fg, bg;
+   int centerx, centery;
+   ASSERT(d);
 
    switch(msg) {
 
@@ -441,38 +470,36 @@ int d_radio_proc(int msg, DIALOG *d, int c)
 	 fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	 bg = (d->bg < 0) ? gui_bg_color : d->bg;
 
-	 rtm = text_mode(d->bg);
-	 gui_textout(screen, d->dp, d->x+d->h-1+text_height(font), d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE);
-	 text_mode(rtm);
+	 rectfill(gui_bmp, d->x, d->y, d->x+d->w-1, d->y+d->h-1, bg);
+	 if (d->flags & D_GOTFOCUS)
+	    dotted_rect(d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg, bg);
+
+	 h = text_height(font);
+	 y = d->y+(d->h-(h-gui_font_baseline))/2;
+
+	 gui_textout_ex(gui_bmp, d->dp, d->x+h+h/2, y, fg, -1, FALSE);
 
 	 x = d->x;
-	 r = d->h/2;
+	 r = h/2;
 
-	 center = x+r;
-	 rectfill(screen, x, d->y, x+d->h-1, d->y+d->h-1, bg);
+	 centerx = d->x+r;
+	 centery = d->y+d->h/2;
 
 	 switch (d->d2) {
 
 	    case 1:
-	       rect(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
+	       rect(gui_bmp, d->x, y, x+h-1, y+h-1, fg);
 	       if (d->flags & D_SELECTED)
-		  rectfill(screen, x+r/2, d->y+r/2, x+d->h-1-r/2, d->y+d->h-1-r/2, fg);
+		  rectfill(gui_bmp, centerx-r/2, centery-r/2, centerx+r/2-1, centery+r/2-1, fg);
 	       break;
 
 	    default:
-	       circle(screen, center, d->y+r, r, fg);
+	       circle(gui_bmp, centerx, centery, r, fg);
 	       if (d->flags & D_SELECTED)
-		  circlefill(screen, center, d->y+r, r/2, fg);
+		  circlefill(gui_bmp, centerx, centery, r/2, fg);
 	       break;
 	 }
 
-	 if (d->flags & D_GOTFOCUS) {
-	    if (d->d2 == 1)
-	       dotted_rect(x+1, d->y+1, x+d->h-2, d->y+d->h-2, fg, bg);
-	    else
-	       dotted_rect(x, d->y, x+d->h-1, d->y+d->h-1, fg, bg);
-	 }
-
 	 return D_O_K;
 
       case MSG_KEY:
@@ -485,9 +512,7 @@ int d_radio_proc(int msg, DIALOG *d, int c)
       case MSG_RADIO:
 	 if ((c == d->d1) && (d->flags & D_SELECTED)) {
 	    d->flags &= ~D_SELECTED;
-	    scare_mouse();
 	    object_message(d, MSG_DRAW, 0);
-	    unscare_mouse();
 	 }
 	 break;
    }
@@ -523,13 +548,17 @@ int d_radio_proc(int msg, DIALOG *d, int c)
  */
 int d_icon_proc(int msg, DIALOG *d, int c)
 {
-   BITMAP *butimage = (BITMAP *)d->dp;
+   BITMAP *butimage;
+   BITMAP *gui_bmp;
    int butx;
    int buty;
    int index;
    int indent;
    int depth;
+   ASSERT(d);
 
+   butimage = (BITMAP *)d->dp;
+   gui_bmp = gui_get_screen();
    if ((msg == MSG_DRAW) && (!(d->flags & D_HIDDEN))) {
       depth = 0;
       if ((d->dp2 == NULL) && (d->flags & D_SELECTED)) {
@@ -550,26 +579,26 @@ int d_icon_proc(int msg, DIALOG *d, int c)
       /* put the graphic on screen, scaled as needed */
       butx = butimage->w;
       buty = butimage->h;
-      stretch_blit(butimage, screen, 0, 0, butx-depth, buty-depth, 
+      stretch_blit(butimage, gui_bmp, 0, 0, butx-depth, buty-depth, 
 		   d->x+depth, d->y+depth, d->w-depth, d->h-depth);
 
       if ((d->flags & D_GOTFOCUS) &&
 	  (!(d->flags & D_SELECTED) || !(d->flags & D_EXIT))) {
 	 /* draw focus lines */
 	 for (index=indent; index<d->w-(indent+1); index+=2) {
-	    putpixel(screen, d->x+index+depth, d->y+indent+depth,d->fg);
-	    putpixel(screen, d->x+index+depth, d->y+d->h-(indent+1)+depth, d->fg);
+	    putpixel(gui_bmp, d->x+index+depth, d->y+indent+depth,d->fg);
+	    putpixel(gui_bmp, d->x+index+depth, d->y+d->h-(indent+1)+depth, d->fg);
 	 }
 	 for (index=indent; index<d->h-(indent+1); index+=2) {
-	    putpixel(screen, d->x+indent+depth, d->y+index+depth, d->fg);
-	    putpixel(screen, d->x+d->w-(indent+1)+depth, d->y+index+depth, d->fg);
+	    putpixel(gui_bmp, d->x+indent+depth, d->y+index+depth, d->fg);
+	    putpixel(gui_bmp, d->x+d->w-(indent+1)+depth, d->y+index+depth, d->fg);
 	 }
       }
 
       /* draw shadowing */
       for (index=0; index<depth; index++) {
-	  hline(screen, d->x, d->y+index, d->x+d->w-1, d->bg);
-	  vline(screen, d->x+index, d->y, d->y+d->h-1, d->bg);
+	  hline(gui_bmp, d->x, d->y+index, d->x+d->w-1, d->bg);
+	  vline(gui_bmp, d->x+index, d->y, d->y+d->h-1, d->bg);
       }
 
       return D_O_K;
@@ -591,6 +620,7 @@ int d_keyboard_proc(int msg, DIALOG *d, int c)
 {
    int (*proc)(void);
    int ret = D_O_K;
+   ASSERT(d);
 
    switch (msg) {
 
@@ -626,11 +656,14 @@ int d_keyboard_proc(int msg, DIALOG *d, int c)
 int d_edit_proc(int msg, DIALOG *d, int c)
 {
    static int ignore_next_uchar = FALSE;
+   BITMAP *gui_bmp;
    int last_was_space, new_pos, i, k;
    int f, l, p, w, x, fg, b, scroll;
    char buf[16];
    char *s, *t;
-   int rtm;
+   ASSERT(d);
+   
+   gui_bmp = gui_get_screen();
 
    s = d->dp;
    l = ustrlen(s);
@@ -685,16 +718,14 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	    f = ugetat(s, p);
 	    usetc(buf+usetc(buf, (f) ? f : ' '), 0);
 	    w = text_length(font, buf);
-	    if (x+w > d->w) 
+	    if (x+w > d->w)
 	       break;
 	    f = ((p == d->d2) && (d->flags & D_GOTFOCUS));
-	    rtm = text_mode((f) ? fg : d->bg);
-	    textout(screen, font, buf, d->x+x, d->y, (f) ? d->bg : fg);
-	    text_mode(rtm);
+	    textout_ex(gui_bmp, font, buf, d->x+x, d->y, (f) ? d->bg : fg, (f) ? fg : d->bg);
 	    x += w;
 	 }
 	 if (x < d->w)
-	    rectfill(screen, d->x+x, d->y, d->x+d->w-1, d->y+text_height(font)-1, d->bg);
+	    rectfill(gui_bmp, d->x+x, d->y, d->x+d->w-1, d->y+text_height(font)-1, d->bg);
 	 break;
 
       case MSG_CLICK:
@@ -713,10 +744,8 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	    if (x > gui_mouse_x()) 
 	       break;
 	 }
-	 d->d2 = MID(0, p, l);
-	 scare_mouse();
+	 d->d2 = CLAMP(0, p, l);
 	 object_message(d, MSG_DRAW, 0);
-	 unscare_mouse();
 	 break;
 
       case MSG_WANTFOCUS:
@@ -779,9 +808,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	 }
 	 else if ((c >> 8) == KEY_ENTER) {
 	    if (d->flags & D_EXIT) {
-	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
-	       unscare_mouse();
 	       return D_CLOSE;
 	    }
 	    else
@@ -795,9 +822,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	    /* don't process regular keys here: MSG_UCHAR will do that */
 	    break;
 	 }
-	 scare_mouse();
 	 object_message(d, MSG_DRAW, 0);
-	 unscare_mouse();
 	 return D_USED_CHAR;
 
       case MSG_UCHAR:
@@ -806,9 +831,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	       uinsert(s, d->d2, c);
 	       d->d2++;
 
-	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
-	       unscare_mouse();
 	    }
 	    return D_USED_CHAR;
 	 }
@@ -844,9 +867,7 @@ void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset, int h
 
 	    if (yy != *offset) {
 	       *offset = yy;
-	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
-	       unscare_mouse();
 	    }
 
 	    /* let other objects continue to animate */
@@ -867,9 +888,7 @@ void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset, int h
 
 	 if (yy != *offset) {
 	    *offset = yy;
-	    scare_mouse();
 	    object_message(d, MSG_DRAW, 0);
-	    unscare_mouse();
 	 }
       }
 
@@ -942,7 +961,7 @@ void _handle_listbox_click(DIALOG *d)
 
    height = (d->h-4) / text_height(font);
 
-   i = MID(0, ((gui_mouse_y() - d->y - 2) / text_height(font)), 
+   i = CLAMP(0, ((gui_mouse_y() - d->y - 2) / text_height(font)), 
 	      ((d->h-4) / text_height(font) - 1));
    i += d->d2;
    if (i < d->d2)
@@ -969,6 +988,8 @@ void _handle_listbox_click(DIALOG *d)
 	    else
 	       sel[i] = !sel[i];
 	 }
+	 else
+	    sel[i] = TRUE;
       }
 
       d->d1 = i;
@@ -978,7 +999,19 @@ void _handle_listbox_click(DIALOG *d)
       d->flags |= D_DIRTY;
 
       if (i != d->d2)
-	 rest_callback(MID(10, text_height(font)*16-d->h-1, 100), idle_cb);
+	 rest_callback(CLAMP(10, text_height(font)*16-d->h-1, 100), idle_cb);
+   }
+   else {
+      if (!(d->flags & D_INTERNAL)) {
+	 if (sel) {
+	    if((key_shifts & KB_CTRL_FLAG))
+	       sel[i] = !sel[i];
+	    else
+	       sel[i] = TRUE;
+
+	    d->flags |= D_DIRTY;
+	 }
+      }
    }
 }
 
@@ -989,16 +1022,17 @@ void _handle_listbox_click(DIALOG *d)
  */
 void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int fg_color, int bg)
 {
+   BITMAP *gui_bmp = gui_get_screen();
    int i, len;
    BITMAP *pattern;
    int xx, yy;
 
    /* draw frame */
-   rect(screen, d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg_color);
+   rect(gui_bmp, d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg_color);
 
    /* possibly draw scrollbar */
    if (listsize > height) {
-      vline(screen, d->x+d->w-13, d->y+1, d->y+d->h-2, fg_color);
+      vline(gui_bmp, d->x+d->w-13, d->y+1, d->y+d->h-2, fg_color);
 
       /* scrollbar with focus */ 
       if (d->flags & D_GOTFOCUS) {
@@ -1006,8 +1040,8 @@ void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int
 	 dotted_rect(d->x+d->w-12, d->y+1, d->x+d->w-2, d->y+d->h-2, fg_color, bg);
       }
       else {
-	 rect(screen, d->x+1, d->y+1, d->x+d->w-14, d->y+d->h-2, bg);
-	 rect(screen, d->x+d->w-12, d->y+1, d->x+d->w-2, d->y+d->h-2, bg);
+	 rect(gui_bmp, d->x+1, d->y+1, d->x+d->w-14, d->y+d->h-2, bg);
+	 rect(gui_bmp, d->x+d->w-12, d->y+1, d->x+d->w-2, d->y+d->h-2, bg);
       }
 
       /* create and draw the scrollbar */
@@ -1023,19 +1057,19 @@ void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int
 
       if (offset > 0) {
 	 len = (((d->h-5) * offset) + listsize/2) / listsize;
-	 rectfill(screen, xx, yy, xx+8, yy+len, bg);
+	 rectfill(gui_bmp, xx, yy, xx+8, yy+len, bg);
 	 yy += len;
       }
       if (yy+i < d->y+d->h-3) {
 	 drawing_mode(DRAW_MODE_COPY_PATTERN, pattern, 0, 0);
-	 rectfill(screen, xx, yy, xx+8, yy+i, 0);
+	 rectfill(gui_bmp, xx, yy, xx+8, yy+i, 0);
 	 solid_mode();
 	 yy += i+1;
-	 rectfill(screen, xx, yy, xx+8, d->y+d->h-3, bg);
+	 rectfill(gui_bmp, xx, yy, xx+8, d->y+d->h-3, bg);
       }
       else {
 	 drawing_mode(DRAW_MODE_COPY_PATTERN, pattern, 0, 0);
-	 rectfill(screen, xx, yy, xx+8, d->y+d->h-3, 0);
+	 rectfill(gui_bmp, xx, yy, xx+8, d->y+d->h-3, 0);
 	 solid_mode();
       }
       destroy_bitmap(pattern);
@@ -1045,7 +1079,7 @@ void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int
       if (d->flags & D_GOTFOCUS)
 	 dotted_rect(d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, fg_color, bg);
       else
-	 rect(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, bg);
+	 rect(gui_bmp, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, bg);
    }
 }
 
@@ -1056,11 +1090,11 @@ void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int
  */
 void _draw_listbox(DIALOG *d)
 {
+   BITMAP *gui_bmp = gui_get_screen();
    int height, listsize, i, len, bar, x, y, w;
    int fg_color, fg, bg;
    char *sel = d->dp2;
    char s[1024];
-   int rtm;
 
    (*(getfuncptr)d->dp)(-1, &listsize);
    height = (d->h-4) / text_height(font);
@@ -1071,13 +1105,23 @@ void _draw_listbox(DIALOG *d)
    /* draw box contents */
    for (i=0; i<height; i++) {
       if (d->d2+i < listsize) {
-	 if (d->d2+i == d->d1) {
+	 if (sel) { 
+	    if ((sel[d->d2+i]) && (d->d2+i == d->d1)) {
+	       fg = d->bg;
+	       bg = fg_color;
+	    }
+	    else if (sel[d->d2+i]) {
+	       fg = d->bg;
+	       bg = gui_mg_color;
+	    }
+	    else {
+	       fg = fg_color;
+	       bg = d->bg;
+	    }
+	 }
+	 else if (d->d2+i == d->d1) {
 	    fg = d->bg;
 	    bg = fg_color;
-	 } 
-	 else if ((sel) && (sel[d->d2+i])) { 
-	    fg = d->bg;
-	    bg = gui_mg_color;
 	 }
 	 else {
 	    fg = fg_color;
@@ -1086,28 +1130,29 @@ void _draw_listbox(DIALOG *d)
 	 ustrzcpy(s, sizeof(s), (*(getfuncptr)d->dp)(i+d->d2, NULL));
 	 x = d->x + 2;
 	 y = d->y + 2 + i*text_height(font);
-	 rtm = text_mode(bg);
-	 rectfill(screen, x, y, x+7, y+text_height(font)-1, bg); 
+	 rectfill(gui_bmp, x, y, x+7, y+text_height(font)-1, bg);
 	 x += 8;
 	 len = ustrlen(s);
 	 while (text_length(font, s) >= MAX(d->w - 1 - (bar ? 22 : 10), 1)) {
 	    len--;
 	    usetat(s, len, 0);
 	 }
-	 textout(screen, font, s, x, y, fg);
-	 text_mode(rtm);
+	 textout_ex(gui_bmp, font, s, x, y, fg, bg);
 	 x += text_length(font, s);
 	 if (x <= d->x+w) 
-	    rectfill(screen, x, y, d->x+w, y+text_height(font)-1, bg);
+	    rectfill(gui_bmp, x, y, d->x+w, y+text_height(font)-1, bg);
+	 if (d->d2+i == d->d1)
+	    dotted_rect(d->x+2, y, d->x+d->w-(bar ? 15 : 3),
+	       y+text_height(font)-1,d->fg, d->bg);
       }
       else {
-	 rectfill(screen, d->x+2,  d->y+2+i*text_height(font), 
+	 rectfill(gui_bmp, d->x+2,  d->y+2+i*text_height(font), 
 		  d->x+w, d->y+1+(i+1)*text_height(font), d->bg);
       }
    }
 
    if (d->y+2+i*text_height(font) <= d->y+d->h-3)
-      rectfill(screen, d->x+2, d->y+2+i*text_height(font), 
+      rectfill(gui_bmp, d->x+2, d->y+2+i*text_height(font), 
 				       d->x+w, d->y+d->h-3, d->bg);
 
    /* draw frame, maybe with scrollbar */
@@ -1135,6 +1180,7 @@ int d_list_proc(int msg, DIALOG *d, int c)
    int listsize, i, bottom, height, bar, orig;
    char *sel = d->dp2;
    int redraw = FALSE;
+   ASSERT(d);
 
    switch (msg) {
 
@@ -1159,11 +1205,8 @@ int d_list_proc(int msg, DIALOG *d, int c)
 		     sel[i] = FALSE;
 		  }
 	       }
-	       if (redraw) {
-		  scare_mouse();
+	       if (redraw)
 		  object_message(d, MSG_DRAW, 0);
-		  unscare_mouse();
-	       }
 	    }
 	    _handle_listbox_click(d);
 	    while (gui_mouse_b()) {
@@ -1205,9 +1248,7 @@ int d_list_proc(int msg, DIALOG *d, int c)
 	       i = MIN(listsize-height, d->d2+delta);
 	    if (i != d->d2) {
 	       d->d2 = i;
-	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
-	       unscare_mouse(); 
 	    }
 	 }
 	 break;
@@ -1294,6 +1335,7 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
    int listsize, i, a, failure;
    char *selected, *thisitem;
    char *sel = d->dp2;
+   ASSERT(d);
 
    switch (msg) {
 
@@ -1341,9 +1383,7 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
 			}
 
 			_handle_scrollable_scroll(d, listsize, &d->d1, &d->d2);
-			scare_mouse();
 			object_message(d, MSG_DRAW, 0);
-			unscare_mouse();
 			return D_USED_CHAR;
 		     }
 		  }
@@ -1375,6 +1415,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 		   int wword, int tabsize, int x, int y, int w, int h,
 		   int disabled, int fore, int deselect, int disable)
 {
+   BITMAP *gui_bmp = gui_get_screen();
    int fg = fore;
    int y1 = y+4;
    int x1;
@@ -1392,7 +1433,6 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
    int line = 0;
    int i = 0;
    int noignore;
-   int rtm;
 
    usetc(s+usetc(s, '.'), 0);
    usetc(text+usetc(text, ' '), 0);
@@ -1407,15 +1447,13 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
    /* do some drawing setup */
    if (draw) {
       /* initial start blanking at the top */
-      rectfill(screen, x+2, y+2, x+w-3, y1-1, deselect);
+      rectfill(gui_bmp, x+2, y+2, x+w-3, y1-1, deselect);
    }
 
    /* choose the text color */
-   if (disabled) 
+   if (disabled)
       fg = disable;
 
-   rtm = text_mode(deselect);
-
    /* loop over the entire string */
    while (1) {
       width = 0;
@@ -1494,7 +1532,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	 x1 = x+4;
 
 	 /* the initial blank bit */
-	 rectfill(screen, x+2, y1, x1-1, y1+text_height(font), deselect);
+	 rectfill(gui_bmp, x+2, y1, x1-1, y1+text_height(font), deselect);
 
 	 /* print up to the marked character */
 	 while (printed != scanned) {
@@ -1510,7 +1548,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	       case '\t':
 		  for (i=0; i<tabsize; i++) {
 		     usetc(s+usetc(s, ' '), 0);
-		     textout(screen, font, s, x1, y1, fg);
+		     textout_ex(gui_bmp, font, s, x1, y1, fg, deselect);
 		     x1 += text_length(font, s);
 		  }
 		  break;
@@ -1519,7 +1557,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	       default:
 		  if (printed != ignore) {
 		     usetc(s+usetc(s, ugetc(printed)), 0);
-		     textout(screen, font, s, x1, y1, fg);
+		     textout_ex(gui_bmp, font, s, x1, y1, fg, deselect);
 		     x1 += text_length(font, s);
 		  }
 	    }
@@ -1529,7 +1567,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	 }
 	 /* the last blank bit */
 	 if (x1 <= x+w-3) 
-	    rectfill(screen, x1, y1, x+w-3, y1+text_height(font)-1, deselect);
+	    rectfill(gui_bmp, x1, y1, x+w-3, y1+text_height(font)-1, deselect);
 
 	 /* print the line end */
 	 y1 += text_height(font);
@@ -1542,17 +1580,15 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
       /* check if we are at the end of the string */
       if (!ugetc(printed)) {
 	 /* the under blank bit */
-	 if (draw) 
-	    rectfill(screen, x+1, y1, x+w-3, y+h-1, deselect);
+	 if (draw)
+	    rectfill(gui_bmp, x+1, y1, x+w-3, y+h-1, deselect);
 
 	 /* tell how many lines we found */
 	 *listsize = line;
-	 text_mode(rtm);
 	 return;
       }
    }
 
-   text_mode(rtm);
 }
 
 
@@ -1572,8 +1608,10 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
    int height, bar, ret = D_O_K;
    int start, top, bottom, l;
    int used, delta;
-   int fg_color = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
+   int fg_color;
+   ASSERT(d);
 
+   fg_color = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
    /* calculate the actual height */
    height = (d->h-8) / text_height(font);
 
@@ -1725,6 +1763,7 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
  */
 int d_slider_proc(int msg, DIALOG *d, int c)
 {
+   BITMAP *gui_bmp = gui_get_screen();
    BITMAP *slhan = NULL;
    int oldpos, newpos;
    int sfg;                /* slider foreground color */
@@ -1744,6 +1783,7 @@ int d_slider_proc(int msg, DIALOG *d, int c)
    fixed slratio, slmax, slpos;
    int (*proc)(void *cbpointer, int d2value);
    int oldval;
+   ASSERT(d);
 
    /* check for slider direction */
    if (d->h < d->w)
@@ -1771,14 +1811,14 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 	 sfg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 
 	 if (vert) {
-	    rectfill(screen, d->x, d->y, d->x+d->w/2-2, d->y+d->h-1, d->bg);
-	    rectfill(screen, d->x+d->w/2-1, d->y, d->x+d->w/2+1, d->y+d->h-1, sfg);
-	    rectfill(screen, d->x+d->w/2+2, d->y, d->x+d->w-1, d->y+d->h-1, d->bg);
+	    rectfill(gui_bmp, d->x, d->y, d->x+d->w/2-2, d->y+d->h-1, d->bg);
+	    rectfill(gui_bmp, d->x+d->w/2-1, d->y, d->x+d->w/2+1, d->y+d->h-1, sfg);
+	    rectfill(gui_bmp, d->x+d->w/2+2, d->y, d->x+d->w-1, d->y+d->h-1, d->bg);
 	 }
 	 else {
-	    rectfill(screen, d->x, d->y, d->x+d->w-1, d->y+d->h/2-2, d->bg);
-	    rectfill(screen, d->x, d->y+d->h/2-1, d->x+d->w-1, d->y+d->h/2+1, sfg);
-	    rectfill(screen, d->x, d->y+d->h/2+2, d->x+d->w-1, d->y+d->h-1, d->bg);
+	    rectfill(gui_bmp, d->x, d->y, d->x+d->w-1, d->y+d->h/2-2, d->bg);
+	    rectfill(gui_bmp, d->x, d->y+d->h/2-1, d->x+d->w-1, d->y+d->h/2+1, sfg);
+	    rectfill(gui_bmp, d->x, d->y+d->h/2+2, d->x+d->w-1, d->y+d->h-1, d->bg);
 	 }
 
 	 /* okay, background and slot are drawn, now draw the handle */
@@ -1791,7 +1831,7 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 	       slx = d->x+slp;
 	       sly = d->y+(d->h/2)-(slhan->h/2);
 	    }
-	    draw_sprite(screen, slhan, slx, sly);
+	    draw_sprite(gui_bmp, slhan, slx, sly);
 	 } 
 	 else {
 	    /* draw default handle */
@@ -1800,7 +1840,8 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 	       sly = d->y+(d->h)-(hh+slp);
 	       slw = d->w-1;
 	       slh = hh-1;
-	    } else {
+	    }
+            else {
 	       slx = d->x+slp;
 	       sly = d->y;
 	       slw = hh-1;
@@ -1808,14 +1849,14 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 	    }
 
 	    /* draw body */
-	    rectfill(screen, slx+2, sly, slx+(slw-2), sly+slh, sfg);
-	    vline(screen, slx+1, sly+1, sly+slh-1, sfg);
-	    vline(screen, slx+slw-1, sly+1, sly+slh-1, sfg);
-	    vline(screen, slx, sly+2, sly+slh-2, sfg);
-	    vline(screen, slx+slw, sly+2, sly+slh-2, sfg);
-	    vline(screen, slx+1, sly+2, sly+slh-2, d->bg);
-	    hline(screen, slx+2, sly+1, slx+slw-2, d->bg);
-	    putpixel(screen, slx+2, sly+2, d->bg);
+	    rectfill(gui_bmp, slx+2, sly, slx+(slw-2), sly+slh, sfg);
+	    vline(gui_bmp, slx+1, sly+1, sly+slh-1, sfg);
+	    vline(gui_bmp, slx+slw-1, sly+1, sly+slh-1, sfg);
+	    vline(gui_bmp, slx, sly+2, sly+slh-2, sfg);
+	    vline(gui_bmp, slx+slw, sly+2, sly+slh-2, sfg);
+	    vline(gui_bmp, slx+1, sly+2, sly+slh-2, d->bg);
+	    hline(gui_bmp, slx+2, sly+1, slx+slw-2, d->bg);
+	    putpixel(gui_bmp, slx+2, sly+2, d->bg);
 	 }
 
 	 if (d->flags & D_GOTFOCUS)
@@ -1894,16 +1935,14 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 		  retval |= (*proc)(d->dp3, d->d2);
 	       }
 
-	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
-	       unscare_mouse();
 	    }
 	 }
 	 break;
       
       case MSG_WHEEL: 
 	 oldval = d->d2;
-	 d->d2 = MID(0, d->d2+c, d->d1);
+	 d->d2 = CLAMP(0, d->d2+c, d->d1);
 	 if (d->d2 != oldval) {
 	    /* call callback function here */
 	    if (d->dp2) {
@@ -1911,9 +1950,7 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 	       retval |= (*proc)(d->dp3, d->d2);
 	    }
 
-	    scare_mouse();
 	    object_message(d, MSG_DRAW, 0);
-	    unscare_mouse();
 	 }
 	 break;
 
@@ -1945,9 +1982,7 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 		  retval |= (*proc)(d->dp3, d->d2);
 	       }
 
-	       scare_mouse();
 	       object_message(d, MSG_DRAW, 0);
-	       unscare_mouse();
 	    }
 
 	    /* let other objects continue to animate */
