@@ -52,6 +52,73 @@ static int repeat_scan = -1;
 
 static int rate_changed = FALSE;
 
+/* Provide a default ASCII mapping for the most common keys. Keys whose
+ * mapping changes dependind on the layout aren't listed - it's up to
+ * the keyboard driver to do that. The reason for this it portability:
+ * With an English keyboard, it makes sense to just list the KEY_* names,
+ * but not with other keyboards. Ideally, every keyboard driver would
+ * be able to fill in all the keys - but where it doesn't, this table
+ * is a good compromise.
+ */
+static int common_ascii[KEY_MAX] =
+{
+   0,   'a', 'b', 'c', 'd', 'e', 'f', 'g',
+   'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
+   'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
+   'x', 'y', 'z', '0', '1', '2', '3', '4',
+   '5', '6', '7', '8', '9', '0', '1', '2',
+   '3', '4', '5', '6', '7', '8', '9', 0,
+   0,   0,   0,   0,   0,   0,   0,   0,
+   0,   0,   0,   27,  0,   0,   0,   0,
+   9,   0,   0,   13,  0,   0,   0,   0,
+   0,   0,   0,   ' ', 0,   0,   0,   0,
+   0,   0,   0,   0,   0,   0,   '/', '*',
+   '-', '+', 0,  13,   0,   0,   0,   0,
+   0,   0,   0,   0,   0,   0,   0,   '=',
+   0,   0,   0,   0,   0,   0,   0,   0,
+   0,   0,   0,   0,   0,   0,   0,   0,
+   0,   0,   0,   0,   0,   0,   0
+};
+
+/* The same as above, but with descriptive names. Again, only keys which are
+ * likely to be found on every keyboard are named. All keyboard drivers should
+ * provide their own implementation though, especially if they use positional
+ * mapping. */
+AL_CONST char *_keyboard_common_names[KEY_MAX] =
+{
+   "(none)",     "A",          "B",          "C",
+   "D",          "E",          "F",          "G",
+   "H",          "I",          "J",          "K",
+   "L",          "M",          "N",          "O",
+   "P",          "Q",          "R",          "S",
+   "T",          "U",          "V",          "W",
+   "X",          "Y",          "Z",          "0",
+   "1",          "2",          "3",          "4",
+   "5",          "6",          "7",          "8",
+   "9",          "0 PAD",      "1 PAD",      "2 PAD",
+   "3 PAD",      "4 PAD",      "5 PAD",      "6 PAD",
+   "7 PAD",      "8 PAD",      "9 PAD",      "F1",
+   "F2",         "F3",         "F4",         "F5",
+   "F6",         "F7",         "F8",         "F9",
+   "F10",        "F11",        "F12",        "ESC",
+   "KEY60",      "KEY61",      "KEY62",      "BACKSPACE",
+   "TAB",        "KEY65",      "KEY66",      "ENTER",
+   "KEY68",      "KEY69",      "BACKSLASH",  "KEY71",
+   "KEY72",      "KEY73",      "KEY74",      "SPACE",
+   "INSERT",     "DEL",        "HOME",       "END",
+   "PGUP",       "PGDN",       "LEFT",       "RIGHT",
+   "UP",         "DOWN",       "/ PAD",      "* PAD",
+   "- PAD",      "+ PAD",      "DEL PAD",    "ENTER PAD",
+   "PRINT",      "PAUSE",      "KEY94",      "KEY95",
+   "KEY96",      "KEY97",      "KEY98",      "KEY99",
+   "KEY100",     "KEY101",     "KEY102",     "= PAD",
+   "KEY104",     "KEY105",     "KEY106",     "KEY107",
+   "KEY108",     "KEY109",     "KEY110",     "KEY111",
+   "KEY112",     "KEY113",     "KEY114",     "LSHIFT",
+   "RSHIFT",     "LCONTROL",   "RCONTROL",   "ALT",
+   "ALTGR",      "LWIN",       "RWIN",       "MENU",
+   "SCRLOCK",    "NUMLOCK",    "CAPSLOCK"
+};
 
 #define KEY_BUFFER_SIZE    64                /* character ring buffer */
 
@@ -125,7 +192,7 @@ static INLINE void add_key(volatile KEY_BUFFER *buffer, int key, int scancode)
 /* clear_keybuf:
  *  Clears the keyboard buffer.
  */
-void clear_keybuf()
+void clear_keybuf(void)
 {
    if (keyboard_polled)
       poll_keyboard();
@@ -164,7 +231,7 @@ static void clear_key(void)
 /* keypressed:
  *  Returns TRUE if there are keypresses waiting in the keyboard buffer.
  */
-int keypressed()
+int keypressed(void)
 {
    if (keyboard_polled)
       poll_keyboard();
@@ -187,7 +254,7 @@ int keypressed()
  *  the return value contains the ASCII code of the key, and the high
  *  byte the scan code. 
  */
-int readkey()
+int readkey(void)
 {
    int key, scancode;
 
@@ -231,7 +298,7 @@ int ureadkey(int *scancode)
       if (keyboard_polled)
 	 poll_keyboard();
 
-      yield_timeslice();
+      rest(1);
    }
 
    c = key_buffer.key[key_buffer.start];
@@ -367,7 +434,7 @@ void _handle_key_press(int keycode, int scancode)
 {
    if ((keyboard_driver->poll) || (!keyboard_polled)) {
       /* process immediately */
-      if (scancode >= 0) {
+      if (scancode > 0) {
 	 if ((!repeat_delay) && (key[scancode]))
 	    return;
 
@@ -377,6 +444,7 @@ void _handle_key_press(int keycode, int scancode)
 	    keyboard_lowlevel_callback(scancode);
       }
 
+      /* e.g. for F1, keycode=0, and scancode=KEY_F1 */
       if (keycode >= 0)
 	 add_key(&key_buffer, keycode, scancode);
 
@@ -384,7 +452,7 @@ void _handle_key_press(int keycode, int scancode)
    }
    else {
       /* deal with this during the next poll_keyboard() */
-      if (scancode >= 0) {
+      if (scancode > 0) {
 	 if ((!repeat_delay) && (_key[scancode]))
 	    return;
 
@@ -397,7 +465,7 @@ void _handle_key_press(int keycode, int scancode)
 
    /* autorepeat? */
    if ((keyboard_driver->autorepeat) && (repeat_delay) && 
-       (keycode >= 0) && (scancode > 0) &&
+       (keycode >= 0) && (scancode > 0) && (scancode != KEY_PAUSE) &&
        ((keycode != repeat_key) || (scancode != repeat_scan))) {
       repeat_key = keycode;
       repeat_scan = scancode;
@@ -451,7 +519,7 @@ END_OF_FUNCTION(_handle_key_release);
  *  switch into polling mode and will no longer operate asynchronously
  *  even if the driver actually does support that.
  */
-int poll_keyboard()
+int poll_keyboard(void)
 {
    int i;
 
@@ -501,7 +569,7 @@ int poll_keyboard()
 /* keyboard_needs_poll:
  *  Checks whether the current driver uses polling.
  */
-int keyboard_needs_poll()
+int keyboard_needs_poll(void)
 {
    return keyboard_polled;
 }
@@ -516,10 +584,34 @@ int keyboard_needs_poll()
  */
 int scancode_to_ascii(int scancode)
 {
+   ASSERT (scancode >= 0 && scancode < KEY_MAX);
    if (keyboard_driver->scancode_to_ascii)
       return keyboard_driver->scancode_to_ascii(scancode);
+   else
+      return common_ascii[scancode];
+}
 
-   return 0;
+
+
+/* scancode_to_name:
+ *  Converts the given scancode to a description of the key.
+ */
+AL_CONST char *scancode_to_name(int scancode)
+{
+   AL_CONST char *name = NULL;
+
+   ASSERT(keyboard_driver);
+   ASSERT((scancode >= 0) && (scancode < KEY_MAX));
+
+   if (keyboard_driver->scancode_to_name)
+      name = keyboard_driver->scancode_to_name(scancode);
+
+   if (!name)
+      name = _keyboard_common_names[scancode];
+
+   ASSERT(name);
+
+   return name;
 }
 
 
@@ -528,7 +620,7 @@ int scancode_to_ascii(int scancode)
  *  Installs Allegro's keyboard handler. You must call this before using 
  *  any of the keyboard input routines. Returns -1 on failure.
  */
-int install_keyboard()
+int install_keyboard(void)
 {
    _DRIVER_INFO *driver_list;
    int i;
@@ -584,12 +676,14 @@ int install_keyboard()
 
    set_leds(-1);
 
-   _add_exit_func(remove_keyboard);
+   _add_exit_func(remove_keyboard, "remove_keyboard");
    _keyboard_installed = TRUE;
 
    if ((keyboard_driver->autorepeat) && (!_timer_installed))
       install_timer();
 
+   update_shifts();
+
    return 0;
 }
 
