@@ -43,8 +43,9 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
    int w, h, i, x, y, bpl, ppl, pbm_mode;
    char ch, cmp_type, bit_plane, color_depth;
    unsigned char uc, check_flags, bit_mask, *line_buf;
-   long id, len, l;
+   long data_id, len, l;
    int dest_depth = _color_load_depth(8, FALSE);
+   ASSERT(filename);
 
    if (!pal)
       pal = tmppal;
@@ -53,26 +54,26 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
    if (!f)
       return NULL;
 
-   id = pack_igetl(f);              /* read file header    */
-   if (id != IFF_FORM) {            /* check for 'FORM' id */
+   data_id = pack_igetl(f);         /* read file header    */
+   if (data_id != IFF_FORM) {       /* check for 'FORM' id */
       pack_fclose(f);
       return NULL;
    }
 
    pack_igetl(f);                   /* skip FORM length    */
 
-   id = pack_igetl(f);              /* read id             */
+   data_id = pack_igetl(f);         /* read id             */
 
    /* check image type ('ILBM' or 'PBM ') */
-   if ((id != IFF_ILBM) && (id != IFF_PBM)) {
+   if ((data_id != IFF_ILBM) && (data_id != IFF_PBM)) {
       pack_fclose(f);
       return NULL;
    }
 
-   pbm_mode = id == IFF_PBM;
+   pbm_mode = data_id == IFF_PBM;
 
-   id = pack_igetl(f);              /* read id               */
-   if (id != IFF_BMHD) {            /* check for header      */
+   data_id = pack_igetl(f);         /* read id               */
+   if (data_id != IFF_BMHD) {       /* check for header      */
       pack_fclose(f);
       return NULL;
    }
@@ -104,6 +105,8 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
       return NULL;
    }
 
+   *allegro_errno = 0;
+
    pack_getc(f);                    /* skip unused field        */
    pack_igetw(f);                   /* skip transparent color   */
    pack_getc(f);                    /* skip x aspect ratio      */
@@ -115,9 +118,9 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
 
    do {  /* We'll use cycle to skip possible junk      */
 	 /*  chunks: ANNO, CAMG, GRAB, DEST, TEXT etc. */
-      id = pack_igetl(f);
+      data_id = pack_igetl(f);
 
-      switch(id) {
+      switch(data_id) {
 
 	 case IFF_CMAP:
 	    memset(pal, 0, 256 * 3);
@@ -149,7 +152,7 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
 	    }
 	    if (bpl & 1)            /* alignment            */
 	       bpl++;
-	    line_buf = malloc(bpl);
+	    line_buf = _AL_MALLOC_ATOMIC(bpl);
 	    if (!line_buf) {
 	       pack_fclose(f);
 	       return NULL;
@@ -255,7 +258,7 @@ BITMAP *load_lbm(AL_CONST char *filename, RGB *pal)
 		  }
 	       }
 	    }
-	    free(line_buf);
+	    _AL_FREE(line_buf);
 	    check_flags |= 2;       /* flag "bitmap read" */
 	    break;
 
