@@ -26,25 +26,19 @@
 #include "allegro.h"
 #include "allegro/internal/aintern.h"
 #include "allegro/platform/aintunix.h"
-#include "allegro/internal/aintvga.h"
 #include "linalleg.h"
 
-#ifdef HAVE_MMAP
+#ifdef ALLEGRO_HAVE_MMAP
 #include <sys/mman.h>
 #endif
 
-#if !defined(_POSIX_MAPPED_FILES) || !defined(HAVE_MMAP)
+#if !defined(_POSIX_MAPPED_FILES) || !defined(ALLEGRO_HAVE_MMAP)
 #error "Sorry, mapped files are required for Allegro/Linux to work!"
 #endif
 
 
 static int switch_mode = SWITCH_PAUSE;
 
-#define MAX_SWITCH_CALLBACKS  8
-
-static void (*switch_in_cb[MAX_SWITCH_CALLBACKS])(void) = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-static void (*switch_out_cb[MAX_SWITCH_CALLBACKS])(void) = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-
 static int vtswitch_initialised = 0;
 static struct vt_mode startup_vtmode;
 
@@ -57,8 +51,6 @@ static volatile int console_should_be_active = 1;  /* should we be? */
 
 int __al_linux_set_display_switch_mode (int mode)
 {
-	int i;
-
 	/* Clean up after the previous mode, if necessary */
 	if (switch_mode == SWITCH_NONE) __al_linux_switching_blocked--;
 
@@ -67,74 +59,24 @@ int __al_linux_set_display_switch_mode (int mode)
 	/* Initialise the new mode */
 	if (switch_mode == SWITCH_NONE) __al_linux_switching_blocked++;
 
-	/* Clear callbacks and return success */
-	for (i=0; i<MAX_SWITCH_CALLBACKS; i++)
-		switch_in_cb[i] = switch_out_cb[i] = NULL;
-
 	return 0;
 }
 
 
 
-int __al_linux_set_display_switch_callback (int dir, void (*cb) (void))
-{
-	int i;
-
-	if (switch_mode == SWITCH_NONE) return -1;
-
-	for (i=0; i<MAX_SWITCH_CALLBACKS; i++) {
-		if (dir == SWITCH_IN) {
-			if (!switch_in_cb[i]) {
-				switch_in_cb[i] = cb;
-				return 0;
-			}
-		}
-		else {
-			if (!switch_out_cb[i]) {
-				switch_out_cb[i] = cb;
-				return 0;
-			}
-		}
-	}
-
-	return -1;
-}
-
-
-
-void __al_linux_remove_display_switch_callback (void (*cb) (void))
-{
-	int i;
-
-	for (i=0; i<MAX_SWITCH_CALLBACKS; i++) {
-		if (switch_in_cb[i] == cb)
-			switch_in_cb[i] = NULL;
-
-		if (switch_out_cb[i] == cb)
-			switch_out_cb[i] = NULL;
-	}
-}
-
-
-
 /* go_away:
  *  Performs a switch away.
  */
 static void go_away(void)
 {
-	int i;
-
-	for (i=0; i<MAX_SWITCH_CALLBACKS; i++)
-		if (switch_out_cb[i])
-			switch_out_cb[i]();
+	_switch_out();
 
 	_unix_bg_man->disable_interrupts();
 	if ((switch_mode == SWITCH_PAUSE) || (switch_mode == SWITCH_AMNESIA))
 		if (timer_driver) timer_driver->exit();
 
 	/* Disable input devices while we're away */
-	__al_linux_disable_standard_driver(STD_MOUSE);
-	__al_linux_disable_standard_driver(STD_KBD);
+	__al_linux_suspend_standard_drivers();
 
 	_save_switch_state(switch_mode);
 
@@ -161,8 +103,6 @@ static void go_away(void)
  */
 static void come_back(void)
 {
-	int i;
-
 	_unix_bg_man->disable_interrupts();
 
 	if (gfx_driver && gfx_driver->restore_video_state)
@@ -173,14 +113,11 @@ static void come_back(void)
 	ioctl(__al_linux_console_fd, VT_RELDISP, VT_ACKACQ);
 	console_active = 1;
 
-	__al_linux_enable_standard_driver(STD_KBD);
-	__al_linux_enable_standard_driver(STD_MOUSE);
+	__al_linux_resume_standard_drivers();
 
 	_unix_bg_man->enable_interrupts();
 
-	for (i=0; i<MAX_SWITCH_CALLBACKS; i++)
-		if (switch_in_cb[i])
-			switch_in_cb[i]();
+	_switch_in();
 
 	__al_linux_switching_blocked--;
 }
@@ -235,6 +172,8 @@ static RETSIGTYPE vt_switch_requested(int signo)
  */
 void __al_linux_acquire_bitmap(BITMAP *bmp)
 {
+	ASSERT(bmp);
+
 	__al_linux_switching_blocked++;
 }
 
@@ -245,6 +184,8 @@ void __al_linux_acquire_bitmap(BITMAP *bmp)
  */
 void __al_linux_release_bitmap(BITMAP *bmp)
 {
+	ASSERT(bmp);
+
 	__al_linux_switching_blocked--;
 	poll_console_switch();
 }
@@ -257,6 +198,10 @@ void __al_linux_release_bitmap(BITMAP *bmp)
  */
 void __al_linux_display_switch_lock(int lock, int foreground)
 {
+	if (__al_linux_console_fd == -1) {
+		return;
+	}
+
 	if (foreground) {
 		__al_linux_wait_for_display();
 	}
