@@ -59,49 +59,52 @@ static void mac_message(const char *msg);
 static void mac_assert(const char *msg);
 static BITMAP * mac_create_bitmap(int color_depth, int width, int height);
 static int mac_desktop_color_depth(void);
+static void mac_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
 static void mac_yield_timeslice(void);
 static int mac_trace_handler(const char *msg);
 
 
 
 SYSTEM_DRIVER system_macos ={
-   SYSTEM_MACOS,
-   empty_string,
-   empty_string,
-   "MacOs",
+   SYSTEM_MACOS, /* id */
+   empty_string, /* name */
+   empty_string, /* desc */
+   "MacOs",      /* ascii_name */
    mac_init,
    mac_exit,
    mac_get_executable_name,
-   NULL,
+   NULL, /* find_resource */
    mac_set_window_title,
-   NULL,
-   NULL,
+   NULL, /* set_close_button_callback */
    mac_message,
    mac_assert,
-   NULL,
-   NULL,
-   NULL, /*mac_create_bitmap,*/
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
+   NULL, /* save_console_state */
+   NULL, /* restore_console_state */
+   NULL, /* mac_create_bitmap, */
+   NULL, /* created_bitmap */
+   NULL, /* create_sub_bitmap */
+   NULL, /* created_sub_bitmap */
+   NULL, /* destroy_bitmap */
+   NULL, /* read_hardware_palette */
+   NULL, /* set_palette_range */
+   NULL, /* get_vtable */
+   NULL, /* set_display_switch_mode */
+   NULL, /* display_switch_lock */
    mac_desktop_color_depth,
-   NULL,
+   NULL, /* get_desktop_resolution */
+   mac_get_gfx_safe_mode,
    mac_yield_timeslice,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
-   NULL,
+   NULL, /* create_mutex */
+   NULL, /* destroy_mutex */
+   NULL, /* lock_mutex */
+   NULL, /* unlock_mutex */
+   NULL, /* gfx_drivers */
+   NULL, /* digi_drivers */
+   NULL, /* midi_drivers */
+   NULL, /* keyboard_drivers */
+   NULL, /* mouse_drivers */
+   NULL, /* joystick_drivers */
+   NULL  /* timer_drivers */
 };
 
 
@@ -119,6 +122,7 @@ MOUSE_DRIVER mouse_macos ={
    NULL,
    NULL,
    NULL,
+   NULL,
    NULL
 };
 
@@ -149,7 +153,8 @@ static int mac_init()
 {
    os_type = OSTYPE_MACOS;
    os_multitasking = TRUE;
-   register_trace_handler(mac_trace_handler);
+   if (!_al_trace_handler)
+      register_trace_handler(mac_trace_handler);
 #if (TRACE_MAC_SYSTEM)
    fprintf(stdout, "mac_init()::OK\n");
    fflush(stdout);
@@ -196,7 +201,7 @@ void _mac_get_executable_name(char *output, int size) {
    e = GetProcessInformation(&curPSN, &info);
 
    size = MIN(size, (unsigned char)appName[0]);
-   strncpy(output, (char const *)appName+1, size);
+   _al_sane_strncpy(output, (char const *)appName+1, size);
    output[size] = 0;
 }
 
@@ -286,15 +291,15 @@ BITMAP *mac_create_bitmap(int color_depth, int width, int height)
    if (!vtable)
       return NULL;
 
-   bitmap = malloc(sizeof(BITMAP) + (sizeof(char *) * height));
+   bitmap = _AL_MALLOC(sizeof(BITMAP) + (sizeof(char *) * height));
    if (!bitmap)
       return NULL;
 
    width_bytes=(width * BYTES_PER_PIXEL(color_depth) + (MEMORY_ALIGN - 1) ) & -MEMORY_ALIGN;
    
-   bitmap->dat = malloc(width_bytes * height + MEMORY_ALIGN);
+   bitmap->dat = _AL_MALLOC(width_bytes * height + MEMORY_ALIGN);
    if (!bitmap->dat) {
-      free(bitmap);
+      _AL_FREE(bitmap);
       return NULL;
    }
 
@@ -332,6 +337,19 @@ static int mac_desktop_color_depth(void)
 
 
 
+/* mac_get_gfx_safe_mode:
+ *  Defines the safe graphics mode for this system.
+ */
+static void mac_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
+{
+   *driver = GFX_DRAWSPROCKET;
+   mode->width = 320;
+   mode->height = 200;
+   mode->bpp = 8;
+}
+
+
+
 /*
  *
  */
@@ -490,9 +508,9 @@ QDGlobals qd;
  * an strdup function to mac
  */
 char *strdup(const char *p){
-   char *tmp=malloc(strlen(p));
+   char *tmp=_AL_MALLOC(strlen(p)+1);
    if(tmp){
-      strcpy(tmp,p);
+      _al_sane_strncpy(tmp,p,strlen(p)+1);
    }
    return tmp;
 }
