@@ -65,6 +65,7 @@ GFX_DRIVER gfx_photon_win =
    NULL,                         /* AL_METHOD(void, drawing_mode, (void)); */
    NULL,                         /* AL_METHOD(void, save_video_state, (void)); */
    NULL,                         /* AL_METHOD(void, restore_video_state, (void)); */
+   NULL,                         /* AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a)); */
    NULL,                         /* AL_METHOD(int, fetch_mode_list, (void)); */
    0, 0,                         /* physical (not virtual!) screen size */
    TRUE,                         /* true if video memory is linear */
@@ -330,7 +331,7 @@ static BITMAP *qnx_private_ph_init_win(GFX_DRIVER *drv, int w, int h, int v_w, i
       window_pitch = ph_window_context->pitch;
       
       /* the pseudo-screen is a memory bitmap */
-      pseudo_screen_addr = malloc(w * h * BYTES_PER_PIXEL(color_depth));
+      pseudo_screen_addr = _AL_MALLOC(w * h * BYTES_PER_PIXEL(color_depth));
       if (!pseudo_screen_addr) {
          ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Not enough memory"));
          return NULL;
@@ -347,7 +348,9 @@ static BITMAP *qnx_private_ph_init_win(GFX_DRIVER *drv, int w, int h, int v_w, i
    }   
 
    /* the last flag serves as an end of loop delimiter */
-   ph_dirty_lines = calloc(h+1, sizeof(char));
+   ph_dirty_lines = _AL_MALLOC((h+1) * sizeof(char));
+   ASSERT(ph_dirty_lines);
+   memset(ph_dirty_lines, 0, (h+1) * sizeof(char));
    ph_dirty_lines[h] = 1;
 
    setup_driver(drv, w, h, color_depth);
@@ -406,12 +409,12 @@ static void qnx_private_ph_exit_win(BITMAP *bmp)
    }
    
    if (pseudo_screen_addr) {
-      free(pseudo_screen_addr);
+      _AL_FREE(pseudo_screen_addr);
       pseudo_screen_addr = NULL;
    }
    
    if (ph_dirty_lines) {
-      free(ph_dirty_lines);
+      _AL_FREE(ph_dirty_lines);
       ph_dirty_lines = NULL;
    }
    
