@@ -22,8 +22,7 @@
 
 
 
-#define floatcos(x)   ((float)(cos((x) * AL_PI / 128.0)))
-#define floatsin(x)   ((float)(sin((x) * AL_PI / 128.0)))
+#define FLOATSINCOS(x, s, c)  _AL_SINCOS((x) * AL_PI / 128.0, s ,c)
 
 
 #define EPSILON (0.001)
@@ -44,6 +43,9 @@ QUAT identity_quat = { 1.0, 0.0, 0.0, 0.0 };
 void quat_mul(AL_CONST QUAT *p, AL_CONST QUAT *q, QUAT *out)
 {
    QUAT temp;
+   ASSERT(p);
+   ASSERT(q);
+   ASSERT(out);
 
    /* qp = ww' - v.v' + vxv' + wv' + w'v */
 
@@ -90,8 +92,9 @@ void quat_mul(AL_CONST QUAT *p, AL_CONST QUAT *q, QUAT *out)
  */
 void get_x_rotate_quat(QUAT *q, float r)
 {
-   q->w = floatcos(r / 2);
-   q->x = floatsin(r / 2);
+   ASSERT(q);
+
+   FLOATSINCOS(r/2, q->x, q->w);
    q->y = 0;
    q->z = 0;
 }
@@ -105,9 +108,10 @@ void get_x_rotate_quat(QUAT *q, float r)
  */
 void get_y_rotate_quat(QUAT *q, float r)
 {
-   q->w = floatcos(r / 2);
+   ASSERT(q);
+
+   FLOATSINCOS(r/2, q->y, q->w);
    q->x = 0;
-   q->y = floatsin(r / 2);
    q->z = 0;
 }
 
@@ -120,10 +124,11 @@ void get_y_rotate_quat(QUAT *q, float r)
  */
 void get_z_rotate_quat(QUAT *q, float r)
 {
-   q->w = floatcos(r / 2);
+   ASSERT(q);
+
+   FLOATSINCOS(r/2, q->z, q->w);
    q->x = 0;
    q->y = 0;
-   q->z = floatsin(r / 2);
 }
 
 
@@ -143,12 +148,11 @@ void get_rotation_quat(QUAT *q, float x, float y, float z)
    float cycz;
    float sysz;
 
-   sx = floatsin(x / 2);
-   sy = floatsin(y / 2);
-   sz = floatsin(z / 2);
-   cx = floatcos(x / 2);
-   cy = floatcos(y / 2);
-   cz = floatcos(z / 2);
+   ASSERT(q);
+
+   FLOATSINCOS(x/2, sx, cx);
+   FLOATSINCOS(y/2, sy, cy);
+   FLOATSINCOS(z/2, sz, cz);
 
    sysz = sy * sz;
    cycz = cy * cz;
@@ -171,6 +175,8 @@ void get_vector_rotation_quat(QUAT *q, float x, float y, float z, float a)
    float l;
    float s;
 
+   ASSERT(q);
+
    l = vector_length_f(x, y, z);
 
    /* NOTE: Passing (x, y, z) = (0, 0, 0) will cause l to equal 0 which will
@@ -183,8 +189,7 @@ void get_vector_rotation_quat(QUAT *q, float x, float y, float z, float a)
    y /= l;
    z /= l;
 
-   q->w = floatcos(a / 2);
-   s    = floatsin(a / 2);
+   FLOATSINCOS(a/2, s, q->w);
    q->x = s * x;
    q->y = s * y;
    q->z = s * z;
@@ -207,6 +212,8 @@ void quat_to_matrix(AL_CONST QUAT *q, MATRIX_f *m)
    float xy;
    float xz;
    float yz;
+   ASSERT(q);
+   ASSERT(m);
 
   /* This is the layout for converting the values in a quaternion to a
    * matrix.
@@ -253,62 +260,37 @@ void quat_to_matrix(AL_CONST QUAT *q, MATRIX_f *m)
  */
 void matrix_to_quat(AL_CONST MATRIX_f *m, QUAT *q)
 {
-   float diag;
-   float s;
-   int   i;
-   int   j;
-   int   k;
-   float out[4];
+   float trace = m->v[0][0] + m->v[1][1] + m->v[2][2] + 1.0f;
 
-   static int next[3] = { 1, 2, 0 };
-
-   diag = m->v[0][0] + m->v[1][1] + m->v[2][2];
-
-   if (diag > 0.0f) {
-      s    = (float)(sqrt(diag + 1.0));
-      q->w = s / 2.0;
-      s    = 0.5 / s;
-      q->x = (m->v[1][2] - m->v[2][1]) * s;
-      q->y = (m->v[2][0] - m->v[0][2]) * s;
-      q->z = (m->v[0][1] - m->v[1][0]) * s;
+   if (trace > EPSILON) {
+      float s = 0.5f / (float)sqrt(trace);
+      q->w = 0.25f / s;
+      q->x = (m->v[2][1] - m->v[1][2]) * s;
+      q->y = (m->v[0][2] - m->v[2][0]) * s;
+      q->z = (m->v[1][0] - m->v[0][1]) * s;
    }
    else {
-      i = 0;
-
-      if (m->v[1][1] > m->v[0][0]) {
-	 i = 1;
+      if (m->v[0][0] > m->v[1][1] && m->v[0][0] > m->v[2][2]) {
+         float s = 2.0f * (float)sqrt(1.0f + m->v[0][0] - m->v[1][1] - m->v[2][2]);
+         q->x = 0.25f * s;
+         q->y = (m->v[0][1] + m->v[1][0]) / s;
+         q->z = (m->v[0][2] + m->v[2][0]) / s;
+         q->w = (m->v[1][2] - m->v[2][1]) / s;
       }
-
-      if (m->v[2][2] > m->v[i][i]) {
-	 i = 2;
+      else if (m->v[1][1] > m->v[2][2]) {
+         float s = 2.0f * (float)sqrt(1.0f + m->v[1][1] - m->v[0][0] - m->v[2][2]);
+         q->x = (m->v[0][1] + m->v[1][0]) / s;
+         q->y = 0.25f * s;
+         q->z = (m->v[1][2] + m->v[2][1]) / s;
+         q->w = (m->v[0][2] - m->v[2][0]) / s;
+      }
+      else {
+         float s = 2.0f * (float)sqrt(1.0f + m->v[2][2] - m->v[0][0] - m->v[1][1]);
+         q->x = (m->v[0][2] + m->v[2][0]) / s;
+         q->y = (m->v[1][2] + m->v[2][1]) / s;
+         q->z = 0.25f * s;
+         q->w = (m->v[0][1] - m->v[1][0]) / s;
       }
-
-      j = next[i];
-      k = next[j];
-
-      s = m->v[i][i] - (m->v[j][j] + m->v[k][k]);
-
-      /* NOTE: Passing non-orthonormalized matrices can result in odd things
-       *       happening, like the calculation of s below trying to find the
-       *       square-root of a negative number, which is imaginary.  Some
-       *       implementations of sqrt will crash, while others return this
-       *       as not-a-number (NaN). NaN could be very subtle because it will
-       *       not throw an exception on Intel processors.
-       */
-      ASSERT(s > 0.0);
-
-      s = (float)(sqrt(s) + 1.0);
-
-      out[i] = s / 2.0;
-      s      = 0.5 / s;
-      out[j] = (m->v[i][j] + m->v[j][i]) * s;
-      out[k] = (m->v[i][k] + m->v[k][i]) * s;
-      out[3] = (m->v[j][k] - m->v[k][j]) * s;
-
-      q->x = out[0];
-      q->y = out[1];
-      q->z = out[2];
-      q->w = out[3];
    }
 }
 
@@ -320,6 +302,9 @@ void matrix_to_quat(AL_CONST MATRIX_f *m, QUAT *q)
  */
 static INLINE void quat_conjugate(AL_CONST QUAT *q, QUAT *out)
 {
+   ASSERT(q);
+   ASSERT(out);
+
    /* q^* = w - x - y - z */
    out->w =  (q->w);
    out->x = -(q->x);
@@ -334,6 +319,8 @@ static INLINE void quat_conjugate(AL_CONST QUAT *q, QUAT *out)
  */
 static INLINE float quat_normal(AL_CONST QUAT *q)
 {
+   ASSERT(q);
+
    /* N(q) = ww + xx + yy + zz */
    return ((q->w * q->w) + (q->x * q->x) + (q->y * q->y) + (q->z * q->z));
 }
@@ -347,6 +334,8 @@ static INLINE void quat_inverse(AL_CONST QUAT *q, QUAT *out)
 {
    QUAT  con;
    float norm;
+   ASSERT(q);
+   ASSERT(out);
 
    /* q^-1 = q^* / N(q) */
 
@@ -379,6 +368,10 @@ void apply_quat(AL_CONST QUAT *q, float x, float y, float z, float *xout, float
    QUAT v;
    QUAT i;
    QUAT t;
+   ASSERT(q);
+   ASSERT(xout);
+   ASSERT(yout);
+   ASSERT(zout);
 
    /* v' = q * v * q^-1 */
 
@@ -391,7 +384,7 @@ void apply_quat(AL_CONST QUAT *q, float x, float y, float z, float *xout, float
     *       by the fact that the inverse of a zero quaternion is undefined
     *       and therefore causes an exception below.
     */
-   ASSERT(!((x == 0) && (y == 0) && (z == 0)));
+   ASSERT(!((q->x == 0.0) && (q->y == 0.0) && (q->z == 0.0) && (q->w == 0.0)));
 
    quat_inverse(q, &i);
    quat_mul(&i, &v, &t);
@@ -429,6 +422,9 @@ void quat_slerp(AL_CONST QUAT *from, AL_CONST QUAT *to, float t, QUAT *out, int
    double scale_from;
    double scale_to;
    double sin_angle;
+   ASSERT(from);
+   ASSERT(to);
+   ASSERT(out);
 
    cos_angle = (from->x * to->x) +
 	       (from->y * to->y) +
