@@ -48,16 +48,16 @@ void register_bitmap_file_type(AL_CONST char *ext, BITMAP *(*load)(AL_CONST char
    if (strlen(aext) == 0) return;
 
    if (!iter) 
-      iter = bitmap_type_list = malloc(sizeof(struct BITMAP_TYPE_INFO));
+      iter = bitmap_type_list = _AL_MALLOC(sizeof(struct BITMAP_TYPE_INFO));
    else {
       for (iter = bitmap_type_list; iter->next; iter = iter->next);
-      iter = iter->next = malloc(sizeof(struct BITMAP_TYPE_INFO));
+      iter = iter->next = _AL_MALLOC(sizeof(struct BITMAP_TYPE_INFO));
    }
 
    if (iter) {
       iter->load = load;
       iter->save = save;
-      iter->ext = strdup(aext);
+      iter->ext = _al_strdup(aext);
       iter->next = NULL;
    }
 }
@@ -71,6 +71,7 @@ BITMAP *load_bitmap(AL_CONST char *filename, RGB *pal)
 {
    char tmp[32], *aext;
    BITMAP_TYPE_INFO *iter;
+   ASSERT(filename);
 
    aext = uconvert_toascii(get_extension(filename), tmp);
    
@@ -94,12 +95,14 @@ int save_bitmap(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal)
 {
    char tmp[32], *aext;
    BITMAP_TYPE_INFO *iter;
+   ASSERT(filename);
+   ASSERT(bmp);
 
    aext = uconvert_toascii(get_extension(filename), tmp);
 
    for (iter = bitmap_type_list; iter; iter = iter->next) {
       if (stricmp(iter->ext, aext) == 0) {
-	 if (iter->load)
+	 if (iter->save)
 	    return iter->save(filename, bmp, pal);
 	 return 1;
       }
@@ -122,6 +125,7 @@ int save_bitmap(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal)
 BITMAP *_fixup_loaded_bitmap(BITMAP *bmp, PALETTE pal, int bpp)
 {
    BITMAP *b2;
+   ASSERT(bmp);
 
    b2 = create_bitmap_ex(bpp, bmp->w, bmp->h);
    if (!b2) {
@@ -137,14 +141,14 @@ BITMAP *_fixup_loaded_bitmap(BITMAP *bmp, PALETTE pal, int bpp)
       else
 	 pal = _current_palette;
 
-      rgb_map = malloc(sizeof(RGB_MAP));
+      rgb_map = _AL_MALLOC(sizeof(RGB_MAP));
       if (rgb_map != NULL)
 	 create_rgb_table(rgb_map, pal, NULL);
 
       blit(bmp, b2, 0, 0, 0, 0, bmp->w, bmp->h);
 
       if (rgb_map != NULL)
-	 free(rgb_map);
+	 _AL_FREE(rgb_map);
       rgb_map = old_map;
    }
    else if (bitmap_color_depth(bmp) == 8) {
@@ -172,8 +176,8 @@ static void register_bitmap_file_type_exit(void)
 
    while (iter) {
       next = iter->next;
-      free(iter->ext);
-      free(iter);
+      _AL_FREE(iter->ext);
+      _AL_FREE(iter);
       iter = next;
    }
    
@@ -199,7 +203,8 @@ void _register_bitmap_file_type_init(void)
 {
    char buf[32];
 
-   _add_exit_func(register_bitmap_file_type_exit);
+   _add_exit_func(register_bitmap_file_type_exit,
+		  "register_bitmap_file_type_exit");
 
    register_bitmap_file_type(uconvert_ascii("bmp", buf), load_bmp, save_bmp);
    register_bitmap_file_type(uconvert_ascii("lbm", buf), load_lbm, NULL);
@@ -235,8 +240,8 @@ void _register_bitmap_file_type_init(void)
 
       while (iter) {
          next = iter->next;
-         free(iter->ext);
-         free(iter);
+         _AL_FREE(iter->ext);
+         _AL_FREE(iter);
          iter = next;
       }
    
