@@ -28,12 +28,12 @@
 #include <unistd.h>
 #include <dmedia/audio.h>
 
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
 #include <pthread.h>
 #endif
 
 #define _AL_SGIAL_PORTSIZE 12288
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
    #define _AL_SGIAL_BUFFERSIZE 2048
 #else
    #define _AL_SGIAL_BUFFERSIZE 4096
@@ -53,7 +53,7 @@ static int _al_sgial_buffer_size(void);
 
 static char _al_sgial_desc[256] = EMPTY_STRING;
 
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
 static pthread_t thread;
 #endif
 
@@ -72,6 +72,7 @@ DIGI_DRIVER digi_sgial =
    _al_sgial_init,
    _al_sgial_exit,
    _al_sgial_mixer_volume,
+   NULL,
 
    NULL,
    NULL,
@@ -124,7 +125,7 @@ static int _al_sgial_buffer_size(void)
 
 
 
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
 
 /* _al_sgial_puller_thread_func: [dedicated thread]
  *  We have threads, therefore we can use a thread to pull sound data
@@ -232,7 +233,7 @@ static int _al_sgial_init(int input, int voices)
    }
 
    _al_sgial_bufsize = _AL_SGIAL_BUFFERSIZE;
-   _al_sgial_bufdata = malloc(_al_sgial_bufsize*((_al_sgial_bits==AL_SAMPLE_16) ? 2 : 1)*((_al_sgial_stereo==AL_STEREO) ? 2 : 1));
+   _al_sgial_bufdata = _AL_MALLOC_ATOMIC(_al_sgial_bufsize*((_al_sgial_bits==AL_SAMPLE_16) ? 2 : 1)*((_al_sgial_stereo==AL_STEREO) ? 2 : 1));
    if (!_al_sgial_bufdata) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Can not allocate audio buffer"));
       alClosePort(_al_sgial_port);
@@ -246,7 +247,7 @@ static int _al_sgial_init(int input, int voices)
 		   ((_al_sgial_stereo == AL_STEREO) ? 1 : 0), ((_al_sgial_bits == AL_SAMPLE_16) ? 1 : 0),
 		   &digi_sgial.voices) != 0) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Can not init software mixer"));
-      free(_al_sgial_bufdata);
+      _AL_FREE(_al_sgial_bufdata);
       alClosePort(_al_sgial_port);
       alFreeConfig(_al_sgial_config);
       return -1;
@@ -254,7 +255,7 @@ static int _al_sgial_init(int input, int voices)
 
    _mix_some_samples((uintptr_t) _al_sgial_bufdata, 0, _al_sgial_signed);
 
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
    /* Add audio thread. */
    pthread_create(&thread, NULL, _al_sgial_puller_thread_func, NULL);
 #else
@@ -282,13 +283,13 @@ static void _al_sgial_exit(int input)
    if (input)
       return;
 
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
    pthread_cancel(thread);
 #else
    _unix_bg_man->unregister_func(_al_sgial_update);
 #endif
 
-   free(_al_sgial_bufdata);
+   _AL_FREE(_al_sgial_bufdata);
    _al_sgial_bufdata = NULL;
 
    _mixer_exit();
