@@ -40,11 +40,17 @@ typedef struct MODULE
 /* list of loaded modules */
 static MODULE *module_list = NULL;
 
+#define PREFIX_I "al-unix INFO: "
 
 /* where to look for modules.lst */
 static char *module_path[] =
 {
-   "/usr/local/lib/allegro/", "/usr/lib/allegro/", NULL
+#ifdef ALLEGRO_MODULES_PATH
+   ALLEGRO_MODULES_PATH,
+#else
+   #error ALLEGRO_WITH_MODULES is defined, but not ALLEGRO_MODULES_PATH
+#endif
+   NULL
 };
 
 
@@ -94,8 +100,10 @@ void _unix_load_modules(int system_driver)
    }
 
    for (pathptr = module_path; *pathptr; pathptr++) {
-      snprintf(fullpath, sizeof fullpath, "%s/%d.%d/modules.lst",
-	       *pathptr, ALLEGRO_VERSION, ALLEGRO_SUB_VERSION);
+      /* ALLEGRO_MODULES_PATH already includes the version,
+       * as should ALLEGRO_MODULES.
+       */
+      snprintf(fullpath, sizeof fullpath, "%s/modules.lst", *pathptr);
       fullpath[(sizeof fullpath) - 1] = 0;
       f = pack_fopen(uconvert_ascii(fullpath, buf), F_READ);
       if (f) goto found;
@@ -105,6 +113,8 @@ void _unix_load_modules(int system_driver)
 
    found:
 
+   TRACE(PREFIX_I "Loading modules from \"%s\".\n", fullpath);
+
    fullpath_slash = strrchr(fullpath, '/');
    
    while (TRUE) {
@@ -138,7 +148,7 @@ void _unix_load_modules(int system_driver)
       if (init)
          init(system_driver);
 
-      m = malloc(sizeof(MODULE));
+      m = _AL_MALLOC(sizeof(MODULE));
       if (m) {
 	 m->handle = handle;
 	 m->next = module_list;
@@ -180,7 +190,7 @@ void _unix_unload_modules(void)
       if (!has_registered || _allegro_in_exit)
 	 dlclose(m->handle);
 
-      free(m);
+      _AL_FREE(m);
    }
    
    module_list = NULL;
