@@ -18,6 +18,10 @@
 
 #include "wddraw.h"
 
+#define PREFIX_I                "al-wddovl INFO: "
+#define PREFIX_W                "al-wddovl WARNING: "
+#define PREFIX_E                "al-wddovl ERROR: "
+
 
 static int gfx_directx_show_video_bitmap_ovl(struct BITMAP *bitmap);
 static int gfx_directx_request_video_bitmap_ovl(struct BITMAP *bitmap);
@@ -52,6 +56,7 @@ GFX_DRIVER gfx_directx_ovl =
    NULL,                        // AL_METHOD(void, drawing_mode, (void));
    NULL,                        // AL_METHOD(void, save_video_state, (void*));
    NULL,                        // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                        // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    NULL,                        // AL_METHOD(int, fetch_mode_list, (void));
    0, 0,                        // int w, h;                     /* physical (not virtual!) screen size */
    TRUE,                        // int linear;                   /* true if video memory is linear */
@@ -84,9 +89,9 @@ static WIN_GFX_DRIVER win_gfx_driver_overlay =
 
 
 static char gfx_driver_desc[256] = EMPTY_STRING;
-static LPDIRECTDRAWSURFACE2 overlay_surface = NULL;
-static BITMAP *viewport = NULL, *background = NULL;
+static DDRAW_SURFACE *overlay_surface = NULL;
 static BOOL overlay_visible = FALSE;
+static HBRUSH overlay_brush;
 
 
 
@@ -95,41 +100,34 @@ static BOOL overlay_visible = FALSE;
  */
 static void switch_in_overlay(void)
 {
-   /* restore all DirectDraw surfaces */
-   _enter_gfx_critical();
-
-   IDirectDrawSurface2_Restore(dd_prim_surface);
-
-   gfx_directx_restore();
-
+   restore_all_ddraw_surfaces();
    show_overlay();
-
-   _exit_gfx_critical();
 }
 
 
 
 /* update_overlay:
- *  moves and resizes the overlay surface to make it fit into the specified area
+ *  Moves and resizes the overlay surface to make it fit
+ *  into the specified area.
  */
 static int update_overlay(int x, int y, int w, int h)
 {
    HRESULT hr;
-   RECT dest_rect = {x, y, x + w, y + h};
+   RECT dest_rect;
 
-   /* paint the window background with the overlay color key */
-   background->x_ofs = x + viewport->x_ofs;
-   background->y_ofs = y + viewport->y_ofs;
-   clear_to_color(background, background->vtable->mask_color);
+   dest_rect.left = x;
+   dest_rect.top = y;
+   dest_rect.right = x + w;
+   dest_rect.bottom = y + h;
 
    /* show the overlay surface */
-   hr = IDirectDrawSurface2_UpdateOverlay(overlay_surface, NULL,
-                                          dd_prim_surface, &dest_rect,
+   hr = IDirectDrawSurface2_UpdateOverlay(overlay_surface->id, NULL,
+                                          gfx_directx_primary_surface->id, &dest_rect,
                                           DDOVER_SHOW | DDOVER_KEYDEST, NULL);
    if (FAILED(hr)) {
-      _TRACE("Can't display overlay (%x)\n", hr);
-      IDirectDrawSurface2_UpdateOverlay(overlay_surface, NULL,
-	                                dd_prim_surface, NULL,
+      _TRACE(PREFIX_E "Can't display overlay (%x)\n", hr);
+      IDirectDrawSurface2_UpdateOverlay(overlay_surface->id, NULL,
+                                        gfx_directx_primary_surface->id, NULL,
                                         DDOVER_HIDE, NULL);
       return -1;
    }
@@ -140,7 +138,7 @@ static int update_overlay(int x, int y, int w, int h)
 
 
 /* show_overlay:
- *  makes the overlay surface visible on the primary surface
+ *  Makes the overlay surface visible on the primary surface.
  */
 static void show_overlay(void)
 {
@@ -153,15 +151,16 @@ static void show_overlay(void)
 
 
 /* move_overlay:
- *  moves the overlay surface
+ *  Moves the overlay surface.
  */
 static void move_overlay(int x, int y, int w, int h)
 {
    RECT window_rect;
+   HWND allegro_wnd = win_get_window();
    int xmod;
 
    /* handle hardware limitations */
-   if ((dd_caps.dwCaps & DDCAPS_ALIGNBOUNDARYDEST) && (xmod = x%dd_caps.dwAlignBoundaryDest)) {
+   if ((ddcaps.dwCaps & DDCAPS_ALIGNBOUNDARYDEST) && (xmod = x%ddcaps.dwAlignBoundaryDest)) {
       GetWindowRect(allegro_wnd, &window_rect);
       SetWindowPos(allegro_wnd, 0, window_rect.left + xmod, window_rect.top,
                    0, 0, SWP_NOZORDER | SWP_NOSIZE);
@@ -173,15 +172,15 @@ static void move_overlay(int x, int y, int w, int h)
 
 
 /* hide_overlay:
- *  makes the overlay surface invisible on the primary surface
+ *  Makes the overlay surface invisible on the primary surface.
  */
 static void hide_overlay(void)
 {
    if (overlay_visible) {
       overlay_visible = FALSE;
 
-      IDirectDrawSurface2_UpdateOverlay(overlay_surface, NULL,
-	                                dd_prim_surface, NULL,
+      IDirectDrawSurface2_UpdateOverlay(overlay_surface->id, NULL,
+                                        gfx_directx_primary_surface->id, NULL,
                                         DDOVER_HIDE, NULL);
    }
 }
@@ -193,25 +192,26 @@ static void hide_overlay(void)
  */
 static void paint_overlay(RECT *rect)
 {
-   /* we may have lost the DirectDraw surfaces
-    * (e.g after the monitor has gone to low power)
+   /* We may have lost the DirectDraw surfaces, for example
+    * after the monitor has gone to low power.
     */
-   if (IDirectDrawSurface2_IsLost(dd_prim_surface))
+   if (IDirectDrawSurface2_IsLost(gfx_directx_primary_surface->id))
       switch_in_overlay();
 }
 
 
 
 /* wnd_set_windowed_coop:
- *  sets the DirectDraw cooperative level to normal
+ *  Sets the DirectDraw cooperative level to normal.
  */
 static int wnd_set_windowed_coop(void)
 {
    HRESULT hr;
+   HWND allegro_wnd = win_get_window();
 
    hr = IDirectDraw2_SetCooperativeLevel(directdraw, allegro_wnd, DDSCL_NORMAL);
    if (FAILED(hr)) {
-      _TRACE("SetCooperative level = %s (%x), hwnd = %x\n", win_err_str(hr), hr, allegro_wnd);
+      _TRACE(PREFIX_E "SetCooperative level = %s (%x), hwnd = %x\n", win_err_str(hr), hr, allegro_wnd);
       return -1;
    }
 
@@ -220,16 +220,16 @@ static int wnd_set_windowed_coop(void)
 
 
 
-/* setup_driver_desc:
- *  sets up the driver description string
+/* gfx_directx_setup_driver_desc:
+ *  Sets up the driver description string.
  */
-static void setup_driver_desc(void)
+static void gfx_directx_setup_driver_desc(void)
 {
    char tmp[256];
 
    uszprintf(gfx_driver_desc, sizeof(gfx_driver_desc), 
 	     uconvert_ascii("DirectDraw, in matching, %d bpp overlay", tmp),
-	     desktop_depth);
+	     _win_desktop_depth);
    
    gfx_directx_ovl.desc = gfx_driver_desc;
 }
@@ -237,11 +237,10 @@ static void setup_driver_desc(void)
 
 
 /* gfx_directx_show_video_bitmap_ovl:
- *  driver-specific method for show_video_bitmap()
  */
 static int gfx_directx_show_video_bitmap_ovl(struct BITMAP *bitmap)
 {
-   if (gfx_directx_show_video_bitmap(bitmap)) {
+   if (gfx_directx_show_video_bitmap(bitmap) != 0) {
       return -1;
    }
    else {
@@ -254,11 +253,10 @@ static int gfx_directx_show_video_bitmap_ovl(struct BITMAP *bitmap)
 
 
 /* gfx_directx_request_video_bitmap_ovl:
- *  driver-specific method for request_video_bitmap()
  */
 static int gfx_directx_request_video_bitmap_ovl(struct BITMAP *bitmap)
 {
-   if (gfx_directx_request_video_bitmap(bitmap)) {
+   if (gfx_directx_request_video_bitmap(bitmap) != 0) {
       return -1;
    }
    else {
@@ -271,14 +269,14 @@ static int gfx_directx_request_video_bitmap_ovl(struct BITMAP *bitmap)
 
 
 /* create_overlay:
- *  creates the overlay surface
+ *  Creates the overlay surface.
  */
 static int create_overlay(int w, int h, int color_depth)
 {
-   overlay_surface = gfx_directx_create_surface(w, h, dd_pixelformat, SURF_OVERLAY);
+   overlay_surface = gfx_directx_create_surface(w, h, ddpixel_format, DDRAW_SURFACE_OVERLAY);
    
    if (!overlay_surface) {
-      _TRACE("Can't create overlay surface.\n");
+      _TRACE(PREFIX_E "Can't create overlay surface.\n");
       return -1;
    }
 
@@ -288,12 +286,13 @@ static int create_overlay(int w, int h, int color_depth)
 
 
 /* init_directx_ovl:
- *  initializes the driver
+ *  Initializes the driver.
  */
 static struct BITMAP *init_directx_ovl(int w, int h, int v_w, int v_h, int color_depth)
 {
    HRESULT hr;
    DDCOLORKEY key;
+   HWND allegro_wnd = win_get_window();
 
    /* overlay would allow scrolling on some cards, but it isn't implemented yet */
    if ((v_w != w && v_w != 0) || (v_h != h && v_h != 0)) {
@@ -306,7 +305,7 @@ static struct BITMAP *init_directx_ovl(int w, int h, int v_w, int v_h, int color
    /* init DirectX */
    if (init_directx() != 0)
       goto Error;
-   if ((dd_caps.dwCaps & DDCAPS_OVERLAY) == 0) {
+   if ((ddcaps.dwCaps & DDCAPS_OVERLAY) == 0) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Overlays not supported"));
       goto Error;
    }
@@ -318,41 +317,45 @@ static struct BITMAP *init_directx_ovl(int w, int h, int v_w, int v_h, int color
       goto Error;
    if (finalize_directx_init() != 0)
       goto Error;
-  
+
+   /* Paint window background with overlay color key. */
+   overlay_brush = CreateSolidBrush(MASK_COLOR_32);
+   SetClassLong(allegro_wnd, GCL_HBRBACKGROUND, (LONG) overlay_brush);
+
    if (adjust_window(w, h) != 0) {
-      _TRACE("window size not supported.\n");
+      _TRACE(PREFIX_E "window size not supported.\n");
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Resolution not supported"));
       goto Error;
    }
 
    /* create surfaces */
-   if (create_primary(FALSE) != 0)
+   if (gfx_directx_create_primary() != 0)
       goto Error;
 
    if (create_overlay(w, h, color_depth) != 0)
       goto Error;
 
-   /* handle hardware limitations: according to the DirectX SDK, "these restrictions
+   /* Handle hardware limitations: according to the DirectX SDK, "these restrictions
     * can vary depending on the pixel formats of the overlay and primary surface", so
-    * we handle them after creating the surfaces
+    * we handle them after creating the surfaces.
     */
-   dd_caps.dwSize = sizeof(dd_caps);
-   hr = IDirectDraw2_GetCaps(directdraw, &dd_caps, NULL);
+   ddcaps.dwSize = sizeof(ddcaps);
+   hr = IDirectDraw2_GetCaps(directdraw, &ddcaps, NULL);
    if (FAILED(hr)) {
-      _TRACE("Can't get driver caps\n");
+      _TRACE(PREFIX_E "Can't get driver caps\n");
       goto Error;
    }
 
-   if (dd_caps.dwCaps & DDCAPS_ALIGNSIZESRC) {
-      if (w%dd_caps.dwAlignSizeSrc) {
-         _TRACE("Alignment requirement not met: source size %d\n", dd_caps.dwAlignSizeSrc);
+   if (ddcaps.dwCaps & DDCAPS_ALIGNSIZESRC) {
+      if (w%ddcaps.dwAlignSizeSrc) {
+         _TRACE(PREFIX_E "Alignment requirement not met: source size %d\n", ddcaps.dwAlignSizeSrc);
          ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Resolution not supported"));
          goto Error;
       }
    } 
-   else if (dd_caps.dwCaps & DDCAPS_ALIGNSIZEDEST) {
-      if (w%dd_caps.dwAlignSizeDest) {
-         _TRACE("Alignment requirement not met: dest size %d\n", dd_caps.dwAlignSizeDest);
+   else if (ddcaps.dwCaps & DDCAPS_ALIGNSIZEDEST) {
+      if (w%ddcaps.dwAlignSizeDest) {
+         _TRACE(PREFIX_E "Alignment requirement not met: dest size %d\n", ddcaps.dwAlignSizeDest);
          ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Resolution not supported"));
          goto Error;
       }
@@ -360,7 +363,7 @@ static struct BITMAP *init_directx_ovl(int w, int h, int v_w, int v_h, int color
 
    /* set color format */
    if (color_depth == 8) {
-      if (create_palette(overlay_surface) != 0)
+      if (gfx_directx_create_palette(overlay_surface) != 0)
 	 goto Error;
    }
    else {
@@ -369,32 +372,35 @@ static struct BITMAP *init_directx_ovl(int w, int h, int v_w, int v_h, int color
    }
 
    /* set up Allegro gfx driver */
-   setup_driver_desc();
+   gfx_directx_setup_driver_desc();
 
-   if (setup_driver(&gfx_directx_ovl, w, h, color_depth) != 0)
+   if (gfx_directx_setup_driver(&gfx_directx_ovl, w, h, color_depth) != 0)
       goto Error;
 
-   dd_frontbuffer = make_directx_bitmap(overlay_surface, w, h, BMP_ID_VIDEO);
-   viewport = make_directx_bitmap(dd_prim_surface, w, h, BMP_ID_VIDEO);
-   background = create_sub_bitmap(viewport, 0, 0, w, h);
-
-   /* display the overlay surface */
-   key.dwColorSpaceLowValue = dd_frontbuffer->vtable->mask_color;
-   key.dwColorSpaceHighValue = dd_frontbuffer->vtable->mask_color;
-
-   hr = IDirectDrawSurface2_SetColorKey(dd_prim_surface, DDCKEY_DESTOVERLAY, &key);
-   if (FAILED(hr)) {
-      _TRACE("Can't set overlay dest color key\n");
+   gfx_directx_forefront_bitmap = gfx_directx_make_bitmap_from_surface(overlay_surface, w, h, BMP_ID_VIDEO);
+   if (!gfx_directx_forefront_bitmap) {
+      _TRACE(PREFIX_E "Can't make a bitmap from the surface\n");
       goto Error;
    }
 
+   /* set the overlay color key */
+   key.dwColorSpaceLowValue = gfx_directx_forefront_bitmap->vtable->mask_color;
+   key.dwColorSpaceHighValue = gfx_directx_forefront_bitmap->vtable->mask_color;
+
+   hr = IDirectDrawSurface2_SetColorKey(gfx_directx_primary_surface->id, DDCKEY_DESTOVERLAY, &key);
+   if (FAILED(hr)) {
+      _TRACE(PREFIX_E "Can't set overlay dest color key\n");
+      goto Error;
+   }
+
+   /* display the overlay surface */
    show_overlay();
 
    /* use hardware accelerated primitives */
-   enable_acceleration(&gfx_directx_ovl);
+   gfx_directx_enable_acceleration(&gfx_directx_ovl);
 
    /* use triple buffering */
-   enable_triple_buffering(&gfx_directx_ovl);
+   gfx_directx_enable_triple_buffering(&gfx_directx_ovl);
 
    /* connect to the system driver */
    win_gfx_driver = &win_gfx_driver_overlay;
@@ -407,7 +413,7 @@ static struct BITMAP *init_directx_ovl(int w, int h, int v_w, int v_h, int color
 
    _exit_critical();
 
-   return dd_frontbuffer;
+   return gfx_directx_forefront_bitmap;
 
  Error:
    _exit_critical();
@@ -421,10 +427,11 @@ static struct BITMAP *init_directx_ovl(int w, int h, int v_w, int v_h, int color
 
 
 /* gfx_directx_ovl_exit:
- *  shuts down the driver
+ *  Shuts down the driver.
  */
 static void gfx_directx_ovl_exit(struct BITMAP *bmp)
 {
+   HWND allegro_wnd = win_get_window();
    _enter_gfx_critical();
 
    if (bmp) {
@@ -438,21 +445,11 @@ static void gfx_directx_ovl_exit(struct BITMAP *bmp)
    /* destroy the overlay surface */
    if (overlay_surface) {
       hide_overlay();
-      gfx_directx_destroy_surf(overlay_surface);
+      SetClassLong(allegro_wnd, GCL_HBRBACKGROUND, (LONG) NULL);
+      DeleteObject(overlay_brush);
+      gfx_directx_destroy_surface(overlay_surface);
       overlay_surface = NULL;
-      destroy_bitmap(background);
-   }
-
-   /* destroy the viewport */
-   if (viewport) {
-      free(viewport);
-      viewport = NULL;
-   }
-
-   /* unregister bitmap */
-   if (bmp) {
-      unregister_directx_bitmap(bmp);
-      free(bmp->extra);
+      gfx_directx_forefront_bitmap = NULL;
    }
 
    gfx_directx_exit(NULL);
