@@ -311,9 +311,9 @@ static void mouse_dinput_handle_event(int ofs, int data)
 static void mouse_dinput_handle(void)
 {
    static DIDEVICEOBJECTDATA message_buffer[DINPUT_BUFFERSIZE];
-   long int waiting_messages;
+   unsigned long int waiting_messages;
    HRESULT hr;
-   int i;
+   unsigned long int i;
 
    /* the whole buffer is free */
    waiting_messages = DINPUT_BUFFERSIZE;
@@ -371,8 +371,8 @@ static void mouse_dinput_handle(void)
          if ((_mouse_x < mouse_minx) || (_mouse_x > mouse_maxx) ||
              (_mouse_y < mouse_miny) || (_mouse_y > mouse_maxy)) {
 
-            _mouse_x = MID(mouse_minx, _mouse_x, mouse_maxx);
-            _mouse_y = MID(mouse_miny, _mouse_y, mouse_maxy);
+            _mouse_x = CLAMP(mouse_minx, _mouse_x, mouse_maxx);
+            _mouse_y = CLAMP(mouse_miny, _mouse_y, mouse_maxy);
 
             mouse_mx = COORD_TO_MICKEY_X(_mouse_x);
             mouse_my = COORD_TO_MICKEY_Y(_mouse_y);
@@ -606,7 +606,11 @@ static int mouse_dinput_init(void)
    };
 
    /* Get DirectInput interface */
-   hr = DirectInputCreate(allegro_inst, DIRECTINPUT_VERSION, &mouse_dinput, NULL);
+   hr = CoCreateInstance(&CLSID_DirectInput, NULL, CLSCTX_INPROC_SERVER, &IID_IDirectInput, &mouse_dinput);
+   if (FAILED(hr))
+      goto Error;
+
+   hr = IDirectInput_Initialize(mouse_dinput, allegro_inst, DIRECTINPUT_VERSION);
    if (FAILED(hr))
       goto Error;
 
@@ -734,8 +738,8 @@ static void mouse_directx_set_range(int x1, int y1, int x2, int y2)
 
    _enter_critical();
 
-   _mouse_x = MID(mouse_minx, _mouse_x, mouse_maxx);
-   _mouse_y = MID(mouse_miny, _mouse_y, mouse_maxy);
+   _mouse_x = CLAMP(mouse_minx, _mouse_x, mouse_maxx);
+   _mouse_y = CLAMP(mouse_miny, _mouse_y, mouse_maxy);
 
    mouse_mx = COORD_TO_MICKEY_X(_mouse_x);
    mouse_my = COORD_TO_MICKEY_Y(_mouse_y);
