@@ -30,19 +30,27 @@
 #error something is wrong with the makefile
 #endif
 
+/* DMC requires a DllMain() function, or else the DLL hangs. */
+#ifndef ALLEGRO_STATICLINK
+BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason, LPVOID lpReserved)
+{
+   return TRUE;
+}
+#endif
+
 
 static int sys_directx_init(void);
 static void sys_directx_exit(void);
 static void sys_directx_get_executable_name(char *output, int size);
 static void sys_directx_set_window_title(AL_CONST char *name);
-static int sys_directx_set_window_close_button(int enable);
-static void sys_directx_set_window_close_hook(void (*proc)(void));
+static int sys_directx_set_close_button_callback(void (*proc)(void));
 static void sys_directx_message(AL_CONST char *msg);
 static void sys_directx_assert(AL_CONST char *msg);
 static void sys_directx_save_console_state(void);
 static void sys_directx_restore_console_state(void);
 static int sys_directx_desktop_color_depth(void);
 static int sys_directx_get_desktop_resolution(int *width, int *height);
+static void sys_directx_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
 static void sys_directx_yield_timeslice(void);
 static int sys_directx_trace_handler(AL_CONST char *msg);
 
@@ -59,8 +67,7 @@ SYSTEM_DRIVER system_directx =
    sys_directx_get_executable_name,
    NULL,                        /* AL_METHOD(int, find_resource, (char *dest, char *resource, int size)); */
    sys_directx_set_window_title,
-   sys_directx_set_window_close_button,
-   sys_directx_set_window_close_hook,
+   sys_directx_set_close_button_callback,
    sys_directx_message,
    sys_directx_assert,
    sys_directx_save_console_state,
@@ -74,12 +81,15 @@ SYSTEM_DRIVER system_directx =
    NULL,                        /* AL_METHOD(void, set_palette_range, (PALETTE p, int from, int to, int vsync)); */
    NULL,                        /* AL_METHOD(struct GFX_VTABLE *, get_vtable, (int color_depth)); */
    sys_directx_set_display_switch_mode,
-   sys_directx_set_display_switch_callback,
-   sys_directx_remove_display_switch_callback,
    NULL,                        /* AL_METHOD(void, display_switch_lock, (int lock)); */
    sys_directx_desktop_color_depth,
    sys_directx_get_desktop_resolution,
+   sys_directx_get_gfx_safe_mode,
    sys_directx_yield_timeslice,
+   sys_directx_create_mutex,
+   sys_directx_destroy_mutex,
+   sys_directx_lock_mutex,
+   sys_directx_unlock_mutex,
    NULL,                        /* AL_METHOD(_DRIVER_INFO *, gfx_drivers, (void)); */
    _get_win_digi_driver_list,   /* AL_METHOD(_DRIVER_INFO *, digi_drivers, (void)); */
    _get_win_midi_driver_list,   /* AL_METHOD(_DRIVER_INFO *, midi_drivers, (void)); */
@@ -122,7 +132,7 @@ static int sys_directx_init(void)
    HANDLE current_process;
 
    /* init thread */
-   win_init_thread();
+   _win_thread_init();
 
    allegro_inst = GetModuleHandle(NULL);
 
@@ -140,8 +150,23 @@ static int sys_directx_init(void)
    os_multitasking = TRUE;
 
    if (win_ver < 0x80000000) {
-      if (os_version == 5) {
-         if(os_revision == 1)
+
+      /* Since doesn't exist os_version == 7 or greater yet,
+         these will be detected as Vista instead of NT. */
+      if (os_version >= 6) {
+         /* If in the future a os_revision == 2 or greater comes,
+            it will be detected as Windows 7 instead of Vista. */
+         if (os_revision >= 1)
+	    os_type = OSTYPE_WIN7;
+         else
+	    os_type = OSTYPE_WINVISTA;
+      }
+      else if (os_version == 5) {
+         /* If in the future a os_revision == 3 or greater comes,
+            it will be detected as Win2003 instead of Win2000. */
+         if (os_revision >= 2)
+	    os_type = OSTYPE_WIN2003;
+         else if (os_revision == 1)
 	    os_type = OSTYPE_WINXP;
 	 else
 	    os_type = OSTYPE_WIN2000;
@@ -170,7 +195,8 @@ static int sys_directx_init(void)
    InitializeCriticalSection(&allegro_critical_section);
 
    /* install a Windows specific trace handler */
-   register_trace_handler(sys_directx_trace_handler);
+   if (!_al_trace_handler)
+      register_trace_handler(sys_directx_trace_handler);
 
    /* setup the display switch system */
    sys_directx_display_switch_init();
@@ -208,7 +234,7 @@ static void sys_directx_exit(void)
    DeleteCriticalSection(&allegro_critical_section);
 
    /* shutdown thread */
-   win_exit_thread();
+   _win_thread_exit();
 
    allegro_inst = NULL;
 }
@@ -220,14 +246,14 @@ static void sys_directx_exit(void)
  */
 static void sys_directx_get_executable_name(char *output, int size)
 {
-   char *temp = malloc(size);
+   char *temp = _AL_MALLOC_ATOMIC(size);
 
    if (GetModuleFileName(allegro_inst, temp, size))
       do_uconvert(temp, U_ASCII, output, U_CURRENT, size);
    else
       usetc(output, 0);
 
-   free(temp);
+   _AL_FREE(temp);
 }
 
 
@@ -237,30 +263,33 @@ static void sys_directx_get_executable_name(char *output, int size)
  */
 static void sys_directx_set_window_title(AL_CONST char *name)
 {
+   HWND allegro_wnd = win_get_window();
+
    do_uconvert(name, U_CURRENT, wnd_title, U_ASCII, WND_TITLE_SIZE);
    SetWindowText(allegro_wnd, wnd_title);
 }
 
 
 
-/* sys_directx_set_window_close_button:
- *  Enables or disables the window close button.
+/* sys_directx_set_close_button_callback:
+ *  Sets the close button callback function.
  */
-static int sys_directx_set_window_close_button(int enable)
+static int sys_directx_set_close_button_callback(void (*proc)(void))
 {
    DWORD class_style;
    HMENU sys_menu;
+   HWND allegro_wnd = win_get_window();
 
-   /* we get the old class style */
+   user_close_proc = proc;
+
+   /* get the old class style */
    class_style = GetClassLong(allegro_wnd, GCL_STYLE);
 
    /* and the system menu handle */
    sys_menu = GetSystemMenu(allegro_wnd, FALSE);
 
-   /* disable or enable the no_close_button flag
-    * and the close menu option
-    */
-   if (enable) {
+   /* enable or disable the no_close_button flag and the close menu option */
+   if (proc) {
       class_style &= ~CS_NOCLOSE;
       EnableMenuItem(sys_menu, SC_CLOSE, MF_BYCOMMAND | MF_ENABLED);
    }
@@ -272,8 +301,8 @@ static int sys_directx_set_window_close_button(int enable)
    /* change the class to the new style */
    SetClassLong(allegro_wnd, GCL_STYLE, class_style);
 
-   /* redraw the whole window to display the changes in the button
-    * note we use this because UpdateWindow() only works for the client area
+   /* Redraw the whole window to display the changes of the button.
+    * (we use this because UpdateWindow() only works for the client area)
     */
    RedrawWindow(allegro_wnd, NULL, NULL, RDW_FRAME | RDW_INVALIDATE | RDW_UPDATENOW);
 
@@ -282,23 +311,14 @@ static int sys_directx_set_window_close_button(int enable)
 
 
 
-/* sys_directx_set_window_close_hook:
- *  Sets the close button user hook or restores default one.
- */
-static void sys_directx_set_window_close_hook(void (*proc)(void))
-{
-   user_close_proc = proc;
-}
-
-
-
 /* sys_directx_message:
  *  Displays a message.
  */
 static void sys_directx_message(AL_CONST char *msg)
 {
-   char *tmp1 = malloc(ALLEGRO_MESSAGE_SIZE);
+   char *tmp1 = _AL_MALLOC_ATOMIC(ALLEGRO_MESSAGE_SIZE);
    char tmp2[WND_TITLE_SIZE*2];
+   HWND allegro_wnd = win_get_window();
 
    while ((ugetc(msg) == '\r') || (ugetc(msg) == '\n'))
       msg += uwidth(msg);
@@ -308,39 +328,46 @@ static void sys_directx_message(AL_CONST char *msg)
 	       (unsigned short *)uconvert(wnd_title, U_ASCII, tmp2, U_UNICODE, sizeof(tmp2)),
 	       MB_OK);
 
-   free(tmp1);
+   _AL_FREE(tmp1);
 }
 
 
 
 /* sys_directx_assert
- *  handles assertions
+ *  Handles assertions.
  */
 static void sys_directx_assert(AL_CONST char *msg)
 {
-   OutputDebugString(msg);
+   OutputDebugString(msg);  /* thread safe */
+   abort();
 }
 
 
 
 /* sys_directx_save_console_state:
- *  safes window size
+ *  Saves console window size and position.
  */
 static void sys_directx_save_console_state(void)
 {
+   HWND allegro_wnd = win_get_window();
    GetWindowRect(allegro_wnd, &wnd_rect);
 }
 
 
 
 /* sys_directx_restore_console_state:
- *  restores old window size
+ *  Restores console window size and position.
  */
 static void sys_directx_restore_console_state(void)
 {
+   HWND allegro_wnd = win_get_window();
    /* unacquire input devices */
-   wnd_unacquire_keyboard();
-   wnd_unacquire_mouse();
+   wnd_schedule_proc(key_dinput_unacquire);
+   wnd_schedule_proc(mouse_dinput_unacquire);
+   wnd_schedule_proc(joystick_dinput_unacquire);
+
+   /* reset switch mode */
+   _win_reset_switch_mode();
 
    /* re-size and hide window */
    SetWindowPos(allegro_wnd, HWND_TOP, wnd_rect.left, wnd_rect.top,
@@ -352,7 +379,7 @@ static void sys_directx_restore_console_state(void)
 
 
 /* sys_directx_desktop_color_depth:
- *  returns the current desktop color depth
+ *  Returns the current desktop color depth.
  */
 static int sys_directx_desktop_color_depth(void)
 {
@@ -382,7 +409,7 @@ static int sys_directx_desktop_color_depth(void)
 
 
 /* sys_directx_get_desktop_resolution:
- *  returns the current desktop resolution
+ *  Returns the current desktop resolution.
  */
 static int sys_directx_get_desktop_resolution(int *width, int *height)
 {
@@ -413,8 +440,21 @@ static int sys_directx_get_desktop_resolution(int *width, int *height)
 
 
 
+/* sys_directx_get_gfx_safe_mode:
+ *  Defines the safe graphics mode for this system.
+ */
+static void sys_directx_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
+{
+   *driver = GFX_GDI;
+   mode->width = 320;
+   mode->height = 200;
+   mode->bpp = 8;
+}
+
+
+
 /* sys_directx_yield_timeslice:
- *  yields remaining timeslice portion to the system
+ *  Yields remaining timeslice portion to the system.
  */
 static void sys_directx_yield_timeslice(void)
 {
@@ -424,11 +464,11 @@ static void sys_directx_yield_timeslice(void)
 
 
 /* sys_directx_trace_handler
- *  handles trace output
+ *  Handles trace output.
  */
 static int sys_directx_trace_handler(AL_CONST char *msg)
 {
-   OutputDebugString(msg);
+   OutputDebugString(msg);  /* thread safe */
    return 0;
 }
 
@@ -452,14 +492,14 @@ int _WinMain(void *_main, void *hInst, void *hPrev, char *Cmd, int nShow)
    /* can't use parameter because it doesn't include the executable name */
    cmdline = GetCommandLine();
    i = strlen(cmdline) + 1;
-   argbuf = malloc(i);
+   argbuf = _AL_MALLOC(i);
    memcpy(argbuf, cmdline, i);
 
    argc = 0;
    argc_max = 64;
-   argv = malloc(sizeof(char *) * argc_max);
+   argv = _AL_MALLOC(sizeof(char *) * argc_max);
    if (!argv) {
-      free(argbuf);
+      _AL_FREE(argbuf);
       return 1;
    }
 
@@ -483,9 +523,9 @@ int _WinMain(void *_main, void *hInst, void *hPrev, char *Cmd, int nShow)
 
          if (argc >= argc_max) {
             argc_max += 64;
-            argv = realloc(argv, sizeof(char *) * argc_max);
+            argv = _AL_REALLOC(argv, sizeof(char *) * argc_max);
             if (!argv) {
-               free(argbuf);
+               _AL_FREE(argbuf);
                return 1;
             }
          }
@@ -505,8 +545,8 @@ int _WinMain(void *_main, void *hInst, void *hPrev, char *Cmd, int nShow)
    /* call the application entry point */
    i = mainfunc(argc, argv);
 
-   free(argv);
-   free(argbuf);
+   _AL_FREE(argv);
+   _AL_FREE(argbuf);
 
    return i;
 }
@@ -514,7 +554,7 @@ int _WinMain(void *_main, void *hInst, void *hPrev, char *Cmd, int nShow)
 
 
 /* win_err_str:
- *  returns a error string for a window error
+ *  Returns a error string for a window error.
  */
 char *win_err_str(long err)
 {
@@ -530,7 +570,7 @@ char *win_err_str(long err)
 
 
 /* thread_safe_trace:
- *  outputs trace message inside threads
+ *  Outputs internal trace message.
  */
 void thread_safe_trace(char *msg,...)
 {
@@ -542,5 +582,5 @@ void thread_safe_trace(char *msg,...)
    vsprintf(buf, msg, ap);
    va_end(ap);
 
-   OutputDebugString(buf);
+   OutputDebugString(buf);  /* thread safe */
 }
