@@ -23,10 +23,9 @@
 /* list the available drivers */
 _DRIVER_INFO _xwin_timer_driver_list[] =
 {
-#ifdef TIMERDRV_UNIX_PTHREADS
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
    {  TIMERDRV_UNIX_PTHREADS,  &timerdrv_unix_pthreads, TRUE  },
-#endif
-#ifdef TIMERDRV_UNIX_SIGALRM
+#else
    {  TIMERDRV_UNIX_SIGALRM,   &timerdrv_unix_sigalrm,  TRUE  },
 #endif
    {  0,                       NULL,                    0     }
