@@ -15,6 +15,7 @@
  *      See readme.txt for copyright information.
  */
 
+#define ALLEGRO_USE_CONSOLE
 
 #include <stdio.h>
 #include <string.h>
@@ -95,34 +96,34 @@ typedef struct AF_MODE_INFO
    unsigned short BytesPerScanLine        __PACKED__;
    unsigned short BitsPerPixel            __PACKED__;
    unsigned short MaxBuffers              __PACKED__;
-   unsigned char  RedMaskSize             __PACKED__;
-   unsigned char  RedFieldPosition        __PACKED__;
-   unsigned char  GreenMaskSize           __PACKED__;
-   unsigned char  GreenFieldPosition      __PACKED__;
-   unsigned char  BlueMaskSize            __PACKED__;
-   unsigned char  BlueFieldPosition       __PACKED__;
-   unsigned char  RsvdMaskSize            __PACKED__;
-   unsigned char  RsvdFieldPosition       __PACKED__;
+   unsigned char  RedMaskSize;
+   unsigned char  RedFieldPosition;
+   unsigned char  GreenMaskSize;
+   unsigned char  GreenFieldPosition;
+   unsigned char  BlueMaskSize;
+   unsigned char  BlueFieldPosition;
+   unsigned char  RsvdMaskSize;
+   unsigned char  RsvdFieldPosition;
    unsigned short MaxBytesPerScanLine     __PACKED__;
    unsigned short MaxScanLineWidth        __PACKED__;
    unsigned short LinBytesPerScanLine     __PACKED__;
-   unsigned char  BnkMaxBuffers           __PACKED__;
-   unsigned char  LinMaxBuffers           __PACKED__;
-   unsigned char  LinRedMaskSize          __PACKED__;
-   unsigned char  LinRedFieldPosition     __PACKED__;
-   unsigned char  LinGreenMaskSize        __PACKED__;
-   unsigned char  LinGreenFieldPosition   __PACKED__;
-   unsigned char  LinBlueMaskSize         __PACKED__;
-   unsigned char  LinBlueFieldPosition    __PACKED__;
-   unsigned char  LinRsvdMaskSize         __PACKED__;
-   unsigned char  LinRsvdFieldPosition    __PACKED__;
+   unsigned char  BnkMaxBuffers;
+   unsigned char  LinMaxBuffers;
+   unsigned char  LinRedMaskSize;
+   unsigned char  LinRedFieldPosition;
+   unsigned char  LinGreenMaskSize;
+   unsigned char  LinGreenFieldPosition;
+   unsigned char  LinBlueMaskSize;
+   unsigned char  LinBlueFieldPosition;
+   unsigned char  LinRsvdMaskSize;
+   unsigned char  LinRsvdFieldPosition;
    unsigned long  MaxPixelClock           __PACKED__;
    unsigned long  VideoCapabilities       __PACKED__;
    unsigned short VideoMinXScale          __PACKED__;
    unsigned short VideoMinYScale          __PACKED__;
    unsigned short VideoMaxXScale          __PACKED__;
    unsigned short VideoMaxYScale          __PACKED__;
-   unsigned char  reserved[76]            __PACKED__;
+   unsigned char  reserved[76];
 } AF_MODE_INFO;
 
 
@@ -133,11 +134,11 @@ typedef struct AF_MODE_INFO
 
 typedef struct AF_DRIVER
 {
-   char           Signature[12]           __PACKED__;
+   char           Signature[12];
    unsigned long  Version                 __PACKED__;
    unsigned long  DriverRev               __PACKED__;
-   char           OemVendorName[80]       __PACKED__;
-   char           OemCopyright[80]        __PACKED__;
+   char           OemVendorName[80];
+   char           OemCopyright[80];
    short          *AvailableModes         __PACKED__;
    unsigned long  TotalMemory             __PACKED__;
    unsigned long  Attributes              __PACKED__;
@@ -217,18 +218,18 @@ MMAP af_linear_mem = NOMM;
 #ifdef ALLEGRO_DOS
 
 /* hooks to let the driver call BIOS routines */
-extern void _af_int86(), _af_call_rm();
+extern void _af_int86(void), _af_call_rm(void);
 
 #else
 
 /* no BIOS on this platform, my friend! */
-void nobios() { }
+void nobios(void) { }
 
 #endif
 
 
 
-void af_exit()
+void af_exit(void)
 {
    int c;
 
@@ -299,7 +300,7 @@ int load_vbeaf_driver(char *filename)
    long size;
    PACKFILE *f;
 
-   size = file_size(filename);
+   size = file_size_ex(filename);
    if (size <= 0)
       return -1;
 
@@ -322,7 +323,7 @@ int load_vbeaf_driver(char *filename)
 
 
 
-void initialise_freebeaf_extensions()
+void initialise_freebeaf_extensions(void)
 {
    typedef unsigned long (*EXT_INIT_FUNC)(AF_DRIVER *af, unsigned long id);
    EXT_INIT_FUNC ext_init;
@@ -377,7 +378,7 @@ void initialise_freebeaf_extensions()
 
 
 
-static int initialise_vbeaf()
+static int initialise_vbeaf(void)
 {
    int c;
 
@@ -469,7 +470,7 @@ void print_af_attributes(unsigned long attrib)
 
 
 
-int get_af_info()
+int get_af_info(void)
 {
    static char *possible_filenames[] =
    {
@@ -735,7 +736,8 @@ int main(int argc, char *argv[])
 {
    int c;
 
-   allegro_init();
+   if (allegro_init() != 0)
+      return 1;
 
    #ifdef SYSTEM_XWINDOWS
       if (system_driver->id == SYSTEM_XWINDOWS) {
@@ -778,7 +780,7 @@ int main(int argc, char *argv[])
 #else       /* ifdef VBE/AF cool on this platform */
 
 
-int main()
+int main(void)
 {
    allegro_init();
    allegro_message("Sorry, the AFINFO program only works on DOS or Linux\n");
@@ -789,4 +791,4 @@ int main()
 #endif
 
 
-END_OF_MAIN();
+END_OF_MAIN()
