@@ -15,6 +15,7 @@
  *      See readme.txt for copyright information.
  */
 
+#define ALLEGRO_USE_CONSOLE
 
 #include <stdio.h>
 #include <string.h>
@@ -51,18 +52,18 @@ int verbose = FALSE;
 
 typedef struct VESA_INFO 
 { 
-   unsigned char  VESASignature[4]     __PACKED__;
+   char           VESASignature[4];
    unsigned short VESAVersion          __PACKED__;
    unsigned long  OEMStringPtr         __PACKED__;
-   unsigned char  Capabilities[4]      __PACKED__;
+   unsigned char  Capabilities[4];
    unsigned long  VideoModePtr         __PACKED__; 
    unsigned short TotalMemory          __PACKED__; 
    unsigned short OemSoftwareRev       __PACKED__; 
    unsigned long  OemVendorNamePtr     __PACKED__; 
    unsigned long  OemProductNamePtr    __PACKED__; 
    unsigned long  OemProductRevPtr     __PACKED__; 
-   unsigned char  Reserved[222]        __PACKED__; 
-   unsigned char  OemData[256]         __PACKED__; 
+   unsigned char  Reserved[222]; 
+   unsigned char  OemData[256]; 
 } VESA_INFO;
 
 
@@ -70,8 +71,8 @@ typedef struct VESA_INFO
 typedef struct MODE_INFO 
 {
    unsigned short ModeAttributes       __PACKED__; 
-   unsigned char  WinAAttributes       __PACKED__; 
-   unsigned char  WinBAttributes       __PACKED__; 
+   unsigned char  WinAAttributes; 
+   unsigned char  WinBAttributes; 
    unsigned short WinGranularity       __PACKED__; 
    unsigned short WinSize              __PACKED__; 
    unsigned short WinASegment          __PACKED__; 
@@ -80,40 +81,40 @@ typedef struct MODE_INFO
    unsigned short BytesPerScanLine     __PACKED__; 
    unsigned short XResolution          __PACKED__; 
    unsigned short YResolution          __PACKED__; 
-   unsigned char  XCharSize            __PACKED__; 
-   unsigned char  YCharSize            __PACKED__; 
-   unsigned char  NumberOfPlanes       __PACKED__; 
-   unsigned char  BitsPerPixel         __PACKED__; 
-   unsigned char  NumberOfBanks        __PACKED__; 
-   unsigned char  MemoryModel          __PACKED__; 
-   unsigned char  BankSize             __PACKED__; 
-   unsigned char  NumberOfImagePages   __PACKED__;
-   unsigned char  Reserved_page        __PACKED__; 
-   unsigned char  RedMaskSize          __PACKED__; 
-   unsigned char  RedMaskPos           __PACKED__; 
-   unsigned char  GreenMaskSize        __PACKED__; 
-   unsigned char  GreenMaskPos         __PACKED__;
-   unsigned char  BlueMaskSize         __PACKED__; 
-   unsigned char  BlueMaskPos          __PACKED__; 
-   unsigned char  ReservedMaskSize     __PACKED__; 
-   unsigned char  ReservedMaskPos      __PACKED__; 
-   unsigned char  DirectColorModeInfo  __PACKED__;
+   unsigned char  XCharSize; 
+   unsigned char  YCharSize; 
+   unsigned char  NumberOfPlanes; 
+   unsigned char  BitsPerPixel; 
+   unsigned char  NumberOfBanks; 
+   unsigned char  MemoryModel; 
+   unsigned char  BankSize; 
+   unsigned char  NumberOfImagePages;
+   unsigned char  Reserved_page; 
+   unsigned char  RedMaskSize; 
+   unsigned char  RedMaskPos; 
+   unsigned char  GreenMaskSize; 
+   unsigned char  GreenMaskPos;
+   unsigned char  BlueMaskSize; 
+   unsigned char  BlueMaskPos; 
+   unsigned char  ReservedMaskSize; 
+   unsigned char  ReservedMaskPos; 
+   unsigned char  DirectColorModeInfo;
    unsigned long  PhysBasePtr          __PACKED__; 
    unsigned long  OffScreenMemOffset   __PACKED__; 
    unsigned short OffScreenMemSize     __PACKED__; 
    unsigned short LinBytesPerScanLine  __PACKED__;
-   unsigned char  BnkNumberOfPages     __PACKED__;
-   unsigned char  LinNumberOfPages     __PACKED__;
-   unsigned char  LinRedMaskSize       __PACKED__;
-   unsigned char  LinRedFieldPos       __PACKED__;
-   unsigned char  LinGreenMaskSize     __PACKED__;
-   unsigned char  LinGreenFieldPos     __PACKED__;
-   unsigned char  LinBlueMaskSize      __PACKED__;
-   unsigned char  LinBlueFieldPos      __PACKED__;
-   unsigned char  LinRsvdMaskSize      __PACKED__;
-   unsigned char  LinRsvdFieldPos      __PACKED__;
+   unsigned char  BnkNumberOfPages;
+   unsigned char  LinNumberOfPages;
+   unsigned char  LinRedMaskSize;
+   unsigned char  LinRedFieldPos;
+   unsigned char  LinGreenMaskSize;
+   unsigned char  LinGreenFieldPos;
+   unsigned char  LinBlueMaskSize;
+   unsigned char  LinBlueFieldPos;
+   unsigned char  LinRsvdMaskSize;
+   unsigned char  LinRsvdFieldPos;
    unsigned long  MaxPixelClock        __PACKED__;
-   unsigned char  Reserved[190]        __PACKED__;
+   unsigned char  Reserved[190];
 } MODE_INFO;
 
 
@@ -480,7 +481,8 @@ int main(int argc, char *argv[])
 {
    int c;
 
-   allegro_init();
+   if (allegro_init() != 0)
+      return -1;
 
    for (c=1; c<argc; c++)
       if (((argv[c][0] == '-') || (argv[c][0] == '/')) &&
@@ -520,4 +522,4 @@ int main(void)
 #endif
 
 
-END_OF_MAIN();
+END_OF_MAIN()
