@@ -16,7 +16,7 @@
  */
 
 
-#define USE_CONSOLE
+#define ALLEGRO_USE_CONSOLE
 
 #include <stdio.h>
 #include <string.h>
@@ -33,6 +33,7 @@ void datedit_startmsg(AL_CONST char *fmt, ...) { }
 void datedit_endmsg(AL_CONST char *fmt, ...) { }
 void datedit_error(AL_CONST char *fmt, ...) { }
 int datedit_ask(AL_CONST char *fmt, ...) { return 0; }
+int datedit_select(AL_CONST char *(*list_getter)(int index, int *list_size), AL_CONST char *fmt, ...) { return 0; }
 
 
 /* this program is not portable! */
@@ -46,12 +47,15 @@ int datedit_ask(AL_CONST char *fmt, ...) { return 0; }
 
 static int err = 0;
 static int truecolor = FALSE;
+static int convert_compiled_sprites = FALSE;
 
 static char prefix[80] = "";
 
 static char *infilename = NULL;
 static char *outfilename = NULL;
 static char *outfilenameheader = NULL;
+static char *dataname = NULL;
+static char default_dataname[]="data";
 static char *password = NULL;
 
 static DATAFILE *data = NULL;
@@ -71,7 +75,9 @@ static void usage(void)
    printf("Options:\n");
    printf("\t'-o outputfile.s' sets the output file (default stdout)\n");
    printf("\t'-h outputfile.h' sets the output header file (default none)\n");
-   printf("\t'-p prefix' sets the object name prefix string\n");
+   printf("\t'-p prefix' sets the object name prefix string (default none)\n");
+   printf("\t'-S' converts COMPILED_SPRITEs to BITMAPs (default abort)\n");
+   printf("\t'-n name' gives a name for the datafile (default 'data')\n");
    printf("\t'-007 password' sets the datafile password\n");
 }
 
@@ -141,9 +147,9 @@ static void output_bitmap(BITMAP *bmp, char *name, int global)
    fprintf(outfile, "\t.long %-16d# cr\n", bmp->w);
    fprintf(outfile, "\t.long %-16d# ct\n", 0);
    fprintf(outfile, "\t.long %-16d# cb\n", bmp->h);
-   fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "__linear_vtable%d\n", bpp);
-   fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "_stub_bank_switch\n");
-   fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "_stub_bank_switch\n");
+   fprintf(outfile, "\t.long %-16d# bpp\n", bpp);
+   fprintf(outfile, "\t.long %-16d# write_bank\n", 0);
+   fprintf(outfile, "\t.long %-16d# read_bank\n", 0);
    fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "%s%s_data\n", prefix, name);
    fprintf(outfile, "\t.long %-16d# bitmap_id\n", 0);
    fprintf(outfile, "\t.long %-16d# extra\n", 0);
@@ -294,22 +300,19 @@ static void output_font_mono(FONT_MONO_DATA *mf, char *name, int depth)
 
 static void output_font(FONT *f, char *name, int depth)
 {
-   if (f->vtable == font_vtable_mono)
-      output_font_mono(f->data, name, 0);
-   else
+   int color_flag = (f->vtable == font_vtable_color ? 1 : 0);
+
+   if (color_flag)
       output_font_color(f->data, name, 0);
+   else
+      output_font_mono(f->data, name, 0);
 
    fprintf(outfile, "# font\n");
    fprintf(outfile, ".globl " ALLEGRO_ASM_PREFIX "%s%s\n", prefix, name);
    fprintf(outfile, ".balign 4\n" ALLEGRO_ASM_PREFIX "%s%s:\n", prefix, name);
    fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "%s%s_data\n", prefix, name);
    fprintf(outfile, "\t.long %-16d# height\n", f->height);
-   fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "_font_vtable_");
-   if (f->vtable == font_vtable_mono)
-      fprintf(outfile, "mono\n");
-   else
-      fprintf(outfile, "color\n");
-   
+   fprintf(outfile, "\t.long %-16d# color flag\n", color_flag);
    fprintf(outfile, "\n");
 }
 
@@ -336,51 +339,6 @@ static void output_rle_sprite(RLE_SPRITE *sprite, char *name)
 
 
 
-#ifndef ALLEGRO_USE_C
-
-static void output_compiled_sprite(COMPILED_SPRITE *sprite, char *name)
-{
-   char buf[160];
-   int c;
-
-   if (sprite->color_depth != 8) {
-      fprintf(stderr, "Error: truecolor compiled sprites not supported (%s)\n", name);
-      err = 1;
-      return;
-   }
-
-   for (c=0; c<4; c++) {
-      if (sprite->proc[c].draw) {
-	 sprintf(buf, "%s_plane_%d", name, c);
-	 output_data(sprite->proc[c].draw, sprite->proc[c].len, buf, "compiled sprite code", FALSE);
-      }
-   } 
-
-   fprintf(outfile, "# compiled sprite\n.globl " ALLEGRO_ASM_PREFIX "%s%s\n", prefix, name);
-   fprintf(outfile, ".balign 4\n" ALLEGRO_ASM_PREFIX "%s%s:\n", prefix, name);
-   fprintf(outfile, "\t.short %-15d# planar\n", sprite->planar);
-   fprintf(outfile, "\t.short %-15d# color depth\n", sprite->color_depth);
-   fprintf(outfile, "\t.short %-15d# w\n", sprite->w);
-   fprintf(outfile, "\t.short %-15d# h\n", sprite->h);
-
-   for (c=0; c<4; c++) {
-      if (sprite->proc[c].draw) {
-	 fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "%s%s_plane_%d\n", prefix, name, c);
-	 fprintf(outfile, "\t.long %-16d# len\n", sprite->proc[c].len);
-      }
-      else {
-	 fprintf(outfile, "\t.long 0\n");
-	 fprintf(outfile, "\t.long %-16d# len\n", 0);
-      }
-   }
-
-   fprintf(outfile, "\n");
-}
-
-#endif      /* ifndef ALLEGRO_USE_C */
-
-
-
 static void get_object_name(char *buf, char *name, DATAFILE *dat, int root)
 {
    if (!root) {
@@ -487,14 +445,15 @@ static void output_object(DATAFILE *object, char *name)
 
       case DAT_C_SPRITE:
       case DAT_XC_SPRITE:
-	 if (outfileheader)
-	    fprintf(outfileheader, "extern COMPILED_SPRITE %s%s;\n", prefix, name);
-
-	 #ifdef ALLEGRO_USE_C
-	    output_rle_sprite((RLE_SPRITE *)object->dat, name);
-	 #else
-	    output_compiled_sprite((COMPILED_SPRITE *)object->dat, name);
-	 #endif
+	 if (convert_compiled_sprites) {
+	    object->type = DAT_BITMAP;
+	    output_object(object, name);
+	 }
+	 else {
+	    fprintf(stderr, "Error: encountered a compiled sprite (%s). Please\n"
+	                    "see documentation for more information.\n", name);
+	    err = 1;
+	 }
 	 break;
 
       case DAT_FILE:
@@ -527,7 +486,8 @@ int main(int argc, char *argv[])
    char tm[80];
    time_t now;
 
-   install_allegro(SYSTEM_NONE, &errno, atexit);
+   if (install_allegro(SYSTEM_NONE, &errno, atexit) != 0)
+      return 1;
    datedit_init();
 
    time(&now);
@@ -558,6 +518,13 @@ int main(int argc, char *argv[])
 	 }
 	 strcpy(prefix, argv[++c]);
       }
+      else if (stricmp(argv[c], "-n") == 0) {
+	 if ((dataname) || (c >= argc-1)) {
+	    usage();
+	    return 1;
+	 }
+	 dataname = argv[++c];
+      }
       else if (stricmp(argv[c], "-007") == 0) {
 	 if ((password) || (c >= argc-1)) {
 	    usage();
@@ -565,6 +532,13 @@ int main(int argc, char *argv[])
 	 }
 	 password = argv[++c];
       }
+      else if (stricmp(argv[c], "-S") == 0) {
+	 if (convert_compiled_sprites) {
+	    usage();
+	    return 1;
+	 }
+	 convert_compiled_sprites = TRUE;
+      }
       else {
 	 if ((argv[c][0] == '-') || (infilename)) {
 	    usage();
@@ -582,9 +556,12 @@ int main(int argc, char *argv[])
    if ((prefix[0]) && (prefix[strlen(prefix)-1] != '_'))
       strcat(prefix, "_");
 
+   if (!dataname)
+      dataname = default_dataname;
+
    set_color_conversion(COLORCONV_NONE);
 
-   data = datedit_load_datafile(infilename, TRUE, password);
+   data = datedit_load_datafile(infilename, FALSE, password);
    if (!data) {
       fprintf(stderr, "Error reading %s\n", infilename);
       err = 1; 
@@ -623,22 +600,26 @@ int main(int argc, char *argv[])
    if (outfilename)
       printf("Converting %s to %s...\n", infilename, outfilename);
 
-   output_datafile(data, "data", TRUE);
+   output_datafile(data, dataname, TRUE);
 
    #ifdef ALLEGRO_USE_CONSTRUCTOR
 
       fprintf(outfile, ".text\n");
       fprintf(outfile, ".balign 4\n");
-      fprintf(outfile, ALLEGRO_ASM_PREFIX "_construct_me:\n");
+      fprintf(outfile, "construct_me:\n");
       fprintf(outfile, "\tpushl %%ebp\n");
       fprintf(outfile, "\tmovl %%esp, %%ebp\n");
-      fprintf(outfile, "\tpushl $" ALLEGRO_ASM_PREFIX "%sdata\n", prefix);
+      fprintf(outfile, "\tpushl $" ALLEGRO_ASM_PREFIX "%s%s\n", prefix, dataname);
       fprintf(outfile, "\tcall " ALLEGRO_ASM_PREFIX "_construct_datafile\n");
       fprintf(outfile, "\taddl $4, %%esp\n");
       fprintf(outfile, "\tleave\n");
       fprintf(outfile, "\tret\n\n");
+      #ifdef ALLEGRO_DJGPP      
       fprintf(outfile, ".section .ctor\n");
-      fprintf(outfile, "\t.long " ALLEGRO_ASM_PREFIX "_construct_me\n");
+      #else
+      fprintf(outfile, ".section .ctors\n");
+      #endif
+      fprintf(outfile, "\t.long construct_me\n");
 
    #endif
 
@@ -689,11 +670,11 @@ int main(int argc, char *argv[])
 int main(void)
 {
    allegro_init();
-   allegro_message("Sorry, the DAT2S program only works on x86 processors\n");
+   allegro_message("Sorry, the DAT2S program only works on x86 processors.\n");
    return 1;
 }
 
 
 #endif
 
-END_OF_MAIN();
+END_OF_MAIN()
