@@ -77,8 +77,8 @@ static void grabbit(DATAFILE *dat, int box)
       y = mouse_y;
 
       if ((x >= grabber_graphic->w) || (y >= grabber_graphic->h)) {
-	 x = MID(0, x, grabber_graphic->w-1);
-	 y = MID(0, y, grabber_graphic->h-1);
+	 x = CLAMP(0, x, grabber_graphic->w-1);
+	 y = CLAMP(0, y, grabber_graphic->h-1);
 	 position_mouse(x, y);
       }
 
@@ -188,8 +188,8 @@ static void grabbit(DATAFILE *dat, int box)
 	 h = mouse_y;
 
 	 if ((w < x) || (w > grabber_graphic->w) || ( h < y) || (h > grabber_graphic->h)) {
-	    w = MID(x, w, grabber_graphic->w);
-	    h = MID(y, h, grabber_graphic->h);
+	    w = CLAMP(x, w, grabber_graphic->w);
+	    h = CLAMP(y, h, grabber_graphic->h);
 	    position_mouse(w, h);
 	 }
 
@@ -478,7 +478,8 @@ DATEDIT_MENU_INFO datgrab_grabber_menu =
    &grabber_menu,
    grabber_query,
    DATEDIT_MENU_OBJECT,
-   0
+   0,
+   "XPOS;YPOS;XSIZ;YSIZ"
 };
 
 
@@ -499,7 +500,8 @@ DATEDIT_MENU_INFO datgrab_boxgrab_menu =
    &boxgrab_menu,
    boxgrab_query,
    DATEDIT_MENU_OBJECT | DATEDIT_MENU_POPUP,
-   2  /* ctrl+B */
+   2,  /* ctrl+B */
+   "XPOS;YPOS;XSIZ;YSIZ"
 };
 
 
@@ -520,6 +522,7 @@ DATEDIT_MENU_INFO datgrab_ungrab_menu =
    &ungrab_menu,
    ungrab_query,
    DATEDIT_MENU_OBJECT | DATEDIT_MENU_POPUP,
-   0
+   0,
+   NULL
 };
 
