@@ -28,7 +28,7 @@
 /* creates a new palette object */
 static void *makenew_palette(long *size)
 {
-   RGB *pal = _al_malloc(sizeof(PALETTE));
+   RGB *pal = _AL_MALLOC(sizeof(PALETTE));
 
    memcpy(pal, default_palette, sizeof(PALETTE));
    *size = sizeof(PALETTE);
@@ -67,16 +67,16 @@ static void plot_palette(AL_CONST DATAFILE *dat, int x, int y)
 	    rectfill(screen, x+(c&15)*8, y+(c/16)*8+32, 
 		     x+(c&15)*8+7, y+(c/16)*8+39, palette_color[c]);
 	 unselect_palette();
-	 textout(screen, font, "A different palette", x+160, y+32, gui_fg_color);
-	 textout(screen, font, "is currently in use.", x+160, y+40, gui_fg_color);
-	 textout(screen, font, "To select this one,", x+160, y+48, gui_fg_color);
-	 textout(screen, font, "double-click on it", x+160, y+56, gui_fg_color);
-	 textout(screen, font, "in the item list.", x+160, y+64, gui_fg_color);
+	 textout_ex(screen, font, "A different palette", x+160, y+32, gui_fg_color, gui_bg_color);
+	 textout_ex(screen, font, "is currently in use.", x+160, y+40, gui_fg_color, gui_bg_color);
+	 textout_ex(screen, font, "To select this one,", x+160, y+48, gui_fg_color, gui_bg_color);
+	 textout_ex(screen, font, "double-click on it", x+160, y+56, gui_fg_color, gui_bg_color);
+	 textout_ex(screen, font, "in the item list.", x+160, y+64, gui_fg_color, gui_bg_color);
       }
       else {
-	 textout(screen, font, "A different palette is currently in use.", x, y+32, gui_fg_color);
-	 textout(screen, font, "To select this one, double-click on it", x, y+40, gui_fg_color);
-	 textout(screen, font, "in the item list.", x, y+48, gui_fg_color);
+	 textout_ex(screen, font, "A different palette is currently in use.", x, y+32, gui_fg_color, gui_bg_color);
+	 textout_ex(screen, font, "To select this one, double-click on it", x, y+40, gui_fg_color, gui_bg_color);
+	 textout_ex(screen, font, "in the item list.", x, y+48, gui_fg_color, gui_bg_color);
       }
    }
    else {
@@ -105,8 +105,7 @@ static int export_palette(AL_CONST DATAFILE *dat, AL_CONST char *filename)
 
    bmp = create_bitmap_ex(8, 32, 8);
    clear_bitmap(bmp);
-   text_mode(0);
-   textout(bmp, font, "PAL.", 0, 0, 255);
+   textout_ex(bmp, font, "PAL.", 0, 0, 255, 0);
 
    ret = (save_bitmap(filename, bmp, (RGB *)dat->dat) == 0);
 
@@ -117,7 +116,7 @@ static int export_palette(AL_CONST DATAFILE *dat, AL_CONST char *filename)
 
 
 /* grabs a palette from an external file */
-static void *grab_palette(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
+static DATAFILE *grab_palette(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
 {
    int oldcolordepth = _color_depth;
    RGB *pal;
@@ -126,21 +125,20 @@ static void *grab_palette(AL_CONST char *filename, long *size, int x, int y, int
    _color_depth = 8;
    set_color_conversion(COLORCONV_TOTAL);
 
-   pal = _al_malloc(sizeof(PALETTE));
+   pal = _AL_MALLOC(sizeof(PALETTE));
    bmp = load_bitmap(filename, pal);
 
    _color_depth = oldcolordepth;
    set_color_conversion(COLORCONV_NONE);
 
    if (!bmp) {
-      _al_free(pal);
+      _AL_FREE(pal);
       return NULL;
    }
 
    destroy_bitmap(bmp);
 
-   *size = sizeof(PALETTE);
-   return pal;
+   return datedit_construct(type, pal, sizeof(PALETTE), prop);
 }
 
 
@@ -198,7 +196,8 @@ DATEDIT_MENU_INFO datpal_grabber_menu =
    &grabber_menu,
    grabber_query,
    DATEDIT_MENU_OBJECT,
-   0
+   0,
+   NULL
 };
 
 
@@ -224,6 +223,7 @@ DATEDIT_GRABBER_INFO datpal_grabber =
    "bmp;lbm;pcx;tga",
    "bmp;pcx;tga",
    grab_palette,
-   export_palette
+   export_palette,
+   NULL
 };
 
