@@ -16,13 +16,12 @@
  */
 
 
-#define USE_CONSOLE
+#define ALLEGRO_USE_CONSOLE
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
-#include "allegro.h"
+#include <allegro.h>
 
 
 #define ICON_MAX  16
@@ -197,7 +196,8 @@ int main(int argc, char *argv[])
    DATAFILE *dat;
    PACKFILE *f;
 
-   install_allegro(SYSTEM_NONE, &errno, atexit);
+   if (install_allegro(SYSTEM_NONE, &errno, atexit) != 0)
+       exit(EXIT_FAILURE);
    set_color_conversion(COLORCONV_NONE);
 
    if (argc < 3)
